Package: RobinCar
Type: Package
Title: Robust Inference for Covariate Adjustment in Randomized Clinical
        Trials
Version: 1.2.0
Authors@R: c(
    person("Marlena", "Bannick", 
        email="marlena.bannick@outlook.com", 
        role=c("cre", "aut"),
        comment=c(ORCID="0000-0001-6279-5978")),
    person("Yuhan", "Qian", role="aut"),
    person("Ting", "Ye", role="aut"),
    person("Yanyao", "Yi", role="aut"),
    person("Faith", "Bian", role="aut")
    )
Description: Performs robust estimation and inference when using covariate adjustment and/or covariate-adaptive randomization in randomized clinical trials.
    Ting Ye, Jun Shao, Yanyao Yi, Qinyuan Zhao (2023) <doi:10.1080/01621459.2022.2049278>.
    Ting Ye, Marlena Bannick, Yanyao Yi, Jun Shao (2023) <doi:10.1080/24754269.2023.2205802>.
    Ting Ye, Jun Shao, Yanyao Yi (2023) <doi:10.1093/biomet/asad045>.
    Marlena Bannick, Jun Shao, Jingyi Liu, Yu Du, Yanyao Yi, Ting Ye (2024) <doi:10.1093/biomet/asaf029>.
    Xiaoyu Qiu, Yuhan Qian, Jaehwan Yi, Jinqiu Wang, Yu Du, Yanyao Yi, Ting Ye (2025) <doi:10.48550/arXiv.2408.12541>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: dplyr, magrittr, tidyr, emulator, numDeriv, tidyverse, stats,
        rlang, survival, fastDummies, data.table, broom, SuperLearner,
        AIPW, MASS, Rdpack (>= 0.7)
Depends: R (>= 2.10)
Suggests: knitr, rmarkdown, ranger, forcats, matrixcalc, testthat (>=
        3.0.0)
RdMacros: Rdpack
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-24 20:08:36 UTC; marlena
Author: Marlena Bannick [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-6279-5978>),
  Yuhan Qian [aut],
  Ting Ye [aut],
  Yanyao Yi [aut],
  Faith Bian [aut]
Maintainer: Marlena Bannick <marlena.bannick@outlook.com>
Repository: CRAN
Date/Publication: 2026-04-24 20:20:16 UTC
Built: R 4.6.0; ; 2026-04-30 21:26:17 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(adjust,CovHR)
S3method(adjust,CoxScore)
S3method(adjust,GLMModel)
S3method(adjust,LogRank)
S3method(adjust,MHModel)
S3method(adjust,SLModel)
S3method(descript,GLMModelResult)
S3method(jacobian,CUSTOM)
S3method(jacobian,DIFF)
S3method(jacobian,LOGODDSRATIO)
S3method(jacobian,LOGRATIO)
S3method(jacobian,ODDSRATIO)
S3method(jacobian,RATIO)
S3method(print,CalibrationResult)
S3method(print,ContrastResult)
S3method(print,GLMModelResult)
S3method(print,LinModelResult)
S3method(print,MHResult)
S3method(print,TTEResult)
S3method(validate,RoboDataGLM)
S3method(validate,RoboDataLinear)
S3method(validate,RoboDataMH)
S3method(validate,RoboDataSL)
S3method(validate,RoboDataTTE)
export("%>%")
export(car_pb)
export(car_ps)
export(car_sr)
export(data_gen)
export(data_gen2)
export(robincar_SL)
export(robincar_SL_median)
export(robincar_calibrate)
export(robincar_contrast)
export(robincar_covhr)
export(robincar_coxscore)
export(robincar_glm)
export(robincar_linear)
export(robincar_logrank)
export(robincar_mh)
export(robincar_tte)
import(AIPW)
import(SuperLearner)
import(broom)
import(data.table)
import(fastDummies)
import(tidyverse)
importFrom(MASS,glm.nb)
importFrom(Rdpack,reprompt)
importFrom(SuperLearner,listWrappers)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(dplyr,arrange)
importFrom(dplyr,cur_group_id)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_all)
importFrom(dplyr,group_modify)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_all)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(emulator,quad.tform)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
importFrom(stats,cov)
importFrom(stats,model.matrix)
importFrom(stats,rbinom)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(utils,combn)

[![R-CMD-check](https://github.com/mbannick/RobinCar/actions/workflows/R-CMD-check.yaml/badge.svg?branch=main)](https://github.com/mbannick/RobinCar/actions/workflows/R-CMD-check.yaml) ![CRAN](https://www.r-pkg.org/badges/version/RobinCar) ![downloads](https://cranlogs.r-pkg.org/badges/grand-total/RobinCar) ![downloads](https://cranlogs.r-pkg.org/badges/RobinCar) [![DOI](https://zenodo.org/badge/506080289.svg)](https://zenodo.org/badge/latestdoi/506080289)


# RobinCar: ROBust estimation and INference for Covariate Adjustment in Randomized clinical trials

RobinCar is a package that allows for robust estimation and inference for treatment effects in randomized clinical trials when covariates are used at the design and/or analysis stages of the trial. Supported covariate-adaptive randomization schemes at the design phase are simple randomization, stratified permuted block randomization, biased coin randomization, and Pocock and Simon's minimization. Statistical methods at the analysis stage are model-assisted and assumption-lean, in accordance with [FDA guidance on covariate adjustment](https://www.fda.gov/regulatory-information/search-fda-guidance-documents/adjusting-covariates-randomized-clinical-trials-drugs-and-biological-products). Publications describing the methods are listed [here](#publications).

See also [RobinCar2](https://openpharma.github.io/RobinCar2/main/), which is a lite version of RobinCar and is supported by the [ASA Biopharmaceutical Section Covariate Adjustment Scientific Working Group Software Subteam](https://carswg.github.io/subteam_software.html).

## Authors
[Ting Ye](https://sites.google.com/view/tingye), Yanyao Yi, [Marlena Bannick](https://marlenabannick.com/) (maintainer), Yuhan Qian, and Faith Bian

## Documentation

To view documentation about the functions, see the RobinCar website here: https://marlenabannick.com/RobinCar/. You will also find vignettes about how to use the functions.

## Installation

RobinCar is now available on CRAN!

### 1. Install via CRAN
![CRAN](https://www.r-pkg.org/badges/version-last-release/RobinCar)

```{}
install.packages("RobinCar")
```

### 2. Install with `devtools`

To get the most recent version in development, you can install the package with `devtools`:
```{bash}
devtools::install_github("mbannick/RobinCar")
```

### 3. Clone repository

Or to download the package, you may clone the repository:
```{bash}
git clone https://github.com/mbannick/RobinCar.git
```

## <a id="publications"></a>Publications

Here are publications and preprints that explain the methods in RobinCar:

* [Ting Ye, Jun Shao, Yanyao Yi (2023)](https://doi.org/10.1093/biomet/asad045)
* [Marlena Bannick, Jun Shao, Jingyi Liu, Yu Du, Yanyao Yi, Ting Ye (2025)](https://doi.org/10.1093/biomet/asaf029)
* [Ting Ye, Marlena Bannick, Yanyao Yi, Jun Shao (2023)](https://doi.org/10.1080/24754269.2023.2205802)
* [Ting Ye, Jun Shao, Yanyao Yi, Qinyuan Zhao (2023)](https://doi.org/10.1080/01621459.2022.2049278)
* [Xiaoyu Qiu, Yuhan Qian, Jaehwan Yi, Jinqiu Wang, Yu Du, Yanyao Yi, Ting Ye (2025)](https://arxiv.org/pdf/2408.12541)

# Perform GLM adjustment, based on the classes
# of the model. Will perform adjustment based on the linear
# model type of `model` and also do G-computation or AIPW
# based on the second model type of `model`.
#' @exportS3Method
adjust.SLModel <- function(model, data, ...){

  dmat <- get.dmat(data, model$adj_vars)

  SL.libraries <- model$SL_libraries
  for(learner in model$SL_learners){
    SL.libraries <- c(SL.libraries, learner$names)
  }

  # TODO: Need to have option for multiple treatment groups
  # TODO: This fails if there are not enough people in each
  #       treatment group... how to fix?
  aipw <- AIPW$new(
    Y=data$response,
    A=as.integer(data$treat) - 1,
    W.Q=dmat,
    W.g=rep(1, data$n),
    Q.SL.library=SL.libraries,
    g.SL.library="SL.mean",
    k_split=model$k_split,
    verbose=T,
    save.sl.fit=T
  )

  # TODO: Need option for whether or not to perform a stratified fit
  # maybe in the adj_method option for homogeneous or heterogeneous?
  aipw$stratified_fit()
  muhat <- do.call(cbind, aipw$obs_est[1:data$k])
  adjustment <- get.mutilde(model, data, muhat)
  mutilde <- adjustment$mutilde
  estimate <- colMeans(mutilde)

  # This is just a placeholder for the data frame, it's not
  # actually using this GLM model fit
  df <- data.frame(response=data$response, treat=data$treat, dmat)
  glmod <- stats::glm(response ~ ., data=df)

  # Compute the asymptotic variance -- not specific to a partition
  asympt.variance <- vcov_car(model, data, glmod, mutilde)
  vcov_wt <- get.vcovHC(model$vcovHC, n=data$n, p=NULL)
  variance <- asympt.variance * vcov_wt / data$n

  result <- format_results(data$treat_levels, estimate, variance)

  return(
    structure(
      class="SLModelResult",
      list(result=result, varcov=variance, settings=model,
           data=data, mod=aipw, mu_a=mutilde)
    )
  )
}

# Gets the vcovHC weights for the sample size and number of parameters
get.vcovHC <- function(vcovHC, n, p){
  if(vcovHC == "HC0"){
    wgt <- 1
  } else if(vcovHC == "HC1") {
    wgt <- n / (n - p)
  } else if(vcovHC == "HC3") {
    wgt <- (n / (n - p)) ** 2
  } else {
    stop("Unrecognized vcovHC method.")
  }
  return(wgt)
}

# Get design matrix for the specified adjustment variables
# using the data stored.
get.dmat <- function(data, adj_vars){
  if(is.null(adj_vars)){
    dmat <- NULL
  } else if(adj_vars == "x"){
    dmat <- data$covariate
  } else if(adj_vars == "z"){
    dmat <- data$car_strata
  } else if(adj_vars == "joint_z"){
    dmat <- as.matrix(data$joint_strata, ncol=1)
    colnames(dmat) <- "joint_strata"
  } else if(adj_vars == "joint_z_x"){
    joint_strata <- data$joint_strata
    dmat <- cbind(data$covariate, joint_strata)
  } else if(adj_vars == "formula"){
    dmat <- data$formula_vars
  } else {
    stop(paste("Unrecognized adjustment variable type ", adj_vars))
  }
  return(dmat)
}

#' @importFrom stats model.matrix
.center.dmat <- function(dmat){
  modmat <- stats::model.matrix(~ 0 + ., data=data.frame(dmat))
  modmat <- t(t(modmat) - colMeans(modmat))
  return(modmat)
}

get.mutilde <- function(model, data, muhat){
  # Get response
  y <- data$response

  # Get treatment group indicators and outcome
  t_ids <- sapply(data$treat_levels, function(x) data$treat == x)

  if(!model$pu_joint_z){
    # Only look within treatment groups
    center_ids <- as.list(data.frame(t_ids))
    center_mus <- as.list(data.frame(muhat))
  } else {

    # Will create matrix for combination of car_strata and treatment groups
    sl <- data$joint_strata_levels
    tl <- data$treat_levels

    # Get indicators for joint car_strata group
    s_ids <- sapply(sl, function(x) data$joint_strata == x)

    # Get joint levels of treatment and car_strata groups
    center_ids <- as.list(data.frame(
      t_ids[, rep(1:ncol(t_ids), each=length(sl))] &
        s_ids[, rep(1:ncol(s_ids), times=length(tl))]
    ))

    # Repeat the mu columns for each car_strata
    center_mus <- as.list(data.frame(muhat[, rep(1:ncol(muhat), each=length(sl))]))
  }

  # Compute AIPW estimator by re-centering predictions
  # within treatment groups
  recenter <- function(u, i){
    cent <- sum(u[i])/sum(i) - sum(y[i])/sum(i)
    return(u - cent)
  }
  mutilde <- mapply(FUN=recenter,
                    u=center_mus,
                    i=center_ids)

  if(model$pu_joint_z){
    new_mutilde <- matrix(data=NA, nrow=data$n, ncol=ncol(muhat))
    for(s_col in 1:ncol(s_ids)){
      scol_seq <- seq(s_col, ncol(mutilde), by=length(sl))
      new_mutilde[s_ids[, s_col], ] <- mutilde[s_ids[, s_col], scol_seq]
    }
    mutilde <- new_mutilde
  }

  # Check prediction un-biasedness for the original muhat
  # g-computation just for warning/error reporting,
  # only up to level of accuracy specified by the user
  check.pu <- function(u, i) round(mean(u[i] - y[i]),
                                   digits=model$g_accuracy)

  # Calculate the group-specific residuals
  # where the groups are defined as treatment or treatment x car_strata above
  resid <- mapply(
    FUN=check.pu,
    u=center_mus,
    i=center_ids
  )

  dfree <- 0
  # Report warning or error messages, whatever
  # is passed through the model settings, if not prediction unbiased.
  if(!all(resid == 0)){
    if(!is.list(model$pu_funcs)){
      funcs <- list(model$pu_funcs)
    } else {
      funcs <- model$pu_funcs
    }
    for(func in funcs){
      func()
    }
    if(model$pu_joint_z){
      # Adjust the degrees of freedom for the model if prediction unbiasedness does not hold
      # and we end up having to adjust within levels of z
      # TODO: This is not going to work when there are covariates in the model...
      # Might need to do a more conservative adjustment
      # dfree <- (dfree - data$k) + data$k * length(data$joint_strata_levels)
      # dfree <- data$k * length(data$joint_strata_levels)
    }
  }

  return(list(mutilde=mutilde, df_adjust=dfree))
}

# Defines allowable arguments to the functions available to users

.check.options <- function(name, var, options){
  if(!var %in% options) stop(paste0(name, " must be one of: ",
                                    paste0(options, collapse = ", ")))
}

.check.car_scheme <- function(car_scheme, car_strata_cols){
  OPTIONS <- c("simple",
               "permuted-block",
               "pocock-simon",
               "biased-coin",
               "urn")
  .check.options("car_scheme", car_scheme, OPTIONS)

  # Check to make sure that existence of
  # strata matches with the car scheme
  if(!is.null(car_strata_cols)){
    if(car_scheme == "simple") .z.exist.warn()
  } else {
    if(car_scheme != "simple") .z.miss.err()
  }
}

.check.variance_type <- function(variance_type){
  OPTIONS <- c(1, 2, 3)
  .check.options("variance_type", variance_type, OPTIONS)
}

.check.adj_method.logrank <- function(adj_method){
  OPTIONS <- c("CL",
               "CSL")
  .check.options("adj_method", adj_method, OPTIONS)
}

.check.adj_method.glm <- function(adj_method){
  OPTIONS <- c("heterogeneous",
               "homogeneous")
  .check.options("adj_method", adj_method, OPTIONS)
}

.check.adj_method.linear <- function(adj_method){
  OPTIONS <- c("ANOVA",
               "ANCOVA",
               "ANHECOVA")
  .check.options("adj_method", adj_method, OPTIONS)
}

.check.vcovHC <- function(vcovHC){
  OPTIONS <- c("HC0",
               "HC1",
               "HC3")
  .check.options("vcovHC", vcovHC, OPTIONS)
}

# .check.sl.libraries <- function(SL_libraries){
#   libs <- invisible(SuperLearner::listWrappers(what="SL"))
#   OPTIONS <- libs[grepl("^SL.", libs)]
#   # for(lib in SL_libraries){
#   #   .check.options("Super Learner libraries", lib, OPTIONS)
#   # }
# }

.check.estimand_ci <- function(estimand,ci_type){
  OPTIONS.estimand <- c("MH",
                        "ATE")
  .check.options("estimand", estimand, OPTIONS.estimand)
  
  OPTIONS.ci_type <- c("GR", 
                       "Sato", 
                       "mGR")
  .check.options("ci_type", ci_type, OPTIONS.ci_type)
  
  if(estimand=="ATE" & ci_type!="mGR"){
    stop(paste0('Using ', ci_type, ' variance estimator is invalid when estimating ATE.\n Please use modified GR estimator and set ci_type = "mGR".'))
  }
}

#' @importFrom stats weighted.mean
estimate.mh <- function(df.mh) {
  delta <- df.mh$y1.k - df.mh$y0.k
  weight <- df.mh$n.1k * df.mh$n.0k / (df.mh$n.1k + df.mh$n.0k)
  delta_all <- weighted.mean(delta, weight)
  
  return(delta_all)
}

var.mh.GR <- function(n11k, n10k, n.1k, n.0k) {
  weight <- n.1k * n.0k / (n.1k + n.0k)
  var.k <- (n11k * (n.1k - n11k) * n.0k^3 + n10k * (n.0k - n10k) * n.1k^3) / n.1k / n.0k / (n.1k + n.0k)^2
  var.est.GR <- sum(var.k[weight != 0]) / (sum(weight)^2)
  return(var.est.GR)
}



var.mh.mGR <- function(n11k, n10k, n.1k, n.0k){
  help_f <- Vectorize(function(x) {
    if (x > 1) {
      return(x / (x - 1))
    } else {
      1
    }
  })
  
  n01k <- n.1k - n11k
  n00k <- n.0k - n10k
  weight <- n.1k * n.0k / (n.1k + n.0k)
  var.k <- weight^2 * (n11k * n01k / n.1k^3 * help_f(n.1k) + n10k * n00k /
                         n.0k^3 * help_f(n.0k))
  var.est.mGR <- sum(var.k[weight != 0]) / (sum(weight)^2)
  return(var.est.mGR)
}


var.mh.Sato <- function(n11k, n10k, n.1k, n.0k){
  weight <- n.1k * n.0k / (n.1k + n.0k)
  Pk <- (n.1k^2 * n10k - n.0k^2 * n11k + n.1k * n.0k * (n.0k - n.1k) / 2) / (n.1k + n.0k)^2
  Qk <- (n11k * (n.0k - n10k) + n10k * (n.1k - n11k)) / 2 / (n.1k  + n.0k)
  delta <- n11k / n.1k - n10k / n.0k
  delta_all <- weighted.mean(delta, weight)
  var.est.Sato <- (delta_all * sum(Pk) + sum(Qk)) / (sum(weight)^2)
  return(var.est.Sato)
}

var.ate.nu <- function(df, ATE.est){
  n.1k <- df$n.1k
  n.0k <- df$n.0k
  n11k <- df$n11k
  n10k <- df$n10k
  y1.k <- df$y1.k
  y0.k <- df$y0.k
  y1.var.k <- df$y1.var.k
  y0.var.k <- df$y0.var.k
  
  K <- length(n11k)
  n.k <- n.1k + n.0k
  n <- sum(n.k)
  pi1 <- sum(n.1k) / n
  pi0 <- sum(n.0k) / n
  rho.hat <- n.k/n
  delta.k <- n11k / n.1k - n10k / n.0k
  weights <- n.1k*n.0k / (n.1k + n.0k)
  
  p1k <- n11k / n.1k
  p0k <- n10k / n.0k
  
  p1k.sq <- p1k^2 - ifelse(n.1k>1,y1.var.k,0) / n.1k
  p0k.sq <- p0k^2 - ifelse(n.0k>1,y0.var.k,0) / n.0k
  
  delta.k.sq <- p1k.sq - 2*p1k * p0k + p0k.sq
  tmp1 <- rho.hat * (delta.k.sq - ATE.est^2)
  tmp2 <- (delta.k.sq - 2 * delta.k * ATE.est + ATE.est^2) * pi1 * pi0 *
    (n.k - 1) / n.k * (n.k - 1 - (4 * n.k - 6) * pi1 * pi0) / n
  var.est.nu <- sum(tmp1[weights!=0])*pi1^2*pi0^2 + sum(tmp2[weights!=0])
  
  var.est.nu <- var.est.nu / n / (sum(weights)/n)^2
  
  return(var.est.nu)
}

est.var.mh <- function(df.mh, estimand, ci_type){
  mh_est <- estimate.mh(df.mh)
  var_est <- with(df.mh,
                  switch(ci_type,
                         "GR"    = var.mh.GR(n11k, n10k, n.1k, n.0k),
                         "mGR"   = var.mh.mGR(n11k, n10k, n.1k, n.0k) + (estimand=="ATE") * var.ate.nu(df.mh, mh_est),
                         "Sato"  = var.mh.Sato(n11k, n10k, n.1k, n.0k),
                         stop("Invalid ci_type. Choose from 'GR', 'mGR', or 'Sato'."))
  )
  return(list(estimate = mh_est, var_est = var_est))
}

# Estimate the MH risk difference or ATE
# contrast is incorporated in adjust.MHModel as we don't have estimate for each treatment.
#' @importFrom utils combn
#' @exportS3Method
adjust.MHModel <- function(model, data, ...){
  
  estimand <- model$estimand
  ci_type <- model$ci_type
  
  treat_col <- data$treat_col
  response_col <- data$response_col
  strata <- data$joint_strata
  
  treat_levels <- data$treat_levels
  n_treat <- length(treat_levels)
  
  strata_list <- split(data$df, strata)
  
  n.k <- matrix(0, nrow = length(strata_list), ncol = n_treat)  # Total in each group
  n1.k <- matrix(0, nrow = length(strata_list), ncol = n_treat) # Events in each group
  y.var.k <- matrix(0, nrow = length(strata_list), ncol = n_treat) # Events in each group
  
  for (i in seq_along(strata_list)) {
    df_k <- strata_list[[i]]
    
    for (j in 1:n_treat) {
      t_level <- treat_levels[j]
      n.k[i, j] <- sum(df_k[[treat_col]] == t_level)   # Total subjects in treatment j
      n1.k[i, j] <- sum(df_k[[response_col]][df_k[[treat_col]] == t_level])  # Events in treatment j
      y.var.k[i, j] <- var(df_k[[response_col]][df_k[[treat_col]] == t_level])
    }
  }
  
  risk_diffs <- matrix(0, nrow = n_treat, ncol = n_treat)
  var_mh <- matrix(0, nrow = n_treat, ncol = n_treat)
  
  # Reserved for multiple treatment levels
  for (j in 1:(n_treat-1)) {
    for (k in (j+1):n_treat) {
        df_mh <- data.frame(
          n.1k = n.k[, j],
          n.0k = n.k[, k],
          n11k = n1.k[, j],
          n10k = n1.k[, k],
          y1.k = n1.k[, j] / n.k[, j],
          y0.k = n1.k[, k] / n.k[, k],
          y1.var.k = y.var.k[, j],
          y0.var.k = y.var.k[, k]
        )
        
        # Compute MH estimate and variance
        mh_result <- est.var.mh(df_mh, estimand = estimand, ci_type = ci_type)
        
        risk_diffs[j, k] <- - mh_result$estimate
        var_mh[j, k] <- mh_result$var_est
      
    }
  }
  
  name <- function(tl) combn(tl, 2, function(x) paste0("treat ", x[2], " - ", x[1]))
  
  lab <- name(treat_levels)
    
  c_est <- risk_diffs[upper.tri(risk_diffs)]
  
  vcv_ncol <- (n_treat^2 - n_treat)/2
  vcv <- diag(x = var_mh[upper.tri(var_mh)], nrow = vcv_ncol, ncol = vcv_ncol)
  result <- format_results(lab, c_est, vcv, label_name = "contrast")
  
  return(structure(class = "MHResult", list(result = result, settings = model)))
}

#' @exportS3Method
descript.GLMModelResult <- function(x, ...){
  output <- c()
  if("AIPW" %in% class(x$settings)){
    etype <- "aipw-type"
  } else {
    etype <- "g-computation-type"
  }

  if(is.character(x$settings$g_family)){
    if(x$settings$g_family == "nb"){
      family <- list(
        family="negative binomial with unknown dispersion",
        link="log"
      )
    } else {
      family <- get(x$settings$g_family)()
    }
  } else if(is.function(x$settings$g_family)){
    family <- (x$settings$g_family)()
  } else {
    family <- x$settings$g_family
  }

  if(!is.null(x$data$formula)){

    if(is.character(x$settings$g_family)){
      if(x$settings$g_family == "nb"){
        form <- x$data$formula
      } else {
        form <- x$mod$formula
      }
    } else {
      form <- x$mod$formula
    }

    output <- c(
      output,
      sprintf("Treatment group mean estimates from a GLM working model of family %s and link %s using formula: \n",
              family$family, family$link),
      form
    )
  } else {
    output <- c(
      output,
      sprintf("Treatment group mean estimates using a %s estimator",
              etype),
      sprintf("\nfrom a GLM working model of family %s and link %s",
              family$family, family$link)
    )
    k <- length(x$data$treat_levels)
    mat <- get.dmat(x$data, x$settings$adj_vars)
    name <- colnames(mat)
    cov_name <- name[name != "joint_strata"]
    if("joint_strata" %in% name){
      cov_name <- c(
        cov_name,
        sprintf("joint levels of %s",
                paste0(names(x$data$car_strata), collapse=", "))
      )
    }
    if(!is.null(x$settings$adj_vars)){
      output <- c(
        output,
        sprintf("\nusing adjustment variables: %s",
                paste0(cov_name, collapse=", "))
      )
      if("ANHECOVA" %in% class(x$settings)){
        output <- c(
          output,
          "\nand their interactions with treatment."
        )
      } else if("ANCOVA" %in% class(x$settings)){
        output <- c(output, ".")
      } else {
        stop("Error with the type of model.")
      }
    }
  }

  output <- c(
    output,
    sprintf("\n\nUsed %s-type of heteroskedasticity-consistent variance estimates ",
            x$settings$vcovHC)
  )
  if(!is.null(x$settings$omegaz_func)){
    if(all(x$settings$omegaz_func(x$data$pie) == 0)){
      car_strata <- colnames(x$data$car_strata)
      output <- c(
        output,
        sprintf("\nand adjusted variance-covariance matrix for randomization car_strata %s \nconsistent with the %s design.",
              paste0(car_strata, collapse=", "),
              x$settings$car_scheme)
      )
    }
  }
  for(o in output){
    if("formula" %in% class(o)){
      print(o, showEnv=FALSE)
    } else {
      cat(o)
    }
  }
}

#' Covariate adjustment for time to event data
#'
#' Perform a covariate-adjusted logrank test (`adj_method="CL"`),
#' covariate-adjusted stratified logrank test (`adj_method="CSL"`),
#' or a covariate-adjusted robust Cox score test (`adj_method="coxscore"`).
#'
#' `robincar_coxscore` and `robincar_logrank` are wrapper functions around
#' `robincar_tte`.
#'
#' @param df A data.frame with the required columns
#' @param treat_col Name of column in df with treatment variable
#' @param response_col Name of the column in df with response variable
#' @param event_col Name of column in df with event indicator
#'                  (0/FALSE=no event, 1/TRUE=event)
#' @param car_strata_cols Names of columns in df with car_strata variables
#' @param covariate_cols Names of columns in df with covariate variables
#' @param car_scheme Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".
#' @param ref_arm Reference arm of the treatment group, defaults to NULL,
#'                which results in using the first element of `unique(data[, treat_col])`.
#' @param p_trt Treatment allocation ratio for the treatment arm.
#' @param adj_method Adjustment method (one of "CL", "CSL", or "coxscore")
#' @param sparse_remove Remove sparse car_strata from calculation
#' @param return_influence Return the influence function for the covariate-adjusted log-rank test for each individual
#' @param id_col Name of the column in df indicating individual ids
#'
#' @export
#'
#' @returns For adjustment method "CL" or "CSL", see value of \link[RobinCar:robincar_linear]{RobinCar::robincar_logrank()}; for adjustment method "coxscore" see value of \link[RobinCar:robincar_coxscore]{RobinCar::robincar_coxscore()}.
robincar_tte <- function(df,
                         treat_col, response_col, event_col,
                         adj_method,
                         car_strata_cols=NULL, covariate_cols=NULL,
                         p_trt=0.5, ref_arm=NULL, sparse_remove=TRUE,
                         car_scheme="simple", return_influence=FALSE, id_col=NULL){

  .check.car_scheme(car_scheme, car_strata_cols)
  data <- .make.data(
    df=df,
    classname="RoboDataTTE",
    treat_col=treat_col,
    response_col=response_col,
    event_col=event_col,
    car_strata_cols=car_strata_cols,
    covariate_cols=covariate_cols,
    id_col=id_col
  )
  validate(data, ref_arm)

  # Create model object
  model <- .make.model(
    data=data,
    adj_method=adj_method,
    car_scheme=car_scheme,
    p_trt=p_trt,
    ref_arm=ref_arm,
    sparse_remove=sparse_remove,
    return_influence=return_influence
  )

  # Perform adjustment
  result <- adjust(model, data, return_influence)
  result$original_df <- df

  return(result)

}

# These are the covariate randomization schemes
# that are applicable; leaves out simple randomization
# and pocock-simon minimization because those are dealt with separately
APPLICABLE.SCHEMES <- c("permuted-block",
                        "biased-coin",
                        "urn")

glmlogic <- function(z_exists, car_scheme){

  # PREDICTION UNBIASEDNESS -------------------------
  # Whether to check prediction un-biasedness in all joint levels
  # of Z, or just overall, and whether to report an error or warning
  pu_joint_z <- FALSE

  # Biased action will be one of "", "warning", "error"
  pu_funcs <- function(){}

  if(z_exists & car_scheme == "pocock-simon"){

    pu_funcs <- c(.pu.z.calibrate, .pu.z.warn)
    pu_joint_z <- TRUE

  } else {

    pu_funcs <- .pu.warn
    pu_joint_z <- FALSE

  }

  # CAR OMEGA FUNCTION -----------------------------
  # Get the Omega Z function
  omegaz_func <- NULL

  if(z_exists & (car_scheme != "simple")){
    omegaz_func <- omegaz.closure(car_scheme)
  }

  return(list(
    method=c("GLMModel"),
    pu_joint_z=pu_joint_z,
    pu_funcs=pu_funcs,
    omegaz_func=omegaz_func
  ))
}

Robin_g <- function(robin.data = data.tmp,
                    car.scheme = "PB", # SR
                    car.z = c("z1", "z2"), # joint levels of 'z' will be used under CAR
                    robin.x = c("x1"),
                    robin.x_to_include_z = TRUE, # default
                    robin.adj_method = c("heterogeneous", "homogeneous"), # option name may be revised, homo ~ ANCOVA-type, hetero ~ ANHECOVA-type
                    robin.formula = as.formula("y ~ z1 + z2 + x1 + A + x1*A"), # if robin.formula is specified, robin.x, robin.x_to_include_z and robin.adj_method are ignored, robin.formula does not work for "minimization"
                    robin.g_family = binomial(link = "logit"), # usually only family is needed, i.e., default canonical link will be applied, similar to glm
                    robin.g_accuracy = 7,
                    robin.vcovHC = c("HC0", "HC1", "HC3")){

  library(tidyverse)

  robin.n <- nrow(robin.data)
  robin.trt_pie <- robin.data %>% group_by(A) %>% summarise(pie_A = n()/robin.n, .groups = "drop")

  # generate joint_z for CAR
  if(car.scheme == "SR"){
    car.z = NULL
    robin.x_to_include_z = FALSE
    robin.g_data <- robin.data
  }else{ # car.scheme == "PB"
    if(is.null(car.z)){ # under CAR, car.z cannot be NULL
      stop("under CAR, please specify 'car.z'.")
    }else{ # car.z is specified and car.scheme is not SR, generate joint levels of car.z
      robin.g_data <- robin.data %>% group_by(across(all_of(car.z))) %>%
        mutate(joint_z = factor(cur_group_id())) %>% ungroup()
    }
  }

  # customized formula or not?
  if(class(robin.formula) == "formula"){
    robin.x <- NULL
    robin.x_to_include_z <- F
    robin.adj_method <- NULL

  }else{

    if(car.scheme == "minimization"){
      # if robin.formula is NOT used, joint_z should at least be included for covariate adjustment under minimization
      if(!robin.x_to_include_z){
        robin.x_to_include_z <- TRUE
        warning("robin.x_to_include_z is set to be 'TRUE'! ")
      }
    }

    if(robin.x_to_include_z){
      robin.x <- c(robin.x, "joint_z")
      robin.g_data <- robin.g_data %>% select(all_of(c("y", "A", car.z, robin.x)))
    }else{
      if(car.scheme == "SR"){
        robin.g_data <- robin.g_data %>% select(all_of(c("y", "A", robin.x)))
      }else{ # car.scheme == "applicable CAR"
        robin.g_data <- robin.g_data %>% select(all_of(c("y", "A", car.z, robin.x, "joint_z")))
      }
    }

  }



  # make sure robin.x does not have elements starting with "g_pred_"
  if(sum(str_detect(colnames(robin.g_data), "g_pred"))>0){
    stop("Please make sure robin.data does not include column name starting with 'g_pred'.")
  }

  # fit glm and get g_pred_k(mu_k_hat)
  if(class(robin.formula) == "formula"){ # this part is not that mature

    robin.g_fit <- glm(as.formula(robin.formula), family = robin.g_family, data = robin.g_data)
    robin.x <- setdiff(as.character(attributes(robin.g_fit$terms)$predvars), c('list', 'y', 'A'))
    robin.g_pred_data <- do.call(cbind.data.frame,
                                 lapply(lapply(robin.trt_pie$A, function(x){cbind.data.frame(A=x, robin.g_data %>% select(all_of(robin.x)))}),
                                        function(x){predict(robin.g_fit, newdata = x, type = "response")}))
    colnames(robin.g_pred_data) <- paste0("g_pred_", robin.trt_pie$A)
    robin.g_data <- cbind.data.frame(robin.g_data, robin.g_pred_data)

  }else if(robin.adj_method == "homogeneous"){

    robin.g_fit <- glm(y ~ A+., family = robin.g_family, data = robin.g_data %>% select(all_of(c("y", "A", robin.x))))
    robin.g_pred_data <- do.call(cbind.data.frame,
                                 lapply(lapply(robin.trt_pie$A, function(x){cbind.data.frame(A=x, robin.g_data %>% select(all_of(robin.x)))}),
                                        function(x){predict(robin.g_fit, newdata = x, type = "response")}))
    colnames(robin.g_pred_data) <- paste0("g_pred_", robin.trt_pie$A)
    robin.g_data <- cbind.data.frame(robin.g_data, robin.g_pred_data)

  }else if(robin.adj_method == "heterogeneous"){

    robin.g_fit <- robin.g_data %>% group_by(A) %>%
      do(g_fit = glm(y ~ ., family = robin.g_family, data = .data %>% select(all_of(c("y", robin.x)))))
    robin.g_pred_data <- do.call(cbind.data.frame,
                                 lapply(robin.g_fit$g_fit,
                                        function(x){predict(x, newdata = robin.g_data %>% select(all_of(robin.x)), type = "response")}))
    names(robin.g_pred_data) <- paste0("g_pred_", robin.g_fit$A)
    robin.g_data <- cbind.data.frame(robin.g_data, robin.g_pred_data)

  }

  # - check predictive unbiasedness
  if(car.scheme == "minimization"){ # check predictive unbiasedness over joint_z
    if(
      all(
        diag(
          robin.g_data %>%
          select(all_of(c("y", "A", "joint_z", paste0("g_pred_", robin.trt_pie$A)))) %>%
          group_by(A, joint_z) %>%
          summarise(across(.cols = everything(), .fns = ~ round(mean(.x), digits = robin.g_accuracy)), .groups = "drop") %>%
          group_by(A) %>%
          summarise(across(.cols = paste0("g_pred_", robin.trt_pie$A), .fns = ~ sum(.x == y)), .groups = "drop") %>%
          select(all_of(paste0("g_pred_", robin.trt_pie$A))) %>%
          as.matrix()
        ) == length(levels(robin.g_data$joint_z))
      )){
      robin.predunbiased <- T
    }else{
      stop("minimization requires estimator to be predictive unbiased across every car_strata of joint z!")
    }

  }else{ # car.scheme != "minimization", i.e., car.scheme %in% SR or commonly used CAR
    # check predictive unbiasedness over all subjects
    if(
      all(
        diag(
          robin.g_data %>%
          select(all_of(c("y", "A", paste0("g_pred_", robin.trt_pie$A)))) %>%
          group_by(A) %>%
          summarise(across(.cols = everything(), .fns = ~ round(mean(.x), digits = robin.g_accuracy)), .groups = "drop") %>%
          group_by(A) %>%
          summarise(across(.cols = paste0("g_pred_", robin.trt_pie$A), .fns = ~ sum(.x == y)), .groups = "drop") %>%
          select(all_of(paste0("g_pred_", robin.trt_pie$A))) %>%
          as.matrix()
        ) == 1)){
      robin.predunbiased <- T
    }else{
      robin.predunbiased <- F
    }
  }

  # - AIPW estimator: g-estimator = AIPW if predictive unbiasedness holds
  robin.g_pred_center <- diag(
    robin.g_data %>%
      group_by(A) %>%
      summarise(across(.cols = paste0("g_pred_", robin.trt_pie$A), .fns = ~ mean(y - .x)), .groups = "drop") %>%
      mutate(A = paste0("g_pred_", A)) %>%
      column_to_rownames("A") %>%
      as.matrix()
  )
  robin.g_pred_center <- as.list(robin.g_pred_center)

  # centering "g_pred_k", if predictive unbiasedness holds, automatically centering by zero
  robin.g_data <- robin.g_data %>% mutate(across(all_of(names(robin.g_pred_center)), ~ .x + robin.g_pred_center[[cur_column()]]))

  robin.est <- apply(robin.g_data %>% select(all_of(paste0("g_pred_", robin.trt_pie$A))), 2, mean)

  if(! robin.predunbiased){ # can be placed in later place.
    warning("The g-estimator does not satisfy predictive unbiasedness, AIPW is reported.")
  }

  # - variance-covariance matrix for AIPW estimator
  robin.varcov.term1 <- robin.g_data %>%
    select(all_of(c("y", "A", paste0("g_pred_", robin.trt_pie$A)))) %>%
    group_by(A) %>%
    summarise(across(.cols = all_of(paste0("g_pred_", robin.trt_pie$A)), .fns = ~ var(y - .x)), .groups = "drop") %>%
    left_join(., robin.trt_pie, by = "A") %>%
    group_by(A) %>%
    summarise(across(.cols = all_of(paste0("g_pred_", robin.trt_pie$A)), .fns = ~ .x/pie_A), .groups = "drop") %>%
    mutate(A = paste0("g_pred_", A)) %>%
    column_to_rownames("A") %>%
    as.matrix()
  robin.varcov.term1 <- diag(diag(robin.varcov.term1))
  colnames(robin.varcov.term1) <- paste0("g_pred_", robin.trt_pie$A)
  rownames(robin.varcov.term1) <- paste0("g_pred_", robin.trt_pie$A)


  robin.varcov.term2 <- robin.g_data %>%
    select(all_of(c("y", "A", paste0("g_pred_", robin.trt_pie$A)))) %>%
    group_by(A) %>%
    summarise(across(.cols = all_of(paste0("g_pred_", robin.trt_pie$A)), .fns = ~ cov(y, .x)), .groups = "drop") %>%
    mutate(A = paste0("g_pred_", A)) %>%
    column_to_rownames("A") %>%
    as.matrix()

  robin.varcov.term3 <- t(robin.varcov.term2)

  robin.varcov.term4 <- robin.g_data %>%
    select(all_of(paste0("g_pred_", robin.trt_pie$A)))
  robin.varcov.term4 <- var(robin.varcov.term4)

  #- robin.varcov.term5 is determined by design
  if(car.scheme %in% c("SR", "minimization")){
    robin.varcov.term5 <- 0
    # by definition, SR > robin.varcov.term5 to be zero
    # by previous check, minimization requires predictive unbiasedness across all car_strata of joint_z

  }else if(car.scheme %in% c("PB", "stratified_biased_coin")){
    robin.varcov.term5.omiga.SR <- diag(robin.trt_pie$pie_A) - robin.trt_pie$pie_A %*% t(robin.trt_pie$pie_A)
    rownames(robin.varcov.term5.omiga.SR) <- paste0("g_pred_", robin.trt_pie$A)
    colnames(robin.varcov.term5.omiga.SR) <- paste0("g_pred_", robin.trt_pie$A)
    robin.varcov.term5.omiga.CAR_by_Z <- diag(0, nrow(robin.varcov.term5.omiga.SR))
    rownames(robin.varcov.term5.omiga.CAR_by_Z) <- paste0("g_pred_", robin.trt_pie$A)
    colnames(robin.varcov.term5.omiga.CAR_by_Z) <- paste0("g_pred_", robin.trt_pie$A)

    robin.varcov.term5.RZ.tmp <- robin.g_data %>%
      select(all_of(c("y", "A", "joint_z", paste0("g_pred_", robin.trt_pie$A)))) %>%
      left_join(., robin.trt_pie, by = "A") %>%
      mutate(across(.cols = paste0("g_pred_", robin.trt_pie$A), .fns = ~ (y - .x)/pie_A)) %>%
      group_by(A, joint_z) %>%
      summarise(p_joint_z = n()/robin.n,
                across(.cols = paste0("g_pred_", robin.trt_pie$A), .fns = ~ mean(.x)),
                .groups = "drop") %>%
      group_by(joint_z) %>%
      group_split()

    robin.varcov.term5.p_joint_z <-
      do.call(
        rbind.data.frame,
        lapply(robin.varcov.term5.RZ.tmp,
               function(x){x %>% group_by(joint_z) %>% summarise(p_joint_z = sum(p_joint_z))})
      )

    robin.varcov.term5 <-
      lapply(robin.varcov.term5.RZ.tmp,
             function(x){
               xI <- x %>% select(A, all_of(paste0("g_pred_", robin.trt_pie$A))) %>%
                 mutate(A = paste0("g_pred_", A)) %>%
                 column_to_rownames("A") %>%
                 as.matrix() # the diagonal terms of xI is robin.varcov.term5.RZ, which is diag(diag(xI))
               xII <- diag(diag(xI)) %*% (robin.varcov.term5.omiga.SR - robin.varcov.term5.omiga.CAR_by_Z) %*% diag(diag(xI))
               colnames(xII) <- paste0("g_pred_", robin.trt_pie$A)
               xII <- cbind.data.frame(joint_z = x$joint_z, as.data.frame(xII)) %>%
                 left_join(., robin.varcov.term5.p_joint_z, by = "joint_z") %>%
                 mutate(across(.cols = paste0("g_pred_", robin.trt_pie$A), .fns = ~ .x*p_joint_z)) %>%
                 select(all_of(paste0("g_pred_", robin.trt_pie$A)))
               return(xII)}
      )

    robin.varcov.term5 <- Reduce("+", robin.varcov.term5) %>% as.matrix() # cautious: do not use "reduce", capital "R" matters

  }else if(car.scheme %in% c("stratified urn design")){ # has not been implemented
    # robin.varcov.term5.omiga.SR <- diag(robin.trt_pie$pie_A) - robin.trt_pie$pie_A %*% t(robin.trt_pie$pie_A)
    # robin.varcov.term5.omiga.CAR_by_Z
  }else{
    stop("There is no such design") # this actually should be checked at the beginning.
  }

  #- put all robin.varcov.terms together
  robin.varcov <- robin.varcov.term1 + robin.varcov.term2 + robin.varcov.term3 - robin.varcov.term4 - robin.varcov.term5

  #--- adjust by HC1 or HC3
  if(length(robin.vcovHC)>1){
    robin.vcovHC.wgt <- 1
  }else if(robin.vcovHC=="HC0"){
    robin.vcovHC.wgt <- 1
  }else if(robin.vcovHC=="HC1"){
    robin.vcovHC.wgt <- 1 # not implemented yet
  }else if(robin.vcovHC=="HC3"){
    robin.vcovHC.wgt <- 1 # not implemented yet
  }else{
    stop("There is no such vcovHC method!")
  }

  robin.varcov <- robin.varcov*robin.vcovHC.wgt/robin.n

  # - summaries return
  robin.est <- data.frame(
    A = sub(".*g_pred_", "", names(robin.est)), # str_extract anything after "g_pred_"
    estimate = robin.est,
    se = sqrt(diag(robin.varcov))
  ) %>%
    mutate(`pval (2-sided)` = 2*pnorm(abs(estimate/se), lower.tail = F))


  robin.return <- list(estimation = robin.est,
                       varcov = robin.varcov,
                       n = robin.n,
                       robin.g_object = list(robin.g_data = robin.g_data,
                                             car.scheme = car.scheme,
                                             car.z = car.z))

  return(robin.return)
}

# Compare to legacy code from Yanyao Yi

test_that("GLM legacy", {

  n <- 10000
  set.seed(10)
  DATA2 <- data.frame(A=rbinom(n, size=1, prob=0.5),
                      y=rbinom(n, size=1, prob=0.2),
                      x1=rnorm(n),
                      x2=rnorm(n),
                      x3=as.factor(rbinom(n, size=1, prob=0.5)),
                      z1=rbinom(n, size=1, prob=0.5),
                      z2=rbinom(n, size=1, prob=0.5))
  DATA2$A <- as.factor(DATA2$A)
  DATA2$x1 <- DATA2$x1 - mean(DATA2$x1)

  for(cov in list(c("x1"), NULL)){
    for(ctis in list(TRUE, FALSE)){
      for(zs in list(c("z1", "z2"), c("z1"), NULL)){
        for(meth in c("homogeneous", "heterogeneous")){
          for(scheme in c("SR",
                          "PB",
                          # "minimization",
                          # TODO: Fix these tests to be in line with new Z logic for pocock simon!
                          "stratified_biased_coin")){

            if(scheme == "SR"){
              scheme0 <- "simple"
            } else if(scheme == "PB"){
              scheme0 <- "permuted-block"
            } else if(scheme == "minimization"){
              scheme0 <- "pocock-simon"
            } else if(scheme == "stratified_biased_coin"){
              scheme0 <- "biased-coin"
            }

            if(scheme != "SR" & is.null(zs)) next
            # We can't compare the behavior in this case
            # because old code automatically includes Z when heterogeneous + pocock simon
            if(scheme == "minimization" & !ctis & meth == "heterogeneous") next

            if(scheme != "SR"){
              if(ctis){
                # Create joint levels of z for formula
                zs_this <- paste0(zs, collapse="*")
                covariate_cols <- c(cov, zs_this)
              } else {
                covariate_cols <- cov
              }
            } else {
              covariate_cols <- cov
            }
            # browser()

            if(meth == "homogeneous"){
              if(is.null(covariate_cols)){
                form <- .create.formula(
                  "ANOVA", "y", "A", covariate_cols
                )
              } else {
                form <- .create.formula(
                  "ANCOVA", "y", "A", covariate_cols
                )
              }
            } else {
              if(is.null(covariate_cols)){
                next
              } else {
                form <- .create.formula(
                  "ANHECOVA", "y", "A", covariate_cols
                )
              }
            }

            runthis <- function(){
              return(robincar_glm(
                df=DATA2,
                response_col="y",
                treat_col="A",
                car_strata_cols=zs,
                car_scheme=scheme0,
                g_family=binomial(link="logit"),
                g_accuracy=7,
                formula=form
              ))
            }

            runthat <- function(){
              return(Robin_g(
                robin.data=DATA2,
                car.scheme=scheme,
                car.z=zs,
                robin.g_family=binomial(link="logit"),
                robin.g_accuracy=7,
                robin.vcovHC="HC0",
                robin.formula=formula(form)
              ))
            }
            if(scheme == "minimization" & meth == "homogeneous"){
              # expect_warning(runthis())
              # expect_error(runthat())
            } else {
              this <- runthis()
              that <- runthat()
              this_est <- this$result$estimate
              this_se <- this$result$se
              names(this_est) <- NULL
              names(this_se) <- NULL
              expect_equal(this_est, that$estimation$estimate)
              # NEW: variance will no longer be the same now that we have done
              # the finite sample correction for the variance estimation
              # when there are lots of 0's in the response vector
              # expect_equal(this_se, that$estimation$se)
            }
          }
        }
      }
    }
  }
})

DATA <- RobinCar:::data_sim
DATA$A <- as.factor(DATA$A)
DATA$y <- ifelse(DATA$y > 2, 1, 0)

test_that("test calibration", {
  this <- suppressWarnings(robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    car_scheme="biased-coin",
    g_family=binomial(link="logit"),
    g_accuracy=7,
    formula="y ~ A + x3"))

  that <- Robin_g(
    robin.data=DATA,
    car.scheme="stratified_biased_coin",
    car.z=c("z1"),
    robin.x=c("x3"),
    robin.x_to_include_z=FALSE,
    robin.g_family=binomial(link="logit"),
    robin.g_accuracy=7,
    robin.formula=NULL,
    robin.adj_method="homogeneous"
  )

  for(joint in c(FALSE, TRUE)){
    for(add_x in c("x3", NULL)){

      calib_this <- suppressWarnings(robincar_calibrate(
        result=this,
        joint=joint,
        add_x=add_x
      ))
      calib_that <- Robin_calibrate(
        robin.g_object=that$robin.g_object,
        robin.add_x=add_x,
        robin.add_x_to_include_z=joint,
        robin.vcovHC="HC0"
      )
      calib_this_est <- calib_this$result$estimate
      calib_this_se <- calib_this$result$se

      calib_this_vc <- calib_this$varcov
      dimnames(calib_this_vc) <- NULL
      calib_that_vc <- calib_that$varcov
      dimnames(calib_that_vc) <- NULL

      names(calib_this_est) <- NULL
      names(calib_this_se) <- NULL
      expect_equal(calib_this_est, calib_that$estimation$estimate)
      # NEW: variance will no longer be the same now that we have done
      # the finite sample correction for the variance estimation
      # when there are lots of 0's in the response vector
      # expect_equal(calib_this_se, calib_that$estimation$se)
      # expect_equal(calib_this_vc, calib_that_vc)
    }
  }
})

###### Author: Ting Ye ######
###### Last modified: Jan 16, 2022 ######
# Use null variance #
data.sort<-function(data.simu){
  n<-dim(data.simu)[1]
  data.simu.order<- data.simu[order(data.simu$t),]
  data.rev<- data.frame(data.simu.order, Y=n:1, Y1=cumsum(data.simu.order$I1[n:1])[n:1], Y0=cumsum(data.simu.order$I0[n:1])[n:1])
  return(data.rev)
}

wlogrank<-function(data.simu){
  n<-dim(data.simu)[1]
  data.rev<-data.sort(data.simu)
  T.seq<-data.rev$t
  T.rep<-c(0,T.seq)[1:n]
  T.diff<-T.seq-T.rep
  same.ind<-which(T.diff==0)
  n_col<-dim(data.rev)[2]
  for(ind in same.ind){
    data.rev[ind,(n_col-2):n_col]<-data.rev[ind-1,(n_col-2):n_col]
  }
  S.alt<-sum(data.rev$delta*(data.rev$I1*data.rev$Y0-data.rev$I0*data.rev$Y1)/data.rev$Y)
  var.alt<-sum(data.rev$delta*data.rev$Y0*data.rev$Y1/data.rev$Y^2)
  res<-S.alt/sqrt(var.alt)
  return(list(S.alt=S.alt, res=res, var.alt=var.alt))
}

ind_to_factor<-function(data.simu){
  z<-data.simu[,grepl("car_strata",names(data.simu))]
  n_strata<-dim(z)[2]
  n<-dim(z)[1]
  fz<-numeric(n)
  for(i in 1:n_strata){
    fz<-fz+z[,i]*i
  }
  return(as.factor(fz))
}

wlogrank_var_calibrated<-function(data.simu,randomization,p_trt){ # try new version 19 May
  n<-dim(data.simu)[1]
  S.alt<-wlogrank(data.simu)$S.alt
  data.rev<-data.sort(data.simu)
  T.seq<-data.rev$t
  T.rep<-c(0,T.seq)[1:n]
  T.diff<-T.seq-T.rep
  same.ind<-which(T.diff==0)
  n_col<-dim(data.rev)[2]
  for(ind in same.ind){
    data.rev[ind,(n_col-2):n_col]<-data.rev[ind-1,(n_col-2):n_col]
  }

  strata_z<-data.rev[,grepl("car_strata",names(data.rev))]
  mean_at_risk<-data.rev$delta/data.rev$Y
  cumsum_at_risk<-cumsum(mean_at_risk)
  mean_at_risk_2<-data.rev$delta*data.rev$Y1/(data.rev$Y)^2
  cumsum_at_risk_2<-cumsum(mean_at_risk_2)
  mu_t<-data.rev$Y1/data.rev$Y
  O_i<-data.rev$delta*(data.rev$I1-data.rev$Y1/data.rev$Y)-data.rev$I1*cumsum_at_risk+cumsum_at_risk_2
  O_i1<-data.rev$I1*O_i
  O_i0<- -data.rev$I0*O_i
  strata_z<-data.rev[,grepl("car_strata",names(data.rev))]
  n_strata<-dim(strata_z)[2]
  z<-strata_z
  cond_var_1<-numeric(n_strata)
  cond_var_0<-numeric(n_strata)
  prob_z<-numeric(n_strata)
  cond_exp_1<-numeric(n_strata)
  cond_exp_0<-numeric(n_strata)
  for(i in 1:n_strata){
    cond_var_1[i]<-var(O_i1[z[,i]==1 & data.rev$I1==1])
    cond_var_0[i]<-var(O_i0[z[,i]==1 & data.rev$I0==1])
    prob_z[i]<-mean(z[,i])
    cond_exp_1[i]<-mean(O_i1[z[,i]==1 & data.rev$I1==1])
    cond_exp_0[i]<-mean(O_i0[z[,i]==1 & data.rev$I0==1])
  }
  na.ind<-is.na(cond_var_1+cond_var_0)
  if(length(which(na.ind))!=0) warning("conditional variance in log-rank equals 0")
  cond_var_1<-cond_var_1[!na.ind]
  cond_var_0<-cond_var_0[!na.ind]
  prob_z<-prob_z[!na.ind]
  cond_exp_1<-cond_exp_1[!na.ind]
  cond_exp_0<-cond_exp_0[!na.ind]
  sai_1<-(p_trt*(cond_var_1%*%prob_z)+(1-p_trt)*(cond_var_0%*%prob_z))
  sai_2<-0 # for cabc and permuted_block
  if(randomization=="SR"){
    sai_2<-p_trt*(1-p_trt)*sum((cond_exp_0+cond_exp_1)^2 * prob_z)
  }
  if(randomization =="urn"){
    if(p_trt!=1/2)  stop("For now, urn design is only designed for pi=1/2")
    sai_2<-1/12*sum((cond_exp_0+cond_exp_1)^2 * prob_z)
  }
  var_cal<-n*(sai_1+sai_2)
  T_cal<-S.alt/sqrt(var_cal)
  return(list(S.alt=S.alt, res=T_cal,var_cal=var_cal))
}

# # proposed T_RS in Ye and Shao, 2020, jrssb.
# score_robust<-function(data.simu,randomization,p_trt){
#   data.rev<-data.sort(data.simu)
#   x<-data.rev[,grepl("model",names(data.rev))] #only those has model
#   x<-cbind(x,data.rev[,grepl("car_strata",names(data.rev))][,-1])
#   n<-dim(data.rev)[1]
#   data_sub<-data.frame(t=data.rev$t,delta=data.rev$delta,x)
#   fit<-survival::coxph(survival::Surv(t,delta)~.,data=data_sub)
#   S_0_seq<-exp(fit$coefficients %*% t(x))
#   S_1_seq<-exp(fit$coefficients %*% t(x))*data.rev$I1
#   S_0<-cumsum(S_0_seq[n:1])[n:1]/n
#   S_1<-cumsum(S_1_seq[n:1])[n:1]/n
#   U<-sum(data.rev$delta*(data.rev$I1-S_1/S_0))
#   #mu_t<-data.rev$Y1/data.rev$Y # two choices, both valid
#   mu_t<-S_1/S_0
#   mean_at_risk<-data.rev$delta/(n*S_0)
#   cumsum_at_risk<-cumsum(mean_at_risk)
#   mean_at_risk_2<-data.rev$delta/(n*S_0)*mu_t
#   cumsum_at_risk_2<-cumsum(mean_at_risk_2)
#   O_i<-data.rev$delta*(data.rev$I1-mu_t)-S_1_seq*cumsum_at_risk+S_0_seq*cumsum_at_risk_2
#
#   O_i1<-data.rev$I1*O_i
#   O_i0<- -data.rev$I0*O_i
#
#   strata_z<-data.rev[,grepl("car_strata",names(data.rev))]
#   n_strata<-dim(strata_z)[2]
#   z<-strata_z
#   cond_var_1<-numeric(n_strata)
#   cond_var_0<-numeric(n_strata)
#   prob_z<-numeric(n_strata)
#   cond_exp_1<-numeric(n_strata)
#   cond_exp_0<-numeric(n_strata)
#   for(i in 1:n_strata){
#     cond_var_1[i]<-var(O_i1[z[,i]==1 & data.rev$I1==1])
#     cond_var_0[i]<-var(O_i0[z[,i]==1 & data.rev$I0==1])
#     prob_z[i]<-mean(z[,i])
#     cond_exp_1[i]<-mean(O_i1[z[,i]==1 & data.rev$I1==1])
#     cond_exp_0[i]<-mean(O_i0[z[,i]==1 & data.rev$I0==1])
#   }
#   na.ind<-is.na(cond_var_1+cond_var_0)
#   if(length(which(na.ind))!=0) warning("conditional variance in log-rank equals 0")
#   cond_var_1<-cond_var_1[!na.ind]
#   cond_var_0<-cond_var_0[!na.ind]
#   prob_z<-prob_z[!na.ind]
#   cond_exp_1<-cond_exp_1[!na.ind]
#   cond_exp_0<-cond_exp_0[!na.ind]
#   sai_1<-(p_trt*(cond_var_1%*%prob_z)+(1-p_trt)*(cond_var_0%*%prob_z))
#   sai_2<-0 # for cabc and permuted_block
#   if(randomization=="SR"){
#     sai_2<-p_trt*(1-p_trt)*sum((cond_exp_0+cond_exp_1)^2 * prob_z)
#   }
#   if(randomization =="urn"){
#     if(p_trt!=1/2)  stop("For now, urn design is only designed for pi=1/2")
#     sai_2<-1/12*sum((cond_exp_0+cond_exp_1)^2 * prob_z)
#   }
#   var_cal<-n*(sai_1+sai_2)
#   T_SR<-U/sqrt(var_cal)
#   return(T_SR)
# }


# proposed T_RS in Ye and Shao, 2020, jrssb.
score_robust<-function(data.simu,randomization,p_trt){
  data.rev<-data.sort(data.simu)
  x<-data.rev[,grepl("model",names(data.rev))] #only those has model
  n<-dim(data.rev)[1]
  data_sub<-data.frame(t=data.rev$t,delta=data.rev$delta,x)
  fit<-survival::coxph(survival::Surv(t,delta)~.,data=data_sub)
  S_0_seq<-exp(fit$coefficients %*% t(x))
  S_1_seq<-exp(fit$coefficients %*% t(x))*data.rev$I1
  S_0<-cumsum(S_0_seq[n:1])[n:1]/n
  S_1<-cumsum(S_1_seq[n:1])[n:1]/n
  U<-sum(data.rev$delta*(data.rev$I1-S_1/S_0))
  #mu_t<-data.rev$Y1/data.rev$Y # two choices, both valid
  mu_t<-S_1/S_0
  mean_at_risk<-data.rev$delta/(n*S_0)
  cumsum_at_risk<-cumsum(mean_at_risk)
  mean_at_risk_2<-data.rev$delta/(n*S_0)*mu_t
  cumsum_at_risk_2<-cumsum(mean_at_risk_2)
  O_i<-data.rev$delta*(data.rev$I1-mu_t)-S_1_seq*cumsum_at_risk+S_0_seq*cumsum_at_risk_2

  O_i1<-data.rev$I1*O_i
  O_i0<- -data.rev$I0*O_i

  strata_z<-data.rev[,grepl("car_strata",names(data.rev))]
  n_strata<-dim(strata_z)[2]
  z<-strata_z
  cond_var_1<-numeric(n_strata)
  cond_var_0<-numeric(n_strata)
  prob_z<-numeric(n_strata)
  cond_exp_1<-numeric(n_strata)
  cond_exp_0<-numeric(n_strata)
  for(i in 1:n_strata){
    cond_var_1[i]<-var(O_i1[z[,i]==1 & data.rev$I1==1])
    cond_var_0[i]<-var(O_i0[z[,i]==1 & data.rev$I0==1])
    prob_z[i]<-mean(z[,i])
    cond_exp_1[i]<-mean(O_i1[z[,i]==1 & data.rev$I1==1])
    cond_exp_0[i]<-mean(O_i0[z[,i]==1 & data.rev$I0==1])
  }
  na.ind<-is.na(cond_var_1+cond_var_0)
  if(length(which(na.ind))!=0) warning("conditional variance in log-rank equals 0")
  cond_var_1<-cond_var_1[!na.ind]
  cond_var_0<-cond_var_0[!na.ind]
  prob_z<-prob_z[!na.ind]
  cond_exp_1<-cond_exp_1[!na.ind]
  cond_exp_0<-cond_exp_0[!na.ind]
  sai_1<-(p_trt*(cond_var_1%*%prob_z)+(1-p_trt)*(cond_var_0%*%prob_z))
  sai_2<-0 # for cabc and permuted_block
  if(randomization=="SR"){
    # if(p_trt!=1/2)  stop("For now, SR is only designed for pi=1/2")
    # sai_2<-1/4*sum((cond_exp_0+cond_exp_1)^2 * prob_z)
    sai_2<-p_trt*(1-p_trt)*sum((cond_exp_0+cond_exp_1)^2 * prob_z)
  }
  # if(randomization =="urn"){
  #   if(p_trt!=1/2)  stop("For now, urn design is only designed for pi=1/2")
  #   sai_2<-1/12*sum((cond_exp_0+cond_exp_1)^2 * prob_z)
  # }
  var_cal<-n*(sai_1+sai_2)
  T_SR<-U/sqrt(var_cal)
  return(list(T_SR=T_SR,
              se=sqrt(var_cal),
              U=U))
}


# score test T_S in Ye and Shao, 2020, jrssb.
score<-function(data.simu){
  data.rev<-data.sort(data.simu)
  x<-data.rev[,grepl("model",names(data.rev))] #only those has model
  x<-cbind(x,data.rev[,grepl("car_strata",names(data.rev))][,-1])
  n<-dim(data.rev)[1]
  data_sub<-data.frame(t=data.rev$t,delta=data.rev$delta,x)
  fit<-survival::coxph(survival::Surv(t,delta)~.,data=data_sub)
  S_0_seq<-exp(fit$coefficients %*% t(x))
  S_1_seq<-exp(fit$coefficients %*% t(x))*data.rev$I1
  S_0<-cumsum(S_0_seq[n:1])[n:1]/n
  S_1<-cumsum(S_1_seq[n:1])[n:1]/n
  U<-sum(data.rev$delta*(data.rev$I1-S_1/S_0))
  #mu_t<-data.rev$Y1/data.rev$Y # two choices, both valid
  mu_t<-S_1/S_0
  mean_at_risk<-data.rev$delta/(n*S_0)
  cumsum_at_risk<-cumsum(mean_at_risk)
  mean_at_risk_2<-data.rev$delta/(n*S_0)*mu_t
  cumsum_at_risk_2<-cumsum(mean_at_risk_2)
  O_i<-data.rev$delta*(data.rev$I1-mu_t)-S_1_seq*cumsum_at_risk+S_0_seq*cumsum_at_risk_2
  var_cal<-mean(O_i^2)*n
  T_WR<-U/sqrt(var_cal)
  return(T_WR)
}

# numerator of T_S
score_numerator<-function(data.simu){
  data.rev<-data.sort(data.simu)
  x<-data.rev[,grepl("model",names(data.rev))] #only those has model
  x<-cbind(x,data.rev[,grepl("car_strata",names(data.rev))][,-1])
  n<-dim(data.rev)[1]
  data_sub<-data.frame(t=data.rev$t,delta=data.rev$delta,x)
  fit<-survival::coxph(survival::Surv(t,delta)~.,data=data_sub)
  S_0_seq<-exp(fit$coefficients %*% t(x))
  S_1_seq<-exp(fit$coefficients %*% t(x))*data.rev$I1
  S_0<-cumsum(S_0_seq[n:1])[n:1]/n
  S_1<-cumsum(S_1_seq[n:1])[n:1]/n
  U<-sum(data.rev$delta*(data.rev$I1-S_1/S_0))
  return(U)
}

#########################
#   Added Sep 5, 2021   #
#########################

# Using log-rank score as the derived outcome (Tangen-Koch, 1999)
logrank_TK_score<-function(data.simu,p_trt,randomization){
  n<-dim(data.simu)[1]
  data.simu$fz<-ind_to_factor(data.simu)
  data.rev<-data.sort(data.simu)
  T.seq<-data.rev$t
  T.rep<-c(0,T.seq)[1:n]
  T.diff<-T.seq-T.rep
  same.ind<-which(T.diff==0)
  n_col<-dim(data.rev)[2]
  for(ind in same.ind){
    data.rev[ind,(n_col-2):n_col]<-data.rev[ind-1,(n_col-2):n_col]
  }
  n<-dim(data.rev)[1]
  data.rev$R<-with(data.rev,delta-cumsum(data.rev$delta/data.rev$Y))

  x.model<-as.matrix(data.rev[,grepl("model",names(data.rev)),drop=FALSE]) #only those has model
  x.mat<-model.matrix(~factor(fz)+x.model,data=data.rev)[,-1]
  x.centered<-sweep(x.mat, 2, colMeans(x.mat))
  fit1<-lm(R~factor(fz)+x.model[data.rev$I1==1,],data=data.rev[data.rev$I1==1,])
  fit0<-lm(R~factor(fz)+x.model[data.rev$I1==0,],data=data.rev[data.rev$I1==0,])
  b1<-fit1$coefficients[-1]
  b0<-fit0$coefficients[-1]
  # calculating beta0 and beta1 for variance estimation
  mu_t<-data.rev$Y1/data.rev$Y
  data.rev$O.hat<-data.rev$delta*(data.rev$I1-data.rev$Y1/data.rev$Y)-
    data.rev$I1*cumsum(data.rev$delta/data.rev$Y)+cumsum(data.rev$delta*data.rev$Y1/(data.rev$Y)^2)
  data.rev$O.hat[data.rev$I0==1]<- (-1)*data.rev$O.hat[data.rev$I0==1]
  fit1.Ohat<-lm(O.hat~factor(fz)+x.model[data.rev$I1==1,],data=data.rev[data.rev$I1==1,])
  fit0.Ohat<-lm(O.hat~factor(fz)+x.model[data.rev$I1==0,],data=data.rev[data.rev$I1==0,])
  beta1.Ohat<-fit1.Ohat$coefficients[-1]
  beta0.Ohat<-fit0.Ohat$coefficients[-1]

  ind.na<-which(is.na(b1))
  if(length(ind.na)==0){
    U_TK<-sum(data.rev$I1*(data.rev$R-t((x.centered)%*% b1 )) -
                data.rev$I0*(data.rev$R-t((x.centered)%*% b0)))/n/2
  }else{
    warning("Removing model variables that are linearly dependent with the stratification variables.")
    b1<-b1[-ind.na]
    b0<-b0[-ind.na]
    x.mat<-x.mat[,-ind.na]
    x.centered<-x.centered[,-ind.na]
    beta1.Ohat<-beta1.Ohat[-ind.na]
    beta0.Ohat<-beta0.Ohat[-ind.na]
    U_TK<-sum(data.rev$I1*(data.rev$R-t((x.centered)%*% b1 )) -
                data.rev$I0*(data.rev$R-t((x.centered)%*% b0)))/n/2
  }
  var_CL.null<-(sum(data.rev$delta*data.rev$Y0*data.rev$Y1/data.rev$Y^2)/n-
                  p_trt*(1-p_trt)*(beta1.Ohat+beta0.Ohat) %*% var(x.mat) %*% (beta1.Ohat+beta0.Ohat) )/n
  var_TK.null<-var_CL.null+p_trt*(1-p_trt)*(beta1.Ohat-b1/2+beta0.Ohat-b0/2)%*%var(x.mat)%*%(beta1.Ohat-b1/2+beta0.Ohat-b0/2)/n
  if(randomization%in% c("permuted_block","SR","CABC")){
    se<-sqrt(var_TK.null)
  }else{
    se<-NA
  }
  return(list(U_TK=U_TK,se=se))
}


# covariate adjusted logrank proposed in Ye, Yi, Shao (2022)
#'@title Ting Ye's original code
#'@export
covariate_adjusted_logrank<-function(data.simu,p_trt){
  n<-dim(data.simu)[1]
  data.simu$fz<-ind_to_factor(data.simu)
  data.rev<-data.sort(data.simu)
  T.seq<-data.rev$t
  T.rep<-c(0,T.seq)[1:n]
  T.diff<-T.seq-T.rep
  same.ind<-which(T.diff==0)
  n_col<-dim(data.rev)[2]
  for(ind in same.ind){
    data.rev[ind,(n_col-2):n_col]<-data.rev[ind-1,(n_col-2):n_col]
  }
  n<-dim(data.rev)[1]

  x.model<-as.matrix(data.rev[,grepl("model",names(data.rev)),drop=FALSE]) #only those has model
  x.mat<-model.matrix(~factor(fz)+x.model,data=data.rev)[,-1]
  x.centered<-sweep(x.mat, 2, colMeans(x.mat))
  # calculating beta0 and beta1 for variance estimation
  mu_t<-data.rev$Y1/data.rev$Y
  data.rev$O.hat<-data.rev$delta*(data.rev$I1-data.rev$Y1/data.rev$Y)-
    data.rev$I1*cumsum(data.rev$delta/data.rev$Y)+cumsum(data.rev$delta*data.rev$Y1/(data.rev$Y)^2)
  data.rev$O.hat[data.rev$I0==1]<- (-1)*data.rev$O.hat[data.rev$I0==1]
  fit1.Ohat<-lm(O.hat~factor(fz)+x.model[data.rev$I1==1,],data=data.rev[data.rev$I1==1,])
  fit0.Ohat<-lm(O.hat~factor(fz)+x.model[data.rev$I1==0,],data=data.rev[data.rev$I1==0,])
  beta1.Ohat<-fit1.Ohat$coefficients[-1]
  beta0.Ohat<-fit0.Ohat$coefficients[-1]

  ind.na<-which(is.na(beta1.Ohat))
  if(length(ind.na)==0){
    U_CL<-sum(data.rev$I1*(data.rev$O.hat-t((x.centered)%*% beta1.Ohat )) -
                data.rev$I0*(data.rev$O.hat-t((x.centered)%*% beta0.Ohat)))/n
  }else{
    warning("Removing model variables that are linearly dependent with the stratification variables.")
    x.mat<-x.mat[,-ind.na]
    x.centered<-x.centered[,-ind.na]
    beta1.Ohat<-beta1.Ohat[-ind.na]
    beta0.Ohat<-beta0.Ohat[-ind.na]
    U_CL<-sum(data.rev$I1*(data.rev$O.hat-t((x.centered)%*% beta1.Ohat )) -
                data.rev$I0*(data.rev$O.hat-t((x.centered)%*% beta0.Ohat)))/n
  }

  score_logrank_anhecova_var<-function(data.rev,p_trt,fit1.Ohat,fit0.Ohat,beta1.Ohat,beta0.Ohat){
    my.var<-(p_trt*var(fit1.Ohat$residuals)+(1-p_trt)*var(fit0.Ohat$residuals)+
               (p_trt*beta1.Ohat-(1-p_trt)*beta0.Ohat) %*% var(x.mat) %*% (p_trt*beta1.Ohat-(1-p_trt)*beta0.Ohat))/n
    my.var.null<-(sum(data.rev$delta*data.rev$Y0*data.rev$Y1/data.rev$Y^2)/n-
                    p_trt*(1-p_trt)*(beta1.Ohat+beta0.Ohat) %*% var(x.mat) %*% (beta1.Ohat+beta0.Ohat) )/n
    return(list(my.var=my.var,my.var.null=my.var.null))
  }
  tmp<-score_logrank_anhecova_var(data.rev,p_trt,fit1.Ohat,fit0.Ohat,beta1.Ohat,beta0.Ohat)
  se<-sqrt(tmp$my.var)
  se.null<-sqrt(tmp$my.var.null)
  return(list(U_CL=U_CL,se=se.null,se.orig=se))
}


# covariate adjusted stratified logrank proposed in Ye, Yi, Shao (2022)
covariate_adjusted_stratified_logrank<-function(data.simu,p_trt){
  n<-dim(data.simu)[1]
  data.simu$fz<-ind_to_factor(data.simu)
  fz.n<-unique(data.simu$fz)
  data.rev<-data.sort(data.simu[data.simu$fz==fz.n[1],])
  T.seq<-data.rev$t
  T.rep<-c(0,T.seq)[1:length(T.seq)]
  T.diff<-T.seq-T.rep
  same.ind<-which(T.diff==0)
  n_col<-dim(data.rev)[2]
  for(ind in same.ind){
    data.rev[ind,(n_col-2):n_col]<-data.rev[ind-1,(n_col-2):n_col]
  }
  mu_t<-data.rev$Y1/data.rev$Y
  data.rev$O.hat<-data.rev$delta*(data.rev$I1-data.rev$Y1/data.rev$Y)-
    data.rev$I1*cumsum(data.rev$delta/data.rev$Y)+cumsum(data.rev$delta*data.rev$Y1/(data.rev$Y)^2)
  data.rev$O.hat[data.rev$I0==1]<- (-1)*data.rev$O.hat[data.rev$I0==1]

  # calculate stratum-specific O_zij
  for(z in fz.n[-1]){
    data.tmp<-data.sort(data.simu[data.simu$fz==z,])
    T.seq<-data.tmp$t
    T.rep<-c(0,T.seq)[1:length(T.seq)]
    T.diff<-T.seq-T.rep
    same.ind<-which(T.diff==0)
    n_col<-dim(data.tmp)[2]
    for(ind in same.ind){
      data.tmp[ind,(n_col-2):n_col]<-data.tmp[ind-1,(n_col-2):n_col]
    }
    mu_t<-data.tmp$Y1/data.tmp$Y
    data.tmp$O.hat<-data.tmp$delta*(data.tmp$I1-data.tmp$Y1/data.tmp$Y)-
      data.tmp$I1*cumsum(data.tmp$delta/data.tmp$Y)+cumsum(data.tmp$delta*data.tmp$Y1/(data.tmp$Y)^2)
    data.tmp$O.hat[data.tmp$I0==1]<- (-1)*data.tmp$O.hat[data.tmp$I0==1]
    data.rev<-rbind(data.rev,data.tmp)
  }
  n<-dim(data.rev)[1]

  x.model<-as.matrix(data.rev[,grepl("model",names(data.rev)),drop=FALSE]) #only those has model

  fit.anhc1<-lm(O.hat~factor(fz)+x.model[data.rev$I1==1,],data=data.rev[data.rev$I1==1,])
  fit.anhc0<-lm(O.hat~factor(fz)+x.model[data.rev$I1==0,],data=data.rev[data.rev$I1==0,])
  which.model<-which(grepl("x.model",names(fit.anhc1$coefficients)))
  which.model.notNA<-which(!is.na(fit.anhc1$coefficients[which.model]))
  if(length(which.model.notNA)==0){
    stop("The model variables are linearly dependent on Z. The covariate-adjusted stratified log-rank is the same as the stratified log-rank.")
  }else{
    fit.anhc1.coefx<-fit.anhc1$coefficients[which.model[which.model.notNA]]
    fit.anhc0.coefx<-fit.anhc0$coefficients[which.model[which.model.notNA]]

    x.coefx<-model.matrix(~factor(fz)+as.matrix(x.model),data=data.rev)[,which.model[which.model.notNA],drop=FALSE]
    for(z in fz.n){
      x.coefx[data.rev$fz==z,]<-sweep(x.coefx[data.rev$fz==z,,drop=FALSE], 2, colMeans(x.coefx[data.rev$fz==z,,drop=FALSE]))
    }
    x.coefx.centered.byZ<-x.coefx
    U_CSL<- sum(data.rev$I1*data.rev$O.hat- data.rev$I0*data.rev$O.hat)-
      sum(data.rev$I1*as.vector(x.coefx.centered.byZ %*% fit.anhc1.coefx) - data.rev$I0*as.vector(x.coefx.centered.byZ %*% fit.anhc0.coefx))
    U_CSL<-U_CSL/n
  }
  ### calculate variance ###
  var_CSL<-0
  for(ind in 1:length(fz.n)){
    z<-fz.n[ind]
    Bz1<-var(data.rev$O.hat[data.rev$fz==z & data.rev$I1==1]-x.coefx[data.rev$fz==z & data.rev$I1==1,,drop=FALSE]%*%fit.anhc1.coefx)
    Bz0<-var(data.rev$O.hat[data.rev$fz==z & data.rev$I1==0]-x.coefx[data.rev$fz==z & data.rev$I1==0,,drop=FALSE]%*%fit.anhc0.coefx)
    if(is.na(Bz1) | is.na(Bz0)){
      warning("conditional variance equals 0")
      Bz1<-ifelse(is.na(Bz1),0,Bz1)
      Bz0<-ifelse(is.na(Bz0),0,Bz0)
    }
    SigmaXz<-var(x.coefx[data.rev$fz==z,,drop=FALSE])
    pz<-length(which(data.rev$fz==z))/n
    var_CSL<-var_CSL+pz*(p_trt*Bz1+(1-p_trt)*Bz0)+
      pz*(p_trt*fit.anhc1.coefx-(1-p_trt)*fit.anhc0.coefx) %*% SigmaXz %*% (p_trt*fit.anhc1.coefx-(1-p_trt)*fit.anhc0.coefx)
  }


  tmp_SL_var<-0
  tmp_CSL_var<-0
  for(z in fz.n){
    tmp_SL_var<-tmp_SL_var+wlogrank(data.rev[data.rev$fz==z,])$var.alt
    pz<-length(which(data.rev$fz==z))/n
    SigmaXz<-var(x.coefx[data.rev$fz==z,,drop=FALSE])
    tmp_CSL_var<-tmp_CSL_var+wlogrank(data.rev[data.rev$fz==z,])$var.alt/n -
      p_trt*(1-p_trt)*(fit.anhc1.coefx+fit.anhc0.coefx) %*% SigmaXz %*%(fit.anhc1.coefx+fit.anhc0.coefx)*pz
  }
  se<-sqrt(var_CSL/n)
  se.null<-sqrt(tmp_CSL_var/n)
  return(list(U_CSL=U_CSL,se=se.null,se.orig=se))
}

# Covariate adjusted logrank proposed in Ye, Yi, Shao (2022)
#' @title Ting Ye's original code
#' @export
covariate_adjusted_logrank <- function(data.simu, p_trt, Z=TRUE) {
  n <- dim(data.simu)[1]
  if(Z) data.simu$fz <- ind_to_factor(data.simu)
  data.rev <- data.sort(data.simu)
  T.seq <- data.rev$t
  T.rep <- c(0, T.seq)[1:n]
  T.diff <- T.seq - T.rep
  same.ind <- which(T.diff == 0)
  n_col <- dim(data.rev)[2]
  for (ind in same.ind) {
    data.rev[ind, (n_col - 2):n_col] <- data.rev[ind - 1, (n_col - 2):n_col]
  }
  n <- dim(data.rev)[1]

  x.model <-
    as.matrix(data.rev[, grepl("model", names(data.rev)), drop = FALSE]) #only those has model
  if(Z){
    x.mat <- model.matrix( ~ factor(fz) + x.model, data = data.rev)[, -1]
  } else {
    x.mat <- model.matrix( ~ x.model, data = data.rev)[, -1]
  }

  # Need this if there's only one covariate and no car_strata
  x.mat <- as.matrix(x.mat)
  x.centered <- sweep(x.mat, 2, colMeans(x.mat))
  # calculating beta0 and beta1 for variance estimation
  mu_t <- data.rev$Y1 / data.rev$Y
  data.rev$O.hat <-
    data.rev$delta * (data.rev$I1 - data.rev$Y1 / data.rev$Y) -
    data.rev$I1 * cumsum(data.rev$delta / data.rev$Y) + cumsum(data.rev$delta *
                                                                 data.rev$Y1 / (data.rev$Y) ^ 2)
  data.rev$O.hat[data.rev$I0 == 1] <-
    (-1) * data.rev$O.hat[data.rev$I0 == 1]
  if(Z){
    fit1.Ohat <-
      lm(O.hat ~ factor(fz) + x.model[data.rev$I1 == 1, ], data = data.rev[data.rev$I1 ==
                                                                             1, ])
    fit0.Ohat <-
      lm(O.hat ~ factor(fz) + x.model[data.rev$I1 == 0, ], data = data.rev[data.rev$I1 ==
                                                                             0, ])
  } else {
    fit1.Ohat <-
      lm(O.hat ~ x.model[data.rev$I1 == 1, ], data = data.rev[data.rev$I1 ==
                                                                1, ])
    fit0.Ohat <-
      lm(O.hat ~ x.model[data.rev$I1 == 0, ], data = data.rev[data.rev$I1 ==
                                                                0, ])
  }

  beta1.Ohat <- fit1.Ohat$coefficients[-1]
  beta0.Ohat <- fit0.Ohat$coefficients[-1]

  ind.na <- which(is.na(beta1.Ohat))
  if (length(ind.na) == 0) {
    U_CL <-
      sum(data.rev$I1 * (data.rev$O.hat - t((x.centered) %*% beta1.Ohat)) -
            data.rev$I0 * (data.rev$O.hat - t((x.centered) %*% beta0.Ohat))) /
      n
  } else{
    warning(
      "Removing model variables that are linearly dependent with the stratification variables."
    )
    x.mat <- x.mat[, -ind.na]
    x.centered <- x.centered[, -ind.na]
    beta1.Ohat <- beta1.Ohat[-ind.na]
    beta0.Ohat <- beta0.Ohat[-ind.na]
    U_CL <-
      sum(data.rev$I1 * (data.rev$O.hat - t((x.centered) %*% beta1.Ohat)) -
            data.rev$I0 * (data.rev$O.hat - t((x.centered) %*% beta0.Ohat))) /
      n
  }

  score_logrank_anhecova_var <-
    function(data.rev,
             p_trt,
             fit1.Ohat,
             fit0.Ohat,
             beta1.Ohat,
             beta0.Ohat) {
      my.var <-
        (
          p_trt * var(fit1.Ohat$residuals) + (1 - p_trt) * var(fit0.Ohat$residuals) +
            (p_trt * beta1.Ohat - (1 - p_trt) * beta0.Ohat) %*% var(x.mat) %*% (p_trt *
                                                                                  beta1.Ohat - (1 - p_trt) * beta0.Ohat)
        ) / n
      my.var.null <-
        (
          sum(data.rev$delta * data.rev$Y0 * data.rev$Y1 / data.rev$Y ^ 2) / n -
            p_trt * (1 - p_trt) * (beta1.Ohat + beta0.Ohat) %*% var(x.mat) %*% (beta1.Ohat +
                                                                                  beta0.Ohat)
        ) / n
      return(list(my.var = my.var, my.var.null = my.var.null))
    }
  tmp <-
    score_logrank_anhecova_var(data.rev, p_trt, fit1.Ohat, fit0.Ohat, beta1.Ohat, beta0.Ohat)
  se <- sqrt(tmp$my.var)
  se.null <- sqrt(tmp$my.var.null)
  return(list(
    U_CL = U_CL,
    se = se.null,
    se.orig = se
  ))
}

library(survival)
library(dplyr)

ting <- function(data){

  data.sort<-function(data.simu){
    n<-dim(data.simu)[1]
    data.simu.order<- data.simu[order(data.simu$t),]
    data.rev<- data.frame(data.simu.order,
                          Y=n:1,
                          Y1=cumsum(data.simu.order$I1[n:1])[n:1],
                          Y0=cumsum(data.simu.order$I0[n:1])[n:1]
    )

    data.rev <- data.rev %>% # handle ties, added by Ting
      group_by(t) %>%
      mutate(
        Y = first(Y),
        Y1 = first(Y1),
        Y0 = first(Y0),
        n.events= sum(delta)
      ) %>%
      ungroup()
    return(data.rev)
  }

  wlogrank<-function(data.simu){
    n<-dim(data.simu)[1]
    data.rev<-data.sort(data.simu)
    S.alt<-sum(data.rev$delta*(data.rev$I1*data.rev$Y0-data.rev$I0*data.rev$Y1)/data.rev$Y)
    var.alt<-sum(data.rev$delta*data.rev$Y0*data.rev$Y1*(data.rev$Y-data.rev$n.events)/data.rev$Y^2/(data.rev$Y-1),na.rm=TRUE)
    res<-S.alt/sqrt(var.alt)
    return(list(S.alt=S.alt, res=res, var.alt=var.alt))
  }

  # covariate adjusted logrank proposed in Ye, Yi, Shao (2022)
  covariate_adjusted_logrank <- function(data.simu){
    p_trt<-mean(data.simu$I1)
    n<-dim(data.simu)[1]
    data.simu$fz<-factor(1) # for illustration purpose, just one stratum
    data.rev<-data.sort(data.simu)

    data.rev <- data.rev %>% mutate(cum.delta.Y0.over.Ysq=cumsum(Y0/Y^2*delta), # added by Ting to handle ties
                                    cum.delta.Y1.over.Ysq=cumsum(Y1/Y^2*delta))
    data.rev <- data.rev %>%
      group_by(t) %>%
      mutate(
        cum.delta.Y0.over.Ysq=last(cum.delta.Y0.over.Ysq),
        cum.delta.Y1.over.Ysq=last(cum.delta.Y1.over.Ysq)
      ) %>%
      ungroup()

    mu_t<-data.rev$Y1/data.rev$Y
    data.rev$O.hat1<-data.rev$delta*(data.rev$Y0/data.rev$Y)-
      data.rev$cum.delta.Y0.over.Ysq
    data.rev$O.hat0<-data.rev$delta*(data.rev$Y1/data.rev$Y)-
      data.rev$cum.delta.Y1.over.Ysq
    data.rev$O.hat<-data.rev$I1*data.rev$O.hat1+data.rev$I0*data.rev$O.hat0

    sum(data.rev$I1*data.rev$O.hat -
          data.rev$I0*data.rev$O.hat)/n

    x.centered<-x.mat<-matrix(0,nrow=n,ncol=1)
    beta1.Ohat<-0
    beta0.Ohat<-0

    ind.na<-which(is.na(beta1.Ohat))
    if(length(ind.na)==0){
      U_CL<-sum(data.rev$I1*(data.rev$O.hat-t((x.centered)%*% beta1.Ohat )) -
                  data.rev$I0*(data.rev$O.hat-t((x.centered)%*% beta0.Ohat)))/n
    }else{
      warning("Removing model variables that are linearly dependent with the stratification variables.")
      x.mat<-x.mat[,-ind.na,drop=FALSE]
      x.centered<-x.centered[,-ind.na,drop=FALSE]
      beta1.Ohat<-beta1.Ohat[-ind.na]
      beta0.Ohat<-beta0.Ohat[-ind.na]
      U_CL<-sum(data.rev$I1*(data.rev$O.hat-t((x.centered)%*% beta1.Ohat )) -
                  data.rev$I0*(data.rev$O.hat-t((x.centered)%*% beta0.Ohat)))/n
    }

    score_logrank_anhecova_var<-function(data.rev,p_trt,fit1.Ohat,fit0.Ohat,beta1.Ohat,beta0.Ohat){
      my.var.null<-(wlogrank(data.simu)$var.alt/n -
                      # added by Ting to handle ties, uses log-rank variance estimator - variance reduction term
                      p_trt*(1-p_trt)*(beta1.Ohat+beta0.Ohat) %*% var(x.mat) %*% (beta1.Ohat+beta0.Ohat) )/n
      return(list(my.var.null=my.var.null))
    }
    tmp<-score_logrank_anhecova_var(data.rev,p_trt,fit1.Ohat,fit0.Ohat,beta1.Ohat,beta0.Ohat)
    se.null<-sqrt(tmp$my.var.null)
    return(list(U_CL=U_CL, se=se.null, res=U_CL/se.null))
  }

  return(covariate_adjusted_logrank(data))

}

DATA <- ovarian %>%
  rename(tte = futime, obs = fustat) %>%
  arrange(tte)

test_that("Logrank ovarian", {

  # (1) Survdiff function
  lr <- survdiff(Surv(tte, obs) ~ rx, data = DATA)
  lr_p <- lr$pvalue
  lr_num <- lr$obs[1] - lr$exp[1]
  lr_var <- lr$var[1, 1] / nrow(DATA)

  RC1 <- robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs"
  )
  RC1res <- (2 * pnorm(abs(RC1$result$statistic), lower.tail = F))

  data <- DATA %>% select(t = tte)
  data$delta <- 1*(DATA$obs==T)
  data$I1 <- DATA$rx - 1
  data$I0 <- 1 - data$I1

  tingres <- c(2 * pnorm(abs(ting(data)$res), lower.tail = F))

  expect_equal(RC1res, lr_p)
  expect_equal(RC1res, tingres)

})

test_that("Logrank Ovarian with ties", {

  # Artificially create ties -----------------------------------
  # ------------------------------------------------------------
  # ------------------------------------------------------------
  DATA2 <- DATA

  DATA2$tte[3:5] <- DATA2$tte[3]
  DATA2$tte[10:12] <- DATA2$tte[10]
  DATA2$tte[19:21] <- DATA2$tte[19]

  # (1) Survdiff function
  lr <- survdiff(Surv(tte, obs) ~ rx, data = DATA2)
  lr_p <- lr$pvalue
  lr_num <- lr$obs[1] - lr$exp[1]
  lr_var <- lr$var[1, 1] / nrow(DATA)

  RC1 <- robincar_logrank(
    adj_method = "CL",
    df = DATA2,
    treat_col = "rx",
    p_trt = 0.5,
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs"
  )
  RC1res <- (2 * pnorm(abs(RC1$result$statistic), lower.tail = F))

  data <- DATA2 %>% select(t = tte)
  data$delta <- 1*(DATA2$obs==T)
  data$I1 <- DATA2$rx - 1
  data$I0 <- 1 - data$I1

  tingres <- c(2 * pnorm(abs(ting(data)$res), lower.tail = F))

  expect_equal(RC1res, lr_p)
  expect_equal(RC1res, tingres)

})

library(survival)
library(dplyr)

test_that("No X no Z case under SR, CL = logrank test", {

  set.seed(0)
  n <- 100
  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case2"
  )

  data.simu0 <- data.simu0 %>% mutate(
      strata1=sample(letters[1:3],n,replace=TRUE),
      strata2=sample(LETTERS[4:5],n,replace=TRUE)
  )

  out <- robincar_logrank(df=data.simu0,
                          treat_col="I1",
                          p_trt=0.5,
                          ref_arm=0,
                          response_col="t",
                          event_col="delta",
                          car_scheme=c("simple"),
                          adj_method=c("CL")
  )

  surv_out <- coxph(Surv(t, delta) ~ I1, data = data.simu0)
  surv_out.score <-
    sqrt(surv_out$score) * ifelse(surv_out$coefficients > 0, 1, -1)

  expect_equal(out$result$statistic, unname(surv_out.score))

})

test_that("No X yes Z case under CAR, CSL = stratified logrank test",{

  set.seed(0)
  n <- 100
  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu <- data.simu0 %>%
    tidyr::pivot_longer(
      cols=starts_with("car_strata"),
      names_prefix="car_strata",
      names_to="strt"
    ) %>%
    dplyr::filter(value == 1) %>%
    dplyr::select(-value) %>%
    dplyr::mutate(strt = forcats::as_factor(strt)) %>%
    dplyr::select(t, strt) %>%
    dplyr::left_join(data.simu0, .)

  out1 <- robincar_logrank(
    df=data.simu,
    treat_col="I1",
    p_trt=0.5,
    ref_arm=0,
    response_col="t",
    event_col="delta",
    car_strata_cols=c("strt"),
    car_scheme=c("permuted-block"),
    adj_method=c("CSL")
  )

  surv_out1 <- coxph(Surv(t, delta) ~ I1 + strata(strt), data = data.simu)
  surv_out.score1 <- sqrt(surv_out1$score) *
    ifelse(surv_out1$coefficients > 0, 1, -1)

  expect_equal(out1$result$statistic, unname(surv_out.score1))

})

test_that("X yes Z yes, case1, CL",{

  set.seed(0)
  n <- 100
  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu <-
    data.simu0 %>%
    tidyr::pivot_longer(
      cols=starts_with("car_strata"),
      names_prefix="car_strata",
      names_to="strt"
    ) %>%
    dplyr::filter(value == 1) %>%
    dplyr::select(-value) %>%
    dplyr::mutate(strt = forcats::as_factor(strt)) %>%
    dplyr::select(t, strt) %>%
    dplyr::left_join(data.simu0, .)

  test1 <- robincar_logrank(
    df=data.simu,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols=c("strt"),
    covariate_cols=c("model_Z1", "model_Z21", "model_Z22"),
    car_scheme="permuted-block",
    adj_method=c("CL"),
    ref_arm=0
  )
  test1_ty <- covariate_adjusted_logrank(data.simu, p_trt = 0.5)
  expect_equal(test1$result$U, test1_ty$U_CL)
  expect_equal(test1$result$se, as.numeric(test1_ty$se))

  data.simu01 <- data_gen2(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu1 <- data.simu01 %>%
    tidyr::pivot_longer(
      cols=starts_with("car_strata"),
      names_prefix="car_strata",
      names_to="strt"
    ) %>%
    dplyr::filter(value == 1) %>%
    dplyr::select(-value) %>%
    dplyr::mutate(strt=forcats::as_factor(strt)) %>%
    dplyr::select(t, strt) %>%
    dplyr::left_join(data.simu01, .)

  test12 <- robincar_logrank(
    df=data.simu1,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols=c("strt"),
    covariate_cols=c("model_w3"),
    car_scheme="permuted-block",
    adj_method=c("CL"),
    ref_arm=0
  )
  test12_ty <- covariate_adjusted_logrank(data.simu1, p_trt = 0.5)
  expect_equal(test12$result$U, test12_ty$U_CL)
  expect_equal(test12$result$se, as.numeric(test12_ty$se))

})

test_that("X (continuous) yes Z no, case1, CL",{

  set.seed(0)
  n <- 100

  data.simu01 <- data_gen2(
    n=n,
    theta=0,
    randomization="SR",
    p_trt=0.5,
    case="case1"
  )

  # Add another continuous covariate because Ting's code requires
  data.simu1 <- data.simu01 %>%
    dplyr::mutate(strt=0)

  test12 <- robincar_logrank(
    df=data.simu1,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    covariate_cols=c("model_w3"),
    car_scheme="simple",
    adj_method=c("CL"),
    ref_arm=0
  )

  # Using the function with no Z, modified version of Ting's original code
  test12_ty <- covariate_adjusted_logrank(data.simu1, p_trt = 0.5, Z=FALSE)
  expect_equal(test12$result$U, test12_ty$U_CL)
  expect_equal(test12$result$se, as.numeric(test12_ty$se))

})

test_that("X yes Z yes, case1, CSL",{

  set.seed(100)
  n <- 100

  data.simu01 <- data_gen2(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu1 <-
    data.simu01 %>% tidyr::pivot_longer(
      cols=starts_with("car_strata"),
      names_prefix="car_strata",
      names_to="strt"
    ) %>%
    dplyr::filter(value == 1) %>% select(-value) %>%
    dplyr::mutate(strt=forcats::as_factor(strt)) %>% select(t, strt) %>%
    dplyr::left_join(data.simu01, .)

  test12 <- robincar_logrank(
    df=data.simu1,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols="strt",
    covariate_cols=c("model_w3"),
    car_scheme="permuted-block",
    adj_method=c("CSL"),
    ref_arm=0
  )
  test12_ty <- covariate_adjusted_stratified_logrank(data.simu1, p_trt=0.5)
  expect_equal(test12$result$U, test12_ty$U_CSL)
  expect_equal(test12$result$se, as.numeric(test12_ty$se))

})

#' @title Data generation function from JRSS-B paper
#' @param n total number of subjects to be generated
#' @param theta true treatment effect
#' @param randomization randomization method in c("SR","CABC","permuted_block","minimization","urn")
#' @param p_trt proportion of treatment arm
#' @param case simulation case in the paper
#' @export
#' @returns A data frame with the following columns:
#'
#' \item{t}{event time}
#' \item{delta}{event indicator}
#' \item{I1}{assignment to treatment group 1}
#' \item{I0}{assignment to treatment group 0}
#' \item{model_z1, model_z2}{covariates}
#' \item{car_strata1, ...}{strata variables}
data_gen <- function(n, theta, randomization, p_trt,
                     case=c("case1", "case2", "case3", "case4", "case5")){
  if(case=="case1") {
    # multiple z
    z1 <- stats::rmultinom(n, 1, c(0.5, 0.5))
    z2 <- stats::rmultinom(n, 1, c(0.4, 0.3, 0.3))
    minimization_z <- t(rbind(z1, z2))
    n_strata <- dim(z1)[1] * dim(z2)[1]
    strata_z <- matrix(nrow = n, ncol = n_strata)
    ind <- 1
    for (i in 1:dim(z1)[1]) {
      for (j in 1:dim(z2)[1]) {
        strata_z[, ind] <- z1[i, ] * z2[j, ]
        ind <- ind + 1
      }
    }
    I <-
      treatment_assignment(n, strata_z, minimization_z, randomization, p_trt)
    lambda0 <- log(2) / 12
    cumuhazard <- stats::rexp(n)
    HR <- exp(I * theta + 1.5 * z1[1, ] - z2[1, ] - 0.5 * z2[2, ])
    t.star <- cumuhazard / lambda0 / HR
    C <- stats::runif(n, 20, 40)
    t <- pmin(t.star, C)
    delta <- 1 * (t.star <= C)
    data.simu <-
      data.frame(t, delta, I, 1 - I, z1[1, ], z2[1, ], z2[2, ], strata_z, minimization_z)[1:n, ]
    names(data.simu) <-
      c(
        "t",
        "delta",
        "I1",
        "I0",
        "model_Z1",
        "model_Z21",
        "model_Z22",
        paste0("car_strata", 1:n_strata),
        paste0("minimization", 1:5)
      )
    return(data.simu)
  }
  else if (case == "case2") {
    # continuous z
    z1 <- stats::rmultinom(n, 1, c(0.5, 0.5))
    row.names(z1) <- paste0("z1_", 1:2)
    n_category <- 16
    tmp <- discretize_z(n, 0, 1, n_category)
    z2_continous <- tmp$Z
    z2 <- t(tmp$Z_dis)

    minimization_z <- t(rbind(z1, z2))
    minimization_n_col <- ncol(minimization_z)

    n_strata <- dim(z1)[1] * dim(z2)[1]
    strata_z <- matrix(nrow = n, ncol = n_strata)
    ind <- 1
    for (i in 1:dim(z1)[1]) {
      for (j in 1:dim(z2)[1]) {
        strata_z[, ind] <- z1[i, ] * z2[j, ]
        ind <- ind + 1
      }
    }
    I <-
      treatment_assignment(n, strata_z, minimization_z, randomization, p_trt)
    lambda0 <- log(2) / 12
    cumuhazard <- stats::rexp(n)
    HR <- exp(I * theta - 1.5 * z1[1, ] + 0.5 * z2_continous ^ 2)
    t.star <- cumuhazard / lambda0 / HR
    C <- stats::runif(n, 10, 40)
    t <- pmin(t.star, C)
    delta <- 1 * (t.star <= C)
    data.simu <-
      data.frame(t, delta, I, 1 - I, z1[1, ], z2_continous ^ 2, strata_z, minimization_z)[1:n, ]
    names(data.simu) <-
      c(
        "t",
        "delta",
        "I1",
        "I0",
        paste0("model_z", 1:2),
        paste0("car_strata", 1:n_strata),
        paste("minimization", 1:minimization_n_col)
      )
    return(data.simu)
  }
  else if (case == "case3") {
    # square (mis)
    z1 <- stats::rmultinom(n, 1, c(0.5, 0.5))
    row.names(z1) <- paste0("z1_", 1:2)
    n_category <- 4
    tmp <- discretize_z(n, 0, 1, n_category)
    z2_continous <- tmp$Z
    z2 <- t(tmp$Z_dis)

    minimization_z <- t(rbind(z1, z2))
    minimization_n_col <- ncol(minimization_z)

    n_strata <- dim(z1)[1] * dim(z2)[1]
    strata_z <- matrix(nrow = n, ncol = n_strata)
    ind <- 1
    for (i in 1:dim(z1)[1]) {
      for (j in 1:dim(z2)[1]) {
        strata_z[, ind] <- z1[i, ] * z2[j, ]
        ind <- ind + 1
      }
    }
    I <-
      treatment_assignment(n, strata_z, minimization_z, randomization, p_trt)
    lambda0 <- log(2) / 12
    cumuhazard <- stats::rexp(n)
    HR <- exp(I * theta - 0.5 * z1[1, ] + 1.5 * z2_continous ^ 2)
    t.star <- cumuhazard / lambda0 / HR
    C <- 10 + stats::rexp(2 * z1[1, ])
    t <- pmin(t.star, C)
    delta <- 1 * (t.star <= C)
    data.simu <-
      data.frame(t, delta, I, 1 - I, z1[1, ], z2_continous, strata_z, minimization_z)[1:n, ]
    names(data.simu) <-
      c(
        "t",
        "delta",
        "I1",
        "I0",
        paste0("model_z", 1:2),
        paste0("car_strata", 1:n_strata),
        paste("minimization", 1:minimization_n_col)
      )
    return(data.simu)
  }
  else if (case == "case4") {
    # non-PH (mis), C unif
    n_category <- 4
    tmp <- discretize_z(n, 0, 1, n_category)
    z1_continuous <- tmp$Z
    z1 <- t(tmp$Z_dis)
    minimization_z <- t(z1)
    minimization_n_col <- ncol(minimization_z)
    n_strata <- dim(z1)[1]
    strata_z <- t(z1)
    I <-
      treatment_assignment(n, strata_z, minimization_z, randomization, p_trt)
    t.star <- exp(theta * I + 1.5 * z1_continuous) + stats::rexp(n, 1)
    C <- stats::runif(n, 10, 20)
    t <- pmin(t.star, C)
    delta <- 1 * (t.star <= C)
    data.simu <-
      data.frame(t, delta, I, 1 - I, z1_continuous, strata_z, minimization_z)[1:n, ]
    names(data.simu) <-
      c(
        "t",
        "delta",
        "I1",
        "I0",
        "model_Z1",
        paste0("car_strata", 1:n_strata),
        paste0("minimization", 1:minimization_n_col)
      )
    return(data.simu)
  }
  else if (case == "case5") {
    # non-PH (mis), C depends on I
    n_category <- 4
    tmp <- discretize_z(n, 0, 1, n_category)
    z1_continuous <- tmp$Z
    z1 <- t(tmp$Z_dis)
    minimization_z <- t(z1)
    minimization_n_col <- ncol(minimization_z)
    n_strata <- dim(z1)[1]
    strata_z <- t(z1)
    I <-
      treatment_assignment(n, strata_z, minimization_z, randomization, p_trt)
    t.star <- exp(theta * I + 1.5 * z1_continuous) + stats::rexp(n, 1)
    C <- 3 - 3 * I + stats::rexp(n, 1)
    t <- pmin(t.star, C)
    delta <- 1 * (t.star <= C)
    data.simu <-
      data.frame(t, delta, I, 1 - I, z1_continuous, strata_z, minimization_z)[1:n, ]
    names(data.simu) <-
      c(
        "t",
        "delta",
        "I1",
        "I0",
        "model_Z1",
        paste0("car_strata", 1:n_strata),
        paste0("minimization", 1:minimization_n_col)
      )
    return(data.simu)
  }
}

#' @title Data generation function from covariate adjusted log-rank paper
#' @inheritParams data_gen
#' @param blocksize block size for permuted block design
#' @export
#'
#' @returns A data frame with the following columns:
#'
#' \item{t}{event time}
#' \item{delta}{event indicator}
#' \item{I1}{assignment to treatment group 1}
#' \item{I0}{assignment to treatment group 0}
#' \item{model_w3}{covariates}
#' \item{car_strata1, ...}{strata variables}
data_gen2 <-
  function(n,
           theta,
           randomization,
           p_trt,
           case = c("case1", "case2", "case3", "case4"),
           blocksize = 4) {
    if (case == "case1") {
      # Cox failure time and uniform censoring
      tmp <- discretize_z(n, 0, 1, n_category = 2)
      w1 <- tmp$Z
      z1 <- t(tmp$Z_dis)
      tmp <- discretize_z(n, 0, 1, n_category = 3)
      w2 <- tmp$Z
      z2 <- t(tmp$Z_dis)
      w3 <- stats::rnorm(n)
      z1.f <- as.factor(z1[1, ] + 2 * z1[2, ])
      z2.f <- as.factor(z2[1, ] + 2 * z2[2, ] + 3 * z2[3, ])
      minimization_z <- data.frame(z1.f, z2.f)

      n_strata <- dim(z1)[1] * dim(z2)[1]
      strata_z <- matrix(nrow = n, ncol = n_strata)
      ind <- 1
      for (i in 1:dim(z1)[1]) {
        for (j in 1:dim(z2)[1]) {
          strata_z[, ind] <- z1[i, ] * z2[j, ]
          ind <- ind + 1
        }
      }
      I <-
        treatment_assignment(n,
                             strata_z,
                             minimization_z,
                             randomization,
                             p_trt,
                             blocksize)
      lambda0 <- log(2)
      cumuhazard <- stats::rexp(n)
      HR <- exp(I * theta + 0.5 * w1 + 0.5 * w2 + 0.5 * w3)
      t.star <- cumuhazard / lambda0 / HR
      C <- stats::runif(n, 10, 40)
      t <- pmin(t.star, C)
      delta <- 1 * (t.star <= C)
      data.simu <- data.frame(t, delta, I, 1 - I, w3, strata_z)[1:n, ]
      names(data.simu) <-
        c("t",
          "delta",
          "I1",
          "I0",
          "model_w3",
          paste0("car_strata", 1:n_strata))
      return(data.simu)
    } else if (case == "case2") {
      # Cox failure time and censoring depends on I
      tmp <- discretize_z(n, 0, 1, n_category = 2)
      w1 <- tmp$Z
      z1 <- t(tmp$Z_dis)
      tmp <- discretize_z(n, 0, 1, n_category = 3)
      w2 <- tmp$Z
      z2 <- t(tmp$Z_dis)
      w3 <- stats::rnorm(n)
      z1.f <- as.factor(z1[1, ] + 2 * z1[2, ])
      z2.f <- as.factor(z2[1, ] + 2 * z2[2, ] + 3 * z2[3, ])
      minimization_z <- data.frame(z1.f, z2.f)

      n_strata <- dim(z1)[1] * dim(z2)[1]
      strata_z <- matrix(nrow = n, ncol = n_strata)
      ind <- 1
      for (i in 1:dim(z1)[1]) {
        for (j in 1:dim(z2)[1]) {
          strata_z[, ind] <- z1[i, ] * z2[j, ]
          ind <- ind + 1
        }
      }
      I <-
        treatment_assignment(n,
                             strata_z,
                             minimization_z,
                             randomization,
                             p_trt,
                             blocksize)
      lambda0 <- log(2)
      cumuhazard <- stats::rexp(n)
      HR <- exp(I * theta + 0.5 * w1 + 0.5 * w2 + 0.5 * w3)
      t.star <- cumuhazard / lambda0 / HR
      C <- 3 - 3 * I + stats::rexp(n, 2)
      t <- pmin(t.star, C)
      delta <- 1 * (t.star <= C)
      data.simu <- data.frame(t, delta, I, 1 - I, w3, strata_z)[1:n, ]
      names(data.simu) <-
        c("t",
          "delta",
          "I1",
          "I0",
          "model_w3",
          paste0("car_strata", 1:n_strata))
      return(data.simu)
    } else if (case == "case3") {
      # Non-Cox failure time and uniform censoring
      tmp <- discretize_z(n, 0, 1, n_category = 2)
      w1 <- tmp$Z
      z1 <- t(tmp$Z_dis)
      tmp <- discretize_z(n, 0, 1, n_category = 3)
      w2 <- tmp$Z
      z2 <- t(tmp$Z_dis)
      w3 <- stats::rnorm(n)
      z1.f <- as.factor(z1[1, ] + 2 * z1[2, ])
      z2.f <- as.factor(z2[1, ] + 2 * z2[2, ] + 3 * z2[3, ])
      minimization_z <- data.frame(z1.f, z2.f)

      n_strata <- dim(z1)[1] * dim(z2)[1]
      strata_z <- matrix(nrow = n, ncol = n_strata)
      ind <- 1
      for (i in 1:dim(z1)[1]) {
        for (j in 1:dim(z2)[1]) {
          strata_z[, ind] <- z1[i, ] * z2[j, ]
          ind <- ind + 1
        }
      }
      I <-
        treatment_assignment(n,
                             strata_z,
                             minimization_z,
                             randomization,
                             p_trt,
                             blocksize)
      t.star <- exp(I * theta + 0.5 * w1 + 0.5 * w2 + 0.5 * w3) + stats::rexp(n, 1)
      C <- stats::runif(n, 10, 40)
      t <- pmin(t.star, C)
      delta <- 1 * (t.star <= C)
      data.simu <- data.frame(t, delta, I, 1 - I, w3, strata_z)[1:n, ]
      names(data.simu) <-
        c("t",
          "delta",
          "I1",
          "I0",
          "model_w3",
          paste0("car_strata", 1:n_strata))
      return(data.simu)
    } else if (case == "case4") {
      # Non-Cox failure time and censoring depends on I
      tmp <- discretize_z(n, 0, 1, n_category = 2)
      w1 <- tmp$Z
      z1 <- t(tmp$Z_dis)
      tmp <- discretize_z(n, 0, 1, n_category = 3)
      w2 <- tmp$Z
      z2 <- t(tmp$Z_dis)
      w3 <- stats::rnorm(n)
      z1.f <- as.factor(z1[1, ] + 2 * z1[2, ])
      z2.f <- as.factor(z2[1, ] + 2 * z2[2, ] + 3 * z2[3, ])
      minimization_z <- data.frame(z1.f, z2.f)

      n_strata <- dim(z1)[1] * dim(z2)[1]
      strata_z <- matrix(nrow = n, ncol = n_strata)
      ind <- 1
      for (i in 1:dim(z1)[1]) {
        for (j in 1:dim(z2)[1]) {
          strata_z[, ind] <- z1[i, ] * z2[j, ]
          ind <- ind + 1
        }
      }
      I <-
        treatment_assignment(n,
                             strata_z,
                             minimization_z,
                             randomization,
                             p_trt,
                             blocksize)
      t.star <- exp(I * theta + 0.5 * w1 + 0.5 * w2 + 0.5 * w3) + stats::rexp(n, 1)
      C <- 3 - 3 * I + stats::rexp(n, 2)
      t <- pmin(t.star, C)
      delta <- 1 * (t.star <= C)
      data.simu <- data.frame(t, delta, I, 1 - I, w3, strata_z)[1:n, ]
      names(data.simu) <-
        c("t",
          "delta",
          "I1",
          "I0",
          "model_w3",
          paste0("car_strata", 1:n_strata))
      return(data.simu)
    } else if (case == "case5") {
      # Non-Cox failure time and censoring depends on I
      tmp <- discretize_z(n, 0, 1, n_category = 2)
      w1 <- tmp$Z
      z1 <- t(tmp$Z_dis)
      z1.f <- as.factor(z1[1, ] + 2 * z1[2, ])
      minimization_z <- data.frame(z1.f)
      n_strata <- dim(z1)[1]
      strata_z <- t(z1)
      I <-
        treatment_assignment(n,
                             strata_z,
                             minimization_z,
                             randomization,
                             p_trt,
                             blocksize)
      t.star <- exp(theta * I + 1.5 * w1) + stats::rexp(n, 1)
      C <- stats::rgamma(n, shape = 5) * (1 - I) + stats::rgamma(n, shape = 1) * I
      t <- pmin(t.star, C)
      delta <- 1 * (t.star <= C)
      data.simu <- data.frame(t, delta, I, 1 - I, w1, strata_z)[1:n, ]
      names(data.simu) <-
        c("t",
          "delta",
          "I1",
          "I0",
          "model_w1",
          paste0("car_strata", 1:n_strata))
      return(data.simu)
    } else if (case == "case4_test") {
      # Non-Cox failure time and censoring depends on I
      tmp <- discretize_z(n, 0, 1, n_category = 2)
      w1 <- tmp$Z
      z1 <- t(tmp$Z_dis)
      tmp <- discretize_z(n, 0, 1, n_category = 3)
      w2 <- tmp$Z
      z2 <- t(tmp$Z_dis)
      w3 <- stats::rnorm(n)
      z1.f <- as.factor(z1[1, ] + 2 * z1[2, ])
      z2.f <- as.factor(z2[1, ] + 2 * z2[2, ] + 3 * z2[3, ])
      minimization_z <- data.frame(z1.f, z2.f)

      n_strata <- dim(z1)[1] * dim(z2)[1]
      strata_z <- matrix(nrow = n, ncol = n_strata)
      ind <- 1
      for (i in 1:dim(z1)[1]) {
        for (j in 1:dim(z2)[1]) {
          strata_z[, ind] <- z1[i, ] * z2[j, ]
          ind <- ind + 1
        }
      }

      fz <- numeric(n)
      for (i in 1:n_strata) {
        fz <- fz + strata_z[, i] * i
      }

      I <-
        treatment_assignment(n,
                             strata_z,
                             data.frame(as.factor(fz)),
                             randomization,
                             p_trt,
                             blocksize)
      t.star <- exp(I * theta + 0.5 * w1 + 0.5 * w2 + 0.5 * w3) + stats::rexp(n, 1)
      C <- 3 - 3 * I + stats::rexp(n, 2)
      t <- pmin(t.star, C)
      delta <- 1 * (t.star <= C)
      data.simu <- data.frame(t, delta, I, 1 - I, w3, strata_z)[1:n, ]
      names(data.simu) <-
        c("t",
          "delta",
          "I1",
          "I0",
          "model_w3",
          paste0("car_strata", 1:n_strata))
      return(data.simu)
    }
  }

discretize_z <- function(n, z_mean, z_sd, n_category) {
  # only normal
  Z <- stats::rnorm(n, z_mean, z_sd)
  p_cut <- seq(0, 1, length.out = (n_category + 1))
  q_cut <- sapply(p_cut, function(x)
    stats::qnorm(x, z_mean, z_sd))
  Z_dis_scale <- as.factor(sapply(Z, function(x)
    max(which(q_cut < x))))
  Z_model_matrix <- model.matrix( ~ 0 + Z_dis_scale)
  return(list(Z = Z, Z_dis = Z_model_matrix))
}

SR <- function(n, p_trt) {
  I <- rbinom(n, 1, p_trt)
  return(I)
}


BC <-
  function(n_within_strata, BCD_p = 2 / 3) {
    # Efron's biased coin randomization (Efron, 1971)
    stopifnot(BCD_p > 1 / 2)
    I <- numeric(n_within_strata)
    I[1] <- rbinom(1, 1, 1 / 2)
    D <- 2 * I[1] - 1
    for (i in 2:n_within_strata) {
      if (D > 0)
        I[i] <- stats::rbinom(1, 1, (1 - BCD_p))
      else if (D < 0)
        I[i] <- stats::rbinom(1, 1, BCD_p)
      else if (D == 0)
        I[i] <- stats::rbinom(1, 1, 1 / 2)
      D <- D + 2 * I[i] - 1
    }
    return(I)
  }

CABC <- function(z, BCD_p = 2 / 3) {
  n_strata <- dim(z)[2]
  n <- dim(z)[1]
  n_each_strata <- apply(z, 2, sum)
  I <- numeric(n)
  for (s in 1:n_strata) {
    if (n_each_strata[s] == 0) {
      next
    }
    else if (n_each_strata[s] == 1) {
      I[z[, s] == 1] <- stats::rbinom(1, 1, 0.5)
    }
    else{
      I[z[, s] == 1] <- BC(n_each_strata[s], BCD_p)
    }
  }
  return(I)
}

urn_design <-
  function(n_within_strata, omega, gamma, beta) {
    # Wei's urn design (Wei, 1978, JASA, Vol.73, No. 363)
    I <- numeric(n_within_strata)
    n_balls_0 <- omega
    n_balls_1 <- omega
    for (i in 1:n_within_strata) {
      p <- n_balls_1 / (n_balls_1 + n_balls_0)
      I[i] <- stats::rbinom(1, 1, p)
      n_balls_0 <- n_balls_0 + (1 - I[i]) * gamma + I[i] * beta
      n_balls_1 <- n_balls_1 + (1 - I[i]) * beta + I[i] * gamma
    }
    return(I)
  }

strata_urn_design <- function(z) {
  omega <- 1
  gamma <- 0
  beta <- 1
  n_strata <- dim(z)[2]
  n <- dim(z)[1]
  n_each_strata <- apply(z, 2, sum)
  I <- numeric(n)
  for (i in 1:n_strata) {
    if (n_each_strata[i] == 0) {
      next
    }
    else if (n_each_strata[i] == 1) {
      I[z[, i] == 1] <- stats::rbinom(1, 1, 0.5)
    }
    else{
      I[z[, i] == 1] <- urn_design(n_each_strata[i], omega, gamma, beta)
    }
  }
  return(I)
}


permuted_block <- function(z, blocksize, p_trt) {
  n_trt_blk <- blocksize * p_trt
  n_ctl_blk <- blocksize * (1 - p_trt)
  if (!(floor(n_ctl_blk) == n_ctl_blk &
        floor(n_trt_blk) == n_trt_blk)) {
    stop("number of treatment and control within block are not integers!")
  }
  if (blocksize)
    n_strata <- dim(z)[2]
  n <- dim(z)[1]
  n_each_strata <- apply(z, 2, sum)
  I <- numeric(n)
  for (i in 1:n_strata) {
    I_tmp <- 0
    n_block_tmp <- ceiling(n_each_strata[i] / blocksize)
    for (b in 1:n_block_tmp) {
      block_permuted_tmp <- sample(c(rep(0, n_ctl_blk), rep(1, n_trt_blk)))
      I_tmp <- c(I_tmp, block_permuted_tmp)
    }
    I_tmp <- I_tmp[-1]
    I[z[, i] == 1] <- I_tmp[1:n_each_strata[i]]
  }
  return(I)
}

treatment_assignment <-
  function(n,
           strata_z,
           minimization_z,
           randomization,
           p_trt,
           blocksize = 4) {
    if (randomization == "SR") {
      I <- SR(n, p_trt)
    } else if (randomization == "CABC") {
      if (p_trt != 1 / 2) {
        stop("For now, the proportion of treatment under CABC has to be 1/2.")
      }
      I <- CABC(strata_z)
    } else if (randomization == "permuted_block") {
      if (p_trt != 1 / 2) {
        stop("For now, the proportion of treatment under permuted_block has to be 1/2.")
      }
      I <- permuted_block(strata_z, blocksize, p_trt)
    } else if (randomization == "minimization") {
      if (p_trt != 1 / 2) {
        stop("For now, the proportion of treatment under CABC has to be 1/2.")
      }
      I <- minimization(minimization_z)
    } else if (randomization == "urn") {
      if (p_trt != 1 / 2) {
        stop("For now, the proportion of treatment under CABC has to be 1/2.")
      }
      I <- strata_urn_design(strata_z)
    }
    return(I)
  }

minimization <- function(z,
                         imbalance_measure = "square",
                         BCD_p = 2 / 3) {
  z <- as.matrix(z)
  n <- dim(z)[1]
  I <- numeric(n)
  I[1] <- rbinom(1, 1, 1 / 2)
  D <- matrix(0, nrow = dim(z)[1], ncol = dim(z)[2])
  D[1, ] <- (2 * I[1] - 1) * z[1, ]
  for (i in 2:n) {
    D_before_i <- D[i - 1, ]
    D_potential_1 <- D_before_i + z[i, ]
    D_potential_0 <- D_before_i - z[i, ]
    if (imbalance_measure == "square") {
      imb_potential_1 <- sum(D_potential_1 ^ 2)
      imb_potential_0 <- sum(D_potential_0 ^ 2)
    } else if (imbalance_measure == "absolute") {
      imb_potential_1 <- sum(abs(D_potential_1))
      imb_potential_0 <- sum(abs(D_potential_0))
    }

    if (imb_potential_1 > imb_potential_0) {
      # imbalance is larger if assinged to I=1
      I[i] <- rbinom(1, 1, (1 - BCD_p)) #BCD_p>1/2
    } else if (imb_potential_1 < imb_potential_0) {
      I[i] <- rbinom(1, 1, BCD_p)
    } else if (imb_potential_1 == imb_potential_0) {
      I[i] <- rbinom(1, 1, 0.5)
    }
    D[i, ] <- D_potential_1 * I[i] + D_potential_0 * (1 - I[i])
  }
  return(I)
}

library(survival)
library(dplyr)

test_that("No X no Z case under SR, CL = logrank test",{

  set.seed(0)

  n <- 1000
  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case2"
  )

  data.simu0 <- data.simu0 %>% mutate(
    strata1=sample(letters[1:3],n,replace=TRUE),
    strata2=sample(LETTERS[4:5],n,replace=TRUE)
  )

  out <- robincar_covhr(df=data.simu0,
                        treat_col="I1",
                        p_trt=0.5,
                        ref_arm=0,
                        response_col="t",
                        event_col="delta",
                        car_strata_cols=NULL,
                        covariate_cols=NULL,
                        car_scheme=c("simple"),
                        adj_method=c("CL")
  )

  surv_out <- coxph(Surv(t, delta) ~ I1, data=data.simu0)
  surv_out.score <-
    sqrt(surv_out$score) * ifelse(surv_out$coefficients > 0, 1, -1)

  expect_equal(out$result$theta_L, out$result$theta_CL)
  # Need a tolerance here because possibly a slightly different optimization
  # algorithm to find the hazard ratio than coxph
  expect_equal(out$result$theta_L, unname(surv_out$coefficients), tolerance=1e-5)
  expect_equal(out$result$se_theta_L, as.numeric(sqrt(surv_out$var)))

})



test_that("X yes Z yes, case1, CL", {

  set.seed(0)
  n <- 100
  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu <- data.simu0 %>%
    tidyr::pivot_longer(cols=starts_with("car_strata"),
                        names_prefix = "car_strata",
                        names_to = "strt") %>%
    filter(value==1) %>% select(-value) %>%
    mutate(strt=forcats::as_factor(strt)) %>%
    select(t,strt) %>%
    left_join(data.simu0,.)

  test1 <- robincar_covhr(
    df=data.simu,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols="strt",
    covariate_cols=c("model_Z1", "model_Z21", "model_Z22"),
    car_scheme="permuted-block",
    adj_method=c("CL"),
    ref_arm=0
  )
  test1_ty <- covariate_adjusted_logrank_estimation(data.simu, p_trt=0.5)
  expect_equal(test1$result$theta_L,test1_ty$theta_L)
  expect_equal(test1$result$theta_CL,test1_ty$theta_CL)
  expect_equal(test1$result$se_theta_L,test1_ty$se.theta_L)
  expect_equal(test1$result$se_theta_CL, as.numeric(test1_ty$se.theta_CL))

  data.simu01 <- data_gen2(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu1 <- data.simu01 %>%
    tidyr::pivot_longer(cols=starts_with("car_strata"),
                        names_prefix = "car_strata",
                        names_to = "strt") %>%
    filter(value==1) %>% select(-value) %>%
    mutate(strt=forcats::as_factor(strt)) %>%
    select(t,strt) %>%
    left_join(data.simu01,.)

  test12 <- robincar_covhr(
    df=data.simu1,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols="strt",
    covariate_cols=c("model_w3"),
    car_scheme="permuted-block",
    adj_method=c("CL"),
    ref_arm=0
  )

  test12_ty <- covariate_adjusted_logrank_estimation(data.simu1, p_trt=0.5)
  expect_equal(test12$result$theta_L,test12_ty$theta_L)
  expect_equal(test12$result$theta_CL,test12_ty$theta_CL)
  expect_equal(test12$result$se_theta_L,test12_ty$se.theta_L)
  expect_equal(test12$result$se_theta_CL,as.numeric(test12_ty$se.theta_CL))

})


test_that("X yes Z yes, case1, CSL",{

  n <- 100

  data.simu01 <- data_gen2(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu1 <- data.simu01 %>%
    tidyr::pivot_longer(cols=starts_with("car_strata"),
                        names_prefix = "car_strata",
                        names_to = "strt") %>%
    filter(value==1) %>% select(-value) %>%
    mutate(strt=forcats::as_factor(strt)) %>%
    select(t,strt) %>%
    left_join(data.simu01,.)

  test12 <- robincar_covhr(
    df=data.simu1,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols="strt",
    covariate_cols=c("model_w3"),
    car_scheme="permuted-block",
    adj_method=c("CSL"),
    ref_arm=0
  )
  test12_ty <- covariate_adjusted_stratified_logrank_estimation(data.simu1,p_trt=0.5)
  expect_equal(test12$result$theta_L,test12_ty$theta_SL)
  expect_equal(test12$result$theta_CL,test12_ty$theta_CSL)
  expect_equal(test12$result$se_theta_L,test12_ty$se.theta_SL)
  expect_equal(test12$result$se_theta_CL,as.numeric(test12_ty$se.theta_CSL))

})

library(dplyr)

test_that("data preprocessing", {

  set.seed(0)
  n <- 100

  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case2"
  ) %>% mutate(
    strata1=sample(letters[1:3], n, replace=TRUE),
    strata2=sample(LETTERS[4:5], n, replace=TRUE)
  )

  # CSL for LOGRANK ----------------------------------------
  # Should have a car_strata with multiple car_strata values

  data_csl <- robincar_logrank(
    df=data.simu0,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols=c("strata1", "strata2"),
    covariate_cols=c("model_z1", "model_z2"),
    car_scheme="permuted-block",
    adj_method="CSL"
  )

  df_csl <- create.tte.df(model=data_csl$settings, data=data_csl$data)
  expect_true(length(unique(df_csl$car_strata)) != 1)
  expect_true(is.factor(df_csl$car_strata))
  expect_true(is.factor(df_csl$carcov_z))

  data_cl <- robincar_logrank(
    df=data.simu0,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols=c("strata1", "strata2"),
    covariate_cols=c("model_z1", "model_z2"),
    car_scheme="permuted-block",
    adj_method="CL"
  )

  df_cl <- create.tte.df(model=data_cl$settings, data=data_cl$data)
  expect_true(length(unique(df_cl$car_strata)) == 1)
  expect_true(is.factor(df_cl$carcov_z))

  data_score <- robincar_coxscore(
    df=data.simu0,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols=c("strata1", "strata2"),
    covariate_cols=c("model_z1", "model_z2"),
    car_scheme="permuted-block",
  )

  df_score <- create.tte.df(model=data_score$settings, data=data_score$data)
  expect_true(length(unique(df_score$car_strata)) == 1)
  expect_true(is.factor(df_score$carcov_z))

  # If there are no covariates
  data_score1 <- robincar_coxscore(
    df=data.simu0,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols=c("strata1", "strata2"),
    car_scheme="permuted-block"
  )
  df_score1 <- create.tte.df(model=data_score$settings, data=data_score$data)
  expect_true(length(unique(df_score1$car_strata)) == 1)
  expect_true(is.factor(df_score1$carcov_z))
  expect_false(all(grepl("robcarx_", colnames(df_score1))))

})

library(dplyr)
library(survival)

test_that("X yes Z yes, permuted-block, coxscore",{

  set.seed(10)
  n <- 100
  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case2"
  )

  data.simu <-
    data.simu0 %>% tidyr::pivot_longer(
      cols=starts_with("car_strata"),
      names_prefix="car_strata",
      names_to="strt"
    ) %>%
    dplyr::filter(value == 1) %>%
    dplyr::select(-value) %>%
    dplyr::mutate(strt=forcats::as_factor(strt)) %>%
    dplyr::select(t, strt) %>%
    dplyr::left_join(data.simu0, .)

  test1 <- robincar_coxscore(
    df=data.simu,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols="strt",
    covariate_cols=c("model_z1", "model_z2"),
    car_scheme="permuted-block",
    ref_arm=0
  )
  # print(test1)
  test1_ty <- score_robust(data.simu,
                           randomization="permuted_block",
                           p_trt=0.5)
  expect_equal(test1$result$U, test1_ty$U)
  expect_equal(test1$result$se, as.numeric(test1_ty$se))
  expect_equal(test1$result$statistic, as.numeric(test1_ty$T_SR))

  data.simu01 <- data_gen2(
    n=n,
    theta=0,
    randomization="permuted_block",
    p_trt=0.5,
    case="case1"
  )

  data.simu1 <-
    data.simu01 %>% tidyr::pivot_longer(
      cols=starts_with("car_strata"),
      names_prefix="car_strata",
      names_to="strt"
    ) %>%
    dplyr::filter(value == 1) %>%
    dplyr::select(-value) %>%
    dplyr::mutate(strt=forcats::as_factor(strt)) %>%
    dplyr::select(t, strt) %>%
    dplyr::left_join(data.simu01, .)

  test2 <- robincar_coxscore(
    df=data.simu1,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols="strt",
    covariate_cols=c("model_w3"),
    car_scheme="permuted-block",
    ref_arm=0
  )
  test2_ty <- score_robust(data.simu1,
                           randomization="permuted_block",
                           p_trt=0.5)
  expect_equal(test2$result$U, test2_ty$U)
  expect_equal(test2$result$se, as.numeric(test2_ty$se))
  expect_equal(test2$result$statistic, as.numeric(test2_ty$T_SR))

})




test_that("No X no Z case under SR, coxscore",{

  n <- 100
  data.simu0 <- data_gen(
    n=n,
    theta=0,
    randomization="SR",
    p_trt=0.5,
    case="case2"
  )

  data.simu <-
    data.simu0 %>% tidyr::pivot_longer(
      cols=starts_with("car_strata"),
      names_prefix="car_strata",
      names_to="strt"
    ) %>%
    dplyr::filter(value == 1) %>%
    dplyr::select(-value) %>%
    dplyr::mutate(strt=forcats::as_factor(strt)) %>%
    dplyr::select(t, strt) %>%
    dplyr::left_join(data.simu0, .)

  test1 <- robincar_coxscore(
    df=data.simu,
    treat_col="I1",
    response_col="t",
    event_col="delta",
    car_strata_cols=NULL,
    covariate_cols=NULL,
    car_scheme="simple",
    ref_arm=0
  )

  expect_true(is.numeric(test1$result$"statistic"))

})

#' Print linear model result
#'
#' @param x A LinModelResult object
#' @param ... Additional arguments
#' @export
#'
#' @returns Prints the treatment mean estimates (and variances) based on a linear working model,
#' along with the settings used. See \link[RobinCar:robincar_linear]{RobinCar::robincar_linear()}.
print.LinModelResult <- function(x, ...){
  descript(x)
  cat("\n\n")
  cat("Estimates:\n")
  print(x$result)
  cat("\nVariance-Covariance Matrix:\n")
  print(x$varcov)
}

#' Print glm model result
#'
#' @param x A GLMModelResult object
#' @param ... Additional arguments
#' @export
#'
#' @returns Prints the treatment mean estimates (and variances) based on a GLM working model,
#' along with the settings used. See \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}.
print.GLMModelResult <- function(x, ...){
  descript(x)
  cat("\n\n")
  cat("Estimates:\n")
  print(x$result)
  cat("\nVariance-Covariance Matrix:\n")
  print(x$varcov)
}

#' Print contrast result
#'
#' @param x A ContrastResult object
#' @param ... Additional arguments
#' @export
#'
#' @returns Prints estimates (and variances) of treatment contrasts based on a linear or GLM working model,
#' along with the settings used. See \link[RobinCar:robincar_contrast]{RobinCar::robincar_contrast()}
print.ContrastResult <- function(x, ...){
  if("DIFF" %in% class(x$settings)){
    c_type <- "linear contrast"
  } else if("RATIO" %in% class(x$settings)){
    c_type <- "ratio contrast"
  } else {
    c_type <- "custom contrast function"
  }
  output <- sprintf("Treatment group contrasts using %s", c_type)
  cat(output)
  cat("\n\n")
  cat("Contrasts:\n")
  print(x$result)
  cat("\nVariance-Covariance Matrix for Contrasts:\n")
  print(x$varcov)
}

#' Print TTE result
#'
#' @param x A TTEResult object
#' @param ... Additional arguments
#'
#' @importFrom data.table data.table setorder setnames .SD :=
#' @import data.table
#' @export
#'
#' @returns Prints results of time-to-event covariate adjusted analyses including covariate-adjusted (stratified) logrank,
#' robust Cox score, and covariate-adjusted hazard ratio. Prints summary statistics about number of observations and events, possibly by strata,
#' and the test statistics and/or estimates, and p-values. See \link[RobinCar:robincar_tte]{RobinCar::robincar_tte()} and \link[RobinCar:robincar_covhr]{RobinCar::robincar_covhr()}.
print.TTEResult <- function(x, ...){

  if(!is.null(x$data$covariate)){
    covariates <- colnames(x$data$covariate)
  } else {
    covariates <- c()
  }
  if(!is.null(x$data$car_strata)){
    car_strata <- colnames(x$data$car_strata)
  } else {
    car_strata <- c()
  }
  if(x$settings$method == "CSL"){
    if((x$settings$car_strata)){

    }
  }
  if((x$settings$method == "CL") | (x$settings$method == "CSL" & !x$settings$car_strata)){
    if((x$settings$adj_cov) | x$settings$adj_strata){
      if("TTEResultEst" %in% class(x)){
        cat("Performed covariate-adjusted cox hazard ratio estimation with covariates ",
            paste0(c(covariates, car_strata), sep=", "))
      } else {
        cat("Performed covariate-adjusted logrank test with covariates ",
            paste0(c(covariates, car_strata), sep=", "))
      }
    } else {
      if("TTEResultEst" %in% class(x)){
        cat("Performed cox hazard ratio estimation.")
      } else {
        cat("Performed logrank test.")
      }
    }
  } else if(x$settings$method == "CSL"){
    if((x$settings$adj_cov)){
      if("TTEResultEst" %in% class(x)){
        cat("Performed covariate-adjusted cox hazard ratio estimation with covariates ",
            paste0(c(covariates), sep=", "),
            " and stratifying by ", paste0(c(car_strata), sep=", "))
      } else {
        cat("Performed covariate-adjusted stratified logrank test with covariates ",
            paste0(c(covariates), sep=", "),
            " and stratifying by ", paste0(c(car_strata), sep=", "))
      }
    } else {
      if("TTEResultEst" %in% class(x)){
        cat("Performed stratified cox hazard ratio estimation stratifying by ",
            paste0(c(car_strata), sep=", "))
      } else {
        cat("Performed stratified logrank test stratifying by ",
            paste0(c(car_strata), sep=", "))
      }
    }
  } else if(x$settings$method == "coxscore"){
    cat("Performed coxscore test")
    if(x$settings$adj_cov){
      cat(", \n  adjusting for covariates", paste0(covariates, collapse=", "))
    }
    if(x$settings$adj_strata){
      cat(", \n  adjusting SE for car_strata", paste0(car_strata, collapse=", "))
    }
  }
  cat("\n------------------------------------\n")

  # Make visible binding for global variables
  # Recommended by data.table developers
  # These are names of columns of the data.table created below
  # and including them as NULL allows the R CMD CHECK to pass
  # https://www.r-bloggers.com/2019/08/no-visible-binding-for-global-variable/

  df <- data.table::data.table(observed=x$data$event, treat=x$data$treat)
  df[, treat := as.character(treat)]

  df[, N := 1]
  id.cols <- c("treat")
  if(x$settings$car_strata){
    df$car_strata <- x$data$joint_strata
    id.cols <- c(id.cols, "car_strata")
    data.table::setorder(df, car_strata, treat)
  } else {
    data.table::setorder(df, treat)
  }
  txtitle <- "Treatment Group"

  summ <- df[, lapply(.SD, sum), by=id.cols, .SDcols=c("N", "observed")]
  summ[, name := paste0(x$data$treat_col, " = ", treat)]

  if(x$settings$car_strata){
    summ[, strata_col := paste0("car_strata = ", car_strata)]
    summ <- summ[, c("strata_col", "name", "N", "observed"), with=F]
    data.table::setnames(summ, c("Strata", "Treatment", "N.total", "N.events"))
  } else {
    summ <- summ[, c("name", "N", "observed")]
    data.table::setnames(summ, c("Treatment", "N.total", "N.events"))
  }

  print(summ)
  cat("\nReference arm is ", x$data$treat_col, "=", x$settings$ref_arm, "\n")
  if("TTEResultEst" %in% class(x)){
    stat <- x$result$theta_CL/x$result$se_theta_CL
    cat(
      "\nTest Stat:", x$result$theta_CL/x$result$se_theta_CL,
      "\n2-side p-value:", 2*(1-stats::pnorm(abs(stat))),
      "\nHR:", exp(x$result$theta_CL),
      "\nLog HR:", (x$result$theta_CL),
      "\nLog HR SE:", x$result$se_theta_CL
    )
  } else {
    cat(
      "\nScore function:", x$result$U,
      "\nStandard error:", x$result$se,
      "\nTest statistic:", x$result$statistic,
      "\n2-side p-value:", 2*(1-stats::pnorm(abs(x$result$statistic)))
    )
  }
}

#' Print calibration result
#'
#' @param x A GLMModel result. If you'd like to calibrate a linear
#'          adjustment, use `robincar_glm` instead of `robincar_linear`.
#' @param ... Additional arguments
#' @export
#'
#' @returns Prints the treatment mean estimates (and variances) based on a calibration on top of a
#' GLM working model, along with the settings used. See \link[RobinCar:robincar_calibrate]{RobinCar::robincar_calibrate()}.
print.CalibrationResult <- function(x, ...){
  if(x$joint){
    type <- "Joint calibration"
  } else {
    type <- "Linear calibration"
  }

  if(!is.null(x$add_x)){
    add_x_text <- paste0(", additionally adjusting for covariates",
                  paste0(x$add_x, sep=", "), ",")
  } else {
    add_x_text <- ""
  }
  output <- sprintf("%s%s on the following model result:",
                    type, add_x_text)

  cat(output)
  cat("\n")
  cat("--------------------------------------------\n")
  descript(x$original)
  cat("\n\nEstimates:\n")
  print(x$result)
  cat("\nVariance-Covariance Matrix:\n")
  print(x$varcov)
}


#' Print MH result
#'
#' @param x A ContrastResult object
#' @param ... Additional arguments
#' @export
#'
#' @returns Prints estimates (and variances) of treatment contrasts based on MH risk difference or ATE
print.MHResult <- function(x, ...){
  if("MH" == x$settings$estimand){
    est_type <- "Mantel-Haenszel risk difference"
  } else {
    est_type <- "ATE"
  }
  if("GR" == x$settings$ci_type){
    c_type <- "Greenland's estimator"
  } else if("mGR" == x$settings$ci_type){
    c_type <- "modified Greenland's estimator"
  } else{
    c_type <- "Sato's estimator"
  }
  output_title <- sprintf("Treatment group contrasts based on %s", est_type)
  output_estimand <- sprintf("Estimand: %s", est_type)
  output_strat <- sprintf("Stratified by %s", paste(x$settings$strata_cols, collapse=", "))
  output_ci <- sprintf("SE calculated via %s", c_type)
  cat(output_title)
  cat("\n")
  cat(output_estimand)
  cat("\n")
  cat(output_strat)
  cat("\n")
  cat(output_ci)
  cat("\n\n")
  cat("Contrasts:\n")
  print(x$result)
}

#' @importFrom dplyr group_by mutate ungroup
get.ordered.data.est <- function(df, lin_predictors){

  # Add on cumulative delta for ties
  df <- df %>%
    group_by(.data$car_strata) %>%
    mutate(
      cum.delta.Y0.over.Ysq.theta_L=cumsum(.data$event*.data$Y0/(exp(lin_predictors)*.data$Y1 + .data$Y0)^2),
      cum.delta.Y1.over.Ysq.theta_L=cumsum(.data$event*.data$Y1/(exp(lin_predictors)*.data$Y1 + .data$Y0)^2)
    ) %>%
    group_by(.data$car_strata, .data$response) %>%
    mutate(
      cum.delta.Y0.over.Ysq.theta_L=last(.data$cum.delta.Y0.over.Ysq.theta_L),
      cum.delta.Y1.over.Ysq.theta_L=last(.data$cum.delta.Y1.over.Ysq.theta_L)
    ) %>%
    ungroup()

  data <- df %>% dplyr::mutate(lin_pred=lin_predictors)

  data <- data %>%
    dplyr::group_by(.data$car_strata) %>%
    dplyr::mutate(
      s0_seq          = exp(.data$lin_pred)
    ) %>% dplyr::mutate(
      mu_num          = .data$s0_seq * .data$Y1,
      mu_denom        = (.data$s0_seq * .data$Y1 + .data$Y0)
    ) %>% dplyr::mutate(
      mu_t            = .data$mu_num / .data$mu_denom,
      score_i         = .data$event * (.data$trt1 - .data$mu_t),
      O.hat1          = .data$event * .data$Y0 / .data$mu_denom -
        .data$s0_seq * .data$cum.delta.Y0.over.Ysq.theta_L,
      O.hat0          = .data$event * .data$s0_seq * .data$Y1 / .data$mu_denom -
        .data$s0_seq * .data$cum.delta.Y1.over.Ysq.theta_L
    ) %>% dplyr::mutate(
      O.hat           = .data$O.hat0 * (.data$trt0 == 1) +
        .data$O.hat1 * (.data$trt1 == 1)
    ) %>% dplyr::ungroup()

  return(data)

}

# Define score of theta function
score.theta <- function(theta, df){
  ordered <- get.ordered.data.est(lin_predictors=theta, df=df)
  score <- mean(ordered$score_i)
  return(score)
}

#' @importFrom dplyr mutate group_by summarise arrange
#' @importFrom stats uniroot
#' @exportS3Method
adjust.CovHR <- function(model, data, ...){

  # Creates data
  df <- create.tte.df(model, data)

  # Risk set and failure time times # THIS IS WHERE THE TEST FAILS
  df <- process.tte.df(df, ref_arm=model$ref_arm)
  df_process <- df # save unmodified for later

  # Get un-adjusted estimate
  score.unadj <- function(theta) score.theta(theta, df=df_process)
  thetaLhat <- stats::uniroot(score.unadj, interval=model$interval)$root

  # Perform covariate adjustment
  df <- get.ordered.data.est(df=df, lin_predictors=thetaLhat)

  # Perform covariate adjustment
  df <- get.tte.adjustment(df, model, data)

  df <- df %>%
    dplyr::mutate(
      uu_cl = .data$trt1 * .data$adjust1 - .data$trt0 * .data$adjust0,
    )
  cov_adjust <- mean(df$uu_cl)

  # Get adjusted estimate
  score.adj <- function(theta) score.theta(theta, df=df_process) - cov_adjust
  thetaCLhat <- stats::uniroot(score.adj, interval=model$interval)$root

  # Use thetaLhat rather than thetaCLhat below Both result in consistent
  # variance estimators but using thetaLhat is expected behavior.
  df <- df %>% dplyr::mutate(
    ssig_l = .data$event * exp(thetaLhat) * .data$Y0 * .data$Y1 /
      (exp(thetaLhat) * .data$Y1 + .data$Y0)**2
  )

  # Summarize by car_strata (if CL, then single car_strata)
  ss <- df %>%
    dplyr::filter(!is.na(.data$uu_cl)) %>%
    dplyr::group_by(.data$car_strata) %>%
    dplyr::summarise(
      var_adj = model$p_trt * (1 - model$p_trt) * unique(.data$bsigb) * dplyr::n(),
      .groups = "drop"
    ) %>%
    dplyr::arrange(.data$car_strata)

  # Final quantities for the C(S)L statistic
  sig2_L  <- mean(df$ssig_l)
  sig2_CL <- sig2_L - sum(ss$var_adj) / data$n
  var_est <- sig2_CL / sig2_L**2

  se_theta_L <- sqrt(1 / sig2_L / data$n)
  se_theta_CL <- sqrt(var_est / data$n)

  result <- list(
    strata_sum=ss,
    theta_L=thetaLhat,
    se_theta_L=se_theta_L,
    theta_CL=thetaCLhat,
    se_theta_CL=se_theta_CL
  )

  return(
    structure(
      class=c("TTEResultEst", "TTEResult"),
      list(result=result, settings=model, data=data)
    )
  )
}

# Fits a working model based on GLM and computes AIPW estimator

# This function allows us to implement the negative binomial model through MASS
#' @importFrom MASS glm.nb
fitmod <- function(family, ...){
  useMASS <- FALSE
  if(is.character(family)){
    if(family == "nb"){
      useMASS <- TRUE
    }
  }
  if(useMASS){
    mod <- MASS::glm.nb(...)
    # copy over the 'model' attribute to the 'data'
    # because it has a different name in glm.nb
    mod$data <- mod$model
  } else {
    mod <- stats::glm(..., family=family)
  }
  return(mod)
}

# Perform GLM adjustment, based on the classes
# of the model. Will perform adjustment based on the linear
# model type of `model` and also do G-computation or AIPW
# based on the second model type of `model`.
#' @importFrom stats setNames
#' @exportS3Method
adjust.GLMModel <- function(model, data, ...){

  # 1. Set up data frame
  df <- data$df
  columns <- colnames(df)
  columns[which(columns == data$treat_col)] <- "treat"
  columns[which(columns == data$response_col)] <- "response"
  df <- setNames(df, columns)

  # 2. Fit GLM working model
  glmod <- fitmod(
    family=model$g_family,
    stats::as.formula(data$formula),
    data=df)

  # 3. Predict potential outcome for each treatment group
  # to compute \hat{\mu}_a for a = 1, ..., k
  predict_a <- function(a){

    # Set up data frame
    df_a <- df
    df_a$treat <- rep(a, data$n)
    df_a$treat <- factor(df_a$treat, levels=data$treat_levels)

    # Create predictions
    predictions <- stats::predict(glmod, newdata=df_a, type="response")

    return(predictions)
  }

  preds <- lapply(data$treat_levels, predict_a)
  muhat <- do.call(cbind, preds)

  # 4. Save the g-computation estimate of \theta for diagnostic purposes
  g.estimate <- colMeans(muhat)

  # 5. Use the \hat{\mu} to compute the adjusted \hat{\mu} for AIPW estimator
  glmod.adjusted <- get.mutilde(model, data, muhat)
  mutilde <- glmod.adjusted$mutilde

  # 6. Compute AIPW estimator of \theta
  estimate <- colMeans(mutilde)

  # 7. Compute the asymptotic variance
  asympt.variance <- vcov_car(model, data, glmod, mutilde)
  df_adjust <- glmod.adjusted$df_adjust
  vcov_wt <- get.vcovHC(model$vcovHC, n=data$n, p=df_adjust)
  variance <- asympt.variance * vcov_wt / data$n

  # 8. Format results
  result <- format_results(data$treat_levels, estimate, variance)

  return(
    structure(
      class="GLMModelResult",
      list(result=result, varcov=variance, settings=model,
           data=data, mod=glmod, mu_a=mutilde, g.estimate=g.estimate)
    )
  )
}

# Creates a time-to-event dataset
# to be directly used in adjustment.
create.tte.df <- function(model, data){

  df <- data.frame(
    treat=data$treat,
    response=data$response,
    event=data$event,
    nu_d=nu.d(model$car_scheme)
  )
  if ("return_influence" %in% names(model)){
    if (model$return_influence) {
      df$id <- data$id
    }
  }
  # Adjust for x-covariates
  if(model$adj_cov){
    df <- cbind(df, data$covariate)
  }
  # Adjust for z-covariates
  if(model$adj_strata){
    df$carcov_z <- data$joint_strata
  } else {
    df$carcov_z <- 0
  }
  # Covariate-adaptive randomization
  if(model$car_strata){
    df$car_strata <- data$joint_strata
  } else {
    df$car_strata <- 0
  }

  return(df)

}

# Pre-process the time to event dataset by calculating
# the risk set size, and fixing ties in the failure times.
#' @importFrom dplyr n
process.tte.df <- function(df, ref_arm=NULL){

  # Get the treatment column and set the reference group
  trts <- levels(df$treat)
  if(!is.null(ref_arm)){
    index <- which(trts == ref_arm)
    trts <- c(ref_arm, trts[-index])
  }

  # Calculate risk set sizes at each failure time
  df <- df %>%
    dplyr::arrange(.data$car_strata, .data$response) %>%
    group_by(.data$car_strata) %>%
    mutate(
      Y=dplyr::n():1,
      trt0=as.integer(.data$treat == trts[1]),
      trt1=as.integer(.data$treat == trts[2]),
      Y0=cumsum(.data$trt0[dplyr::n():1])[dplyr::n():1],
      Y1=cumsum(.data$trt1[dplyr::n():1])[dplyr::n():1]) %>%
    ungroup() %>%
    group_by(.data$car_strata, .data$response) %>%
    mutate(
      Y=first(.data$Y),
      Y1=first(.data$Y1),
      Y0=first(.data$Y0),
      n.events=sum(.data$event)
    ) %>%
    ungroup() %>%
    group_by(.data$car_strata) %>%
    mutate(
      cum.delta.Y0.over.Ysq=cumsum(.data$Y0/.data$Y^2*.data$event),
      cum.delta.Y1.over.Ysq=cumsum(.data$Y1/.data$Y^2*.data$event)
    ) %>%
    group_by(.data$car_strata, .data$response) %>%
    mutate(
      cum.delta.Y0.over.Ysq=last(.data$cum.delta.Y0.over.Ysq),
      cum.delta.Y1.over.Ysq=last(.data$cum.delta.Y1.over.Ysq)
    ) %>%
    ungroup()

  return(df)
}

#' @importFrom dplyr group_by mutate ungroup n
get.ordered.data <- function(df, ref_arm){

  df <- df %>%
    group_by(.data$car_strata) %>%
    mutate(
      mu_t            = .data$Y1 / .data$Y,

      O.hat1          = .data$event * (.data$Y0/.data$Y) -
        .data$cum.delta.Y0.over.Ysq,
      O.hat0          = .data$event * (.data$Y1/.data$Y) -
        .data$cum.delta.Y1.over.Ysq,
      O.hat           = .data$trt1 * .data$O.hat1 +
        .data$trt0 * .data$O.hat0,

      s0_seq          = exp(.data$lin_pred),
      s1_seq          = .data$s0_seq * .data$trt1,

      s0              = cumsum(.data$s0_seq[dplyr::n():1])[dplyr::n():1]/dplyr::n(),
      s1              = cumsum(.data$s1_seq[dplyr::n():1])[dplyr::n():1]/dplyr::n(),

      mu_t            = .data$s1 / .data$s0,

      mean_at_risk    = .data$event / (dplyr::n()*.data$s0),
      cumsum_at_risk  = cumsum(.data$mean_at_risk),

      mean_at_risk2   = .data$event / (dplyr::n()*.data$s0) * .data$mu_t,
      cumsum_at_risk2 = cumsum(.data$mean_at_risk2),

      O_i             = .data$event * (.data$trt1 - .data$mu_t) -
        .data$s1_seq * .data$cumsum_at_risk +
        .data$s0_seq * .data$cumsum_at_risk2,

      u_i             = .data$event * (.data$trt1 - .data$mu_t)
    ) %>% ungroup()

  return(df)
}

#' @importFrom dplyr mutate group_by ungroup everything
#' @importFrom stats model.matrix
get_design_matrix <- function(df, covnames){

  formula <- stats::as.formula(
    paste0("~ 1 + ", paste0(covnames, collapse="+"))
  )
  # Get design matrix based on formula
  # remove intercept column because will use centered covariates
  mat <- stats::model.matrix(formula, df)[,-1]
  mat <- data.frame(mat)
  colnames(mat) <- paste0("xmat_", colnames(mat))

  # Center each column of the design matrix
  # This shouldn't do anything for the covariates, because
  # they should already be centered in the .make.data function,
  # but this will center the car_strata variables also, if they are included.
  mat <- data.frame(mat) %>%
    dplyr::mutate(car_strata=df$car_strata) %>%
    dplyr::group_by(.data$car_strata, ) %>%
    dplyr::mutate(dplyr::across(
      .cols=everything(),
      .fns=list(center=~scale(., center=TRUE, scale=FALSE)),
      .names="{col}_{fn}"
    )) %>%
    dplyr::ungroup() %>%
    subset(select=-car_strata)
  return(mat)
}

#' @importFrom dplyr mutate
#' @importFrom tidyr replace_na
check.collinearity <- function(df, covnames, stratified){

  if(!stratified){
    if(anyNA(df$estimate)){
      .lin.dep.error()
    }
  } else {
    names <- unique(df$term)
    x_covnames <- names[grepl("^xmat_", names)]
    x_covnames <- x_covnames[!grepl("carcov_z", x_covnames)]
    if(all(is.na(df %>% dplyr::filter(grepl("xmat_", .data$term)) %>% .$estimate))){
      .lin.dep.strat.error()
    }
  }

  df <- df %>% dplyr::mutate(
    estimate=tidyr::replace_na(.data$estimate, 0)
  )

  return(df)
}

#' @import broom
#' @importFrom dplyr group_by group_modify ungroup select
#' @importFrom tidyr pivot_wider
regress_to_Ohat <- function(df, stratified){

  # Get design matrix covariate variables, using uncentered variables
  covnames <- colnames(df)[grepl("^xmat_", colnames(df))]
  covnames <- covnames[!grepl("_center", covnames)]

  # Perform a linear regression by treatment group
  res <- df %>%
    dplyr::group_by(.data$trt1) %>%
    dplyr::group_modify(~broom::tidy(
      lm(O.hat ~ 1 + ., data=.x %>%
           select("O.hat", covnames))
    ))
  res <- check.collinearity(res, covnames, stratified)

  # Extract the coefficients
  betas <- res %>%
    dplyr::ungroup() %>%
    tidyr::expand_grid(
      # car_strata-specific betas (same)
      car_strata=unique(df$car_strata), .) %>%
    dplyr::select(.data$car_strata,
                  .data$trt1,
                  .data$term,
                  .data$estimate) %>%
    tidyr::pivot_wider(
      names_from=c(.data$trt1, .data$term),
      names_prefix="trt1_",
      values_from=.data$estimate
    )

  return(betas)
}

#' @importFrom dplyr left_join mutate group_by group_modify
calculate.adjustment <- function(df, betas, covnames, stratified){

  dat <- dplyr::left_join(df, betas, by="car_strata")

  if(stratified){
    covnames_carcov <- grepl("carcov_z", covnames)
    covnames_use <- c(covnames)[!covnames_carcov]
  } else {
    covnames_use <- covnames
  }

  # If there are some covariates:
  if(length(covnames_use > 0)){
    covnames_x <- paste0("xmat_", covnames_use)
    covnames_xc <- paste0("xmat_", covnames_use, "_center")
    beta1_names <- paste0("trt1_1_xmat_", covnames_use)
    beta0_names <- paste0("trt1_0_xmat_", covnames_use)

    dat <- dat %>%
      dplyr::group_by(.data$car_strata) %>%
      dplyr::group_modify(~ {
        matcx   <- dplyr::select(.x, all_of(covnames_xc)) %>% as.matrix(ncol=p)
        matx    <- dplyr::select(.x, all_of(covnames_x)) %>% as.matrix(ncol=p)
        b1      <- dplyr::select(.x, all_of(beta1_names)) %>% head(n=1) %>% as.matrix(ncol=p)
        b0      <- dplyr::select(.x, all_of(beta0_names)) %>% head(n=1) %>% as.matrix(ncol=p)
        adjust1 <- matcx %*% t(b1)
        adjust0 <- matcx %*% t(b0)
        bsigb   <- as.numeric((b1+b0) %*% var(matx) %*% t(b1 + b0))
        out     <- data.frame(adjust1, adjust0, bsigb) %>% dplyr::as_tibble()
        cbind(.x,out)
      })
  } else {
    dat <- dat %>% dplyr::mutate(
      adjust1 = 0,
      adjust0 = 0,
      bsigb   = 0
    )
  }
  return(dat)
}

get.tte.adjustment <- function(df, model, data){
  if(model$adj_strata | model$adj_cov){
    # Add covariate names in order
    # with the car_strata covariates first
    # so that if there is collinearity, they
    # are chosen first over the x covariates.
    covnames <- c()
    if(model$adj_strata){
      covnames <- c(covnames, "carcov_z")
    }
    if(model$adj_cov){
      covnames <- c(covnames, names(data$covariate))
    }
    # Get design matrix and centered versions of variables
    xmat <- get_design_matrix(df, covnames)
    df <- cbind(df, xmat)

    # Regress to Ohat -- get betas, then calculate adjustment using betas
    betas <- regress_to_Ohat(df, stratified=(model$method == "CSL"))

    # Get new covariate names based on the design matrix (helpful for factors)
    new_covnames <- colnames(xmat)
    new_covnames <- gsub("xmat_", "", new_covnames)
    new_covnames <- gsub("_center", "", new_covnames)
    new_covnames <- unique(new_covnames)

    # Calculate adjustment using betas
    df <- calculate.adjustment(df, betas,
                               covnames=new_covnames,
                               stratified=(model$method == "CSL"))
  } else {
    df <- df %>%
      dplyr::mutate(
        adjust1 = 0,
        adjust0 = 0,
        bsigb   = 0
      )
  }
  return(df)
}

#' @importFrom survival coxph Surv
#' @importFrom stats model.matrix
get.linear.predictor <- function(df, covnames){

  coxdf <- df[, c("response", "event", covnames)]

  # Fit a cox PH model using only the covariates to adjust for
  fit <- survival::coxph(survival::Surv(response, event) ~ ., data=coxdf)
  beta <- fit$coefficients

  # Get the design matrix with only the covariates
  design <- stats::model.matrix(~ 0 + ., data=coxdf[, -c(1, 2)])
  lin_preds <- as.numeric(design %*% beta)

  return(lin_preds)
}

#' @importFrom stats var
#' @importFrom dplyr n
get.car_strata.sum <- function(df, n, p_trt, sparse_remove=FALSE){

  ss <- df %>%
    dplyr::group_by(.data$carcov_z, .drop=TRUE) %>%
    dplyr::summarise(
      cond_var0       = stats::var(.data$O_i[.data$trt0 == 1]),
      cond_var1       = stats::var(.data$O_i[.data$trt1 == 1]),
      cond_mean0      = mean(-.data$O_i[.data$trt0 == 1]),
      cond_mean1      = mean(.data$O_i[.data$trt1 == 1]),
      prob_z          = dplyr::n() / n,
      nu_d            = unique(.data$nu_d),
      .groups = "drop"
    ) %>%
    dplyr::mutate(na_ind=is.na(.data$cond_var0 + .data$cond_var1))

  if(sparse_remove){
    if(any(ss$na_ind)){
      .sparse.car_strata.warn()
      ss <- ss %>% dplyr::filter(!.data$na_ind)
    } else {
      ss <- ss %>% tidyr::replace_na(
        list(cond_var0 = 0, cond_var1 = 0)
      )
    }
  }

  ss <- ss %>%
    dplyr::mutate(
      z_var = .data$prob_z * (p_trt * .data$cond_var0 +
                        (1 - p_trt) * .data$cond_var1 +
                        .data$nu_d * (.data$cond_mean0 + .data$cond_mean1)^2)
    )

  return(ss)
}

#' @exportS3Method
adjust.CoxScore <- function(model, data, ...){

  # Creates data
  df <- create.tte.df(model, data)
  # Risk set and failure time times
  df <- process.tte.df(df, ref_arm=model$ref_arm)

  # If there are covariates to adjust for,
  # fit a Cox model to get linear predictor
  if(model$adj_cov){
    lin_preds <- get.linear.predictor(df,
                                      covnames=colnames(data$covariate))
    df <- df %>% dplyr::mutate(lin_pred=lin_preds)
  } else {
    df <- df %>% dplyr::mutate(lin_pred=0)
  }

  df <- get.ordered.data(df, ref_arm=model$ref_arm)
  strata_sum <- get.car_strata.sum(df,
                               n=data$n,
                               p_trt=model$p_trt,
                               sparse_remove=model$sparse_remove)

  # If there's stratification this calculates the
  # Robust Stratified Score Test (EQ #21)
  numerator <- sum(df$u_i)
  denominator <- sqrt(sum(strata_sum$z_var) * data$n)
  statistic <- numerator / denominator

  result <- list(
    U=numerator,
    se=denominator,
    statistic=statistic
  )

  return(
    structure(
      class="TTEResult",
      list(result=result, settings=model, data=data)
    )
  )
}

#' Covariate adjustment using generalized linear working model
#'
#' Estimate treatment-group-specific response means and (optionally)
#' treatment group contrasts using a generalized linear working model.
#'
#' The output is the AIPW estimator given by (for each treatment group \eqn{a}):
#'
#' \deqn{\frac{1}{n} \sum_{i=1}^{n} \hat{\mu}_a(X_i) + \frac{1}{n_a} \sum_{i:A_i=a} \{Y_i - \hat{\mu}(X_i)\}}
#'
#' where \eqn{Y_i} is the outcome, \eqn{A_i} is the treatment assignment, \eqn{X_i} are the covariates, \eqn{n_a = \sum_{i=1}^{n} A_i=a},
#' and \eqn{\hat{\mu}_a} is the estimated conditional mean function based on the GLM working model.
#' This working model has treatment \eqn{a}-specific coefficients if `adj_method` is "heterogeneous". Otherwise, they are shared across the treatment arms.
#' Alternatively, if `formula` is used, the working model can be specified according to the user.
#'
#' Importantly, the estimated variance accounts for misspecification of the working model, and for covariate-adaptive randomization.
#' The variance estimator is given by
#' \deqn{\hat{V} = \hat{V}_{\rm SR} - \hat{V}_{\Omega}}
#' where \eqn{\hat{V}_{\rm SR}} is the contribution to the variance under simple randomization, and \eqn{\hat{V}_{\Omega}} is a term
#' that only appears when a covariate-adaptive randomization scheme is used.
#' The \eqn{\hat{V}_{\Omega}} is the second line of \eqn{\hat{V}} in \insertCite{bannickGeneralFormCovariate2025}{RobinCar}.
#'
#' There are three different estimators available for \eqn{\hat{V}_{\rm SR}}, which the user
#' can choose with the argument \code{variance_type}. We describe these here.
#'
#' The three variance types are given as follows:
#'
#' \itemize{
#'  \item{
#'    Type 1 (default):
#'    \deqn{\mathrm{diag}\left[\hat{\pi}_a^{-1} (\mathrm{Var}_a(Y_i) - 2\hat{Q}_{a,a} + \hat{\Sigma}_{a,a} ), a = 1, \dots, K \right] + \hat{Q} + \hat{Q}^T - \hat{\Sigma}}
#'  }
#'  \item{
#'    Type 2:
#'    \deqn{\mathrm{diag}\left[\hat{\pi}_a^{-1} (\mathrm{Var}_a(Y_i - \hat{\mu}_a(X_i)) - 2\hat{Q}_{a,a} + \hat{\Sigma}_{a,a} ), a = 1, \dots, K \right] + \hat{Q} + \hat{Q}^T - \hat{\Sigma}}
#'  }
#'  \item{
#'    Type 3:
#'    \deqn{\mathrm{diag}\left[\hat{\pi}_a^{-1} E_a([Y_i - \hat{\mu}_a(X_i)]^2), a = 1, \dots, K \right] + \hat{A}}
#'  }
#' }
#'
#' where \eqn{\hat{\pi}_a} is the treatment proportion for group a,
#' \eqn{\hat{Q}_{a,b} = \mathrm{Cov}_a(Y_i, \hat{\mu}_b(X_i))},
#' \eqn{\hat{\Sigma}_{a,b} = \mathrm{Cov}(\hat{\mu}_a(X_i), \hat{\mu}_b(X_i))},
#' and the matrix \eqn{\hat{A}} has diagonal entries for \eqn{(a, a)} given by
#' \deqn{2\mathrm{Cov}_a(Y_i - \hat{\mu}_a(X_i), \hat{\mu}_a(X_i)) + \mathrm{Var}_a(\hat{\mu}_a(X_i))}
#' and off-diagonal entries for \eqn{(a, b)} given by
#' \deqn{\mathrm{Cov}_a(Y_i, \hat{\mu}_b(X_i)) + \mathrm{Cov}_b(Y_i, \hat{\mu}_a(X_i)) - (1/2) \left[\mathrm{Cov}_a(\hat{\mu}_a(X_i), \hat{\mu}_b(X_i)) + \mathrm{Cov}_b(\hat{\mu}_a(X_i), \hat{\mu}_b(X_i)) \right].}
#' We use \eqn{E_a}, \eqn{\mathrm{Var}_a}, and \eqn{\mathrm{Cov}_a} to refer to the empirical expectation, variance, and
#'  covariance among observations in group a only, and \eqn{\mathrm{Cov}} is the covariance within
#' the entire sample.
#'
#' Please see the Supplemental Material Sect. H of \insertCite{bannickGeneralFormCovariate2025}{RobinCar} for a discussion
#' of the merits of each type of variance estimator. Briefly, we recommend
#' variance types 1 generally, and variance type 3 if it is anticipated
#' that the distribution of \eqn{X} varies substantially over treatment groups.
#'
#' @param df A data.frame with the required columns
#' @param treat_col Name of column in df with treatment variable
#' @param response_col Name of the column in df with response variable
#' @param formula The formula to use for adjustment specified using as.formula("..."). This overrides car_strata_cols and covariate_cols.
#' @param car_strata_cols Names of columns in df with car_strata variables
#' @param car_scheme Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".
#' @param contrast_h An optional function to specify a desired contrast
#' @param contrast_dh An optional jacobian function for the contrast (otherwise use numerical derivative)
#' @param g_family Family that would be supplied to glm(...), e.g., binomial. If no link specified, will use default link, like behavior in glm.
#'                 If you wish to use a negative binomial working model with an unknown dispersion parameter, then use `g_family="nb"`.
#' @param g_accuracy Level of accuracy to check prediction un-biasedness.
#' @param variance_type The type of variance estimator to use, type 1, 2, or 3. All three are asymptotically equivalent. See details for more.
#' @export
#'
#' @importFrom Rdpack reprompt
#'
#' @returns If `contrast_h` argument is used, outputs a `main` and a `contrast` object. The `main` object has the following structure:
#'
#'  \item{result}{A \link[dplyr:tibble]{dplyr::tibble()} with the treatment label, treatment mean estimate using AIPW, estimated SE, and p-value based on a z-test with estimate and SE.}
#'  \item{varcov}{The variance-covariance matrix for the treatment mean estimates.}
#'  \item{settings}{List of model settings used in covariate adjustment.}
#'  \item{original_df}{The original dataset provided by the user.}
#'  \item{mod}{The fit from the \link[stats:glm]{glm()} working model used for covariate adjustment.}
#'  \item{mu_a}{Predicted potential outcomes for each treatment category (columns) and individual (rows). These are the \eqn{\hat{\mu}_a}}.
#'  \item{g.estimate}{The G-computation estimate based only on \eqn{\frac{1}{n} \sum_{i=1}^{n} \hat{\mu}_a(X_i)}. This is equivalent to the AIPW estimate when a canonical link function is used.}
#'  \item{data}{Attributes about the dataset.}
#'
#'  The `contrast` object has a structure that is documented in \link[RobinCar:robincar_contrast]{RobinCar::robincar_contrast()}.
#'
#' @references
#'   \insertAllCited{}
robincar_glm <- function(df,
                         treat_col, response_col,
                         formula=NULL, car_strata_cols=NULL,
                         car_scheme="simple",
                         g_family=stats::gaussian, g_accuracy=7,
                         contrast_h=NULL, contrast_dh=NULL,
                         variance_type=1){

  .check.car_scheme(car_scheme, car_strata_cols)
  .check.variance_type(variance_type)

  vcovHC <- "HC0"

  data <- .make.data(
    df=df, classname="RoboDataGLM",
    treat_col=treat_col,
    response_col=response_col,
    car_strata_cols=car_strata_cols,
    formula=formula
  )
  validate(data)

  # Create model object
  model <- .make.model(
    data=data,
    car_scheme=car_scheme,
    vcovHC=vcovHC,
    g_family=g_family,
    g_accuracy=g_accuracy,
    variance_type=variance_type
  )

  # Perform adjustment
  result <- adjust(model, data)
  # This is to save the original dataset for calibration add_x later on
  result$original_df <- df

  # Create transformation object
  if(!is.null(contrast_h)){
    c_result <- robincar_contrast(
      result=result,
      contrast_h=contrast_h,
      contrast_dh=contrast_dh
    )
    result <- list(main=result, contrast=c_result)
  }

  return(result)
}

test_that("Test contrast function ordering", {

  data <- RobinCar:::data_sim %>%
    # recode A from 0 to "a", 1 to "b", 2 to "c"
    mutate(A=factor(A, levels=0:2, labels=c("b", "c", "a")))

  my.f<-function(x){
    x[2]-x[1] # should be "c"-"b" based on the ordering
  }

  fit.anova <- robincar_linear(df = data,
                               response_col="y",
                               treat_col="A",
                               car_scheme="simple",
                               adj_method="ANOVA",
                               contrast_h=my.f)

  # Check levels are in the right order
  lev <- fit.anova$main$result$treat
  expect_equal(lev, c("b", "c", "a"))

  # Check that the estimates are in correct order
  anova <- mean(data$y[data$A == "c"]) - mean(data$y[data$A == "b"])
  expect_equal(as.vector(fit.anova$contrast$result$estimate[1]), anova)

})

test_that("Test warnings for ratio and odds ratio functions", {

  data <- RobinCar:::data_sim %>%
    # recode A from 0 to "a", 1 to "b", 2 to "c"
    mutate(A=factor(A, levels=0:2, labels=c("a", "b", "c"))) %>%
    mutate(y_bin=ifelse(y > mean(y), 1, 0)) # create binary outcome

  warning_txt <- "Using a ratio or odds ratio is not recommended.
            Consider using log_ratio or log_odds_ratio for better
            performance of the variance estimator."

  # Test for linear model with ratio
  expect_warning(
    fit <- robincar_glm(
      df=data,
      response_col="y_bin",
      treat_col="A",
      formula= y_bin ~ A,
      contrast_h="odds_ratio"
    ),
    warning_txt
  )

  # Test for linear model with ratio
  expect_warning(
    fit <- robincar_glm(
      df=data,
      response_col="y",
      treat_col="A",
      formula= y ~ A,
      contrast_h="ratio"
    ),
    warning_txt
  )

})



test_that("Test warnings for contrast functions that are not probabilities", {
  # We want these to give warnings when someone
  # specifies that they want a log odds ratio.

  data <- RobinCar:::data_sim %>%
    # recode A from 0 to "a", 1 to "b", 2 to "c"
    mutate(A=factor(A, levels=0:2, labels=c("a", "b", "c"))) %>%
    mutate(y_bin=ifelse(y > mean(y), 1, 0)) # create binary outcome

  # Test for linear model with ratio
  expect_warning(
    fit <- robincar_glm(
      df=data,
      response_col="y",
      treat_col="A",
      formula= y ~ A,
      contrast_h="log_odds_ratio"
    ),
    "Estimates are not between 0 and 1: are you sure you want an odds ratio?"
  )

})

test_that("Compare log odds ratio SEs to GLM", {
  # We want these to give warnings when someone
  # specifies that they want a log odds ratio.

  data <- RobinCar:::data_sim %>%
    # recode A from 0 to "a", 1 to "b", 2 to "c"
    mutate(A=factor(A, levels=0:2, labels=c("a", "b", "c"))) %>%
    mutate(y_bin=ifelse(y > mean(y), 1, 0)) # create binary outcome

  # Test for linear model with ratio
  fit <- robincar_glm(
    df=data,
    response_col="y_bin",
    treat_col="A",
    formula= y_bin ~ A,
    contrast_h="log_odds_ratio"
  )

  or <- with(data, glm(y_bin ~ A, family=binomial(link="logit")))

  # Check that log odds ratios match
  expect_equal(
    unname(or$coefficients[2:3]),
    unname(fit$contrast$result$estimate), tolerance=1e-6)

})

library(matrixcalc)

DATA <- RobinCar:::data_sim
DATA$A <- as.factor(DATA$A)
DATA$y_bin <- ifelse(DATA$y > 2, 1, 0)

test_that("ANCOVA for GLM vartypes.", {

  res1 <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A + x1",
    variance_type=1)

  res2 <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A + x1", variance_type=2)

  res3 <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A + x1",
    variance_type=3)

  # These will be equal -- variance type should not
  # change the point estimate
  expect_equal(res1$result$estimate, res2$result$estimate)
  expect_equal(res2$result$estimate, res3$result$estimate)

  # These won't be exactly equivalent, only asymptotically
  # Just make sure things aren't really different
  expect_equal(res1$result$se, res2$result$se, tolerance=1e-2)
  expect_equal(res1$result$se, res3$result$se, tolerance=1e-2)

  # Check that variance type 3 is PD matrix -- this should
  # be true for ANCOVA
  expect_true(is.positive.definite(res3$varcov))

})

library(MASS)

DATA <- RobinCar:::data_sim
DATA$A <- as.factor(DATA$A)
DATA$y_bin <- ifelse(DATA$y > 2, 1, 0)

# LINEAR V. GLM WITHOUT Z ----------------------------------- #

test_that("GLM full function -- linear (ANOVA)", {
  lin <- robincar_linear(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    adj_method="ANOVA")
  non <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A")
  expect_equal(class(non), "GLMModelResult")

  # Check that the result from the linear and glm function is the same
  # when using the identity link.
  est_lin <- lin$result$estimate
  names(est_lin) <- NULL
  est_non <- non$result$estimate
  names(est_non) <- NULL
  expect_equal(est_lin, est_non, tolerance=1e-10)

  # These won't be exactly equivalent, only asymptotically (?)
  var_lin <- lin$result$se
  names(var_lin) <- NULL
  var_non <- non$result$se
  names(var_non) <- NULL
  expect_equal(var_lin, var_non, tolerance=1e-2)
})

test_that("GLM full function -- linear (ANCOVA)", {
  lin <- robincar_linear(
    df=DATA,
    response_col="y",
    treat_col="A",
    covariate_cols=c("x1"),
    car_scheme="simple",
    adj_method="ANCOVA")
  non <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A + x1")
  expect_equal(class(non), "GLMModelResult")

  # Check that the result from the linear and glm function is the same
  # when using the identity link.
  est_lin <- lin$result$estimate
  names(est_lin) <- NULL
  est_non <- non$result$estimate
  names(est_non) <- NULL
  expect_equal(est_lin, est_non, tolerance=1e-10)

  # These won't be exactly equivalent, only asymptotically (?)
  var_lin <- lin$result$se
  names(var_lin) <- NULL
  var_non <- non$result$se
  names(var_non) <- NULL
  expect_equal(var_lin, var_non, tolerance=1e-2)
})

test_that("GLM full function -- linear (ANHECOVA)", {
  lin <- robincar_linear(
    df=DATA,
    response_col="y",
    treat_col="A",
    covariate_cols=c("x1"),
    car_scheme="simple",
    adj_method="ANHECOVA")
  non <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A * x1")
  expect_equal(class(non), "GLMModelResult")

  # Check that the result from the linear and glm function is the same
  # when using the identity link.
  est_lin <- lin$result$estimate
  names(est_lin) <- NULL
  est_non <- non$result$estimate
  names(est_non) <- NULL
  expect_equal(est_lin, est_non, tolerance=1e-10)

  # These won't be exactly equivalent, only asymptotically (?)
  var_lin <- lin$result$se
  names(var_lin) <- NULL
  var_non <- non$result$se
  names(var_non) <- NULL
  expect_equal(var_lin, var_non, tolerance=1e-2)
})

# LINEAR V. GLM WITH Z ----------------------------------- #

test_that("GLM full function -- linear (ANCOVA) with Z and simple randomization", {
  lin <- expect_warning(robincar_linear(
    df=DATA,
    response_col="y",
    treat_col="A",
    covariate_cols=c("x1"),
    car_strata_cols=c("z1"),
    car_scheme="simple",
    adj_method="ANCOVA"))
  non <- expect_warning(robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    car_strata_cols=c("z1"),
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A + x1"))

  # Check that the result from the linear and glm function is the same
  # when using the identity link.
  est_lin <- lin$result$estimate
  names(est_lin) <- NULL
  est_non <- non$result$estimate
  names(est_non) <- NULL
  expect_equal(est_lin, est_non, tolerance=1e-10)

  # These won't be exactly equivalent, only asymptotically (?)
  var_lin <- lin$result$se
  names(var_lin) <- NULL
  var_non <- non$result$se
  names(var_non) <- NULL
  expect_equal(var_lin, var_non, tolerance=1e-2)
})

test_that("GLM full function -- linear (ANCOVA) with Z", {
  lin <- robincar_linear(
    df=DATA,
    response_col="y",
    treat_col="A",
    covariate_cols=c("x1"),
    car_strata_cols=c("z1"),
    car_scheme="permuted-block",
    adj_method="ANCOVA")
  non <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="permuted-block",
    car_strata_cols=c("z1"),
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A + x1")
  expect_equal(class(non), "GLMModelResult")

  # Check that the result from the linear and glm function is the same
  # when using the identity link.
  est_lin <- lin$result$estimate
  names(est_lin) <- NULL
  est_non <- non$result$estimate
  names(est_non) <- NULL
  expect_equal(est_lin, est_non, tolerance=1e-10)

  # These won't be exactly equivalent, only asymptotically (?)
  var_lin <- lin$result$se
  names(var_lin) <- NULL
  var_non <- non$result$se
  names(var_non) <- NULL
  expect_equal(var_lin, var_non, tolerance=1e-2)
})

test_that("GLM full function -- linear (ANHECOVA) with Z", {
  lin <- robincar_linear(
    df=DATA,
    response_col="y",
    treat_col="A",
    covariate_cols=c("x1", "z1"),
    car_strata_cols=c("z1"),
    car_scheme="permuted-block",
    adj_method="ANHECOVA")
  non <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A",
    car_scheme="permuted-block",
    car_strata_cols=c("z1"),
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A * (x1 + z1)")
  expect_equal(class(non), "GLMModelResult")

  # Check that the result from the linear and glm function is the same
  # when using the identity link.
  est_lin <- lin$result$estimate
  names(est_lin) <- NULL
  est_non <- non$result$estimate
  names(est_non) <- NULL
  expect_equal(est_lin, est_non, tolerance=1e-10)

  # These won't be exactly equivalent, only asymptotically (?)
  var_lin <- lin$result$se
  names(var_lin) <- NULL
  var_non <- non$result$se
  names(var_non) <- NULL
  expect_equal(var_lin, var_non, tolerance=1e-2)
})

# GLM TESTS ---------------------------------------- #

n <- 1000
set.seed(10)
DATA2 <- data.frame(A=rbinom(n, size=1, prob=0.5),
                 y=rbinom(n, size=1, prob=0.2),
                 x1=rnorm(n),
                 x2=rnorm(n),
                 x3=as.factor(rbinom(n, size=1, prob=0.5)),
                 z1=rbinom(n, size=1, prob=0.5),
                 z2=rbinom(n, size=1, prob=0.5))
DATA2$A <- as.factor(DATA2$A)
DATA2$x1 <- DATA2$x1 - mean(DATA2$x1)

test_that("GLM full function -- NEGATIVE binomial, permuted block", {

  # Known dispersion parameter
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    car_scheme="permuted-block",
    g_family=negative.binomial(1),
    g_accuracy=7,
    formula="y ~ A * (x1 + z1)")
  expect_equal(class(non), "GLMModelResult")

  # Known dispersion parameter
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    car_scheme="permuted-block",
    g_family="nb",
    g_accuracy=7,
    formula="y ~ A * (x1 + z1)")
  expect_equal(class(non), "GLMModelResult")

  non

  # Test with formula
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    formula="y ~ A + z1",
    car_scheme="permuted-block",
    g_family="nb",
    g_accuracy=7)

  non

})

test_that("GLM Settings", {
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_scheme="simple",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A * x1")
  expect_equal(non$settings$pu_joint_z, FALSE)

  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    car_scheme="biased-coin",
    g_family=gaussian(link="identity"),
    g_accuracy=7,
    formula="y ~ A * (x1 + z1)")
  expect_equal(non$settings$pu_joint_z, FALSE)
})

test_that("GLM full function -- binomial, permuted block", {
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    car_scheme="permuted-block",
    g_family=binomial(link="logit"),
    g_accuracy=7,
    formula="y ~ A * (x1 + z1)")
  expect_equal(class(non), "GLMModelResult")
  expect_equal(non$result$estimate,
               c(X1=0.20774694, X2=0.15547416), tolerance=1e-5)
})


test_that("GLM full function -- binomial, pocock simon", {
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    car_scheme="pocock-simon",
    g_family=binomial(link="logit"),
    g_accuracy=7,
    formula="y ~ A * (x1 + z1)")
  expect_equal(class(non), "GLMModelResult")
  expect_equal(non$settings$pu_joint_z, TRUE)
  expect_equal(non$result$estimate,
               c(0.20774694, 0.15547416), tolerance=1e-5)
})

test_that("GLM -- no covariates", {
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    car_scheme="biased-coin",
    g_family=binomial(link="logit"),
    g_accuracy=7,
    formula="y ~ A")
  expect_equal(class(non), "GLMModelResult")
  expect_equal(length(non$mod$coefficients), 2)
})

# TEST FOR THE DMAT.
test_that("GLM -- no covariates except car_strata", {
  non <- robincar_glm(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_scheme="biased-coin",
    car_strata_cols=c("z1"),
    g_family=binomial(link="logit"),
    g_accuracy=7,
    formula="y ~ A * z1")
  expect_equal(length(non$mod$coefficients), 4)
})

n <- 2000
DATA4 <- data.frame(
  y=rbinom(n=n, prob=0.5, size=1),
  TRT01P=sample(1:3, replace=TRUE, size=n),
  BWTGR1=rbinom(n=n, prob=0.1, size=1)
)
DATA4$TRT01P <- factor(DATA4$TRT01P)

test_that("GLM -- test g_family types in print function", {
  # Test with character
  res1 <- robincar_glm(
    df=DATA4,
    response_col="y",
    treat_col="TRT01P",
    car_scheme="simple",
    g_family="binomial",
    formula="y ~ TRT01P + BWTGR1"
  )
  # Test with function
  res2 <- robincar_glm(
    df=DATA4,
    response_col="y",
    treat_col="TRT01P",
    car_scheme="simple",
    g_family=binomial,
    formula="y ~ TRT01P + BWTGR1"
  )
  # Test with object
  res3 <- robincar_glm(
    df=DATA4,
    response_col="y",
    treat_col="TRT01P",
    car_scheme="simple",
    g_family=binomial(link="logit"),
    formula="y ~ TRT01P + BWTGR1"
  )
  expect_equal(res1$result, res2$result)
  expect_equal(res1$result, res3$result)
})

DATA5 <- DATA4
DATA5$TRT01P <- factor(DATA5$TRT01P,
                       levels=1:3,
                       labels=c("placebo", "trt1", "trt2"))

test_that("GLM -- test that it does not matter if treatment levels
          are labeled or not.", {
  # Un-labeled treatment groups
  res1 <- robincar_glm(
    df=DATA4,
    response_col="y",
    treat_col="TRT01P",
    car_scheme="simple",
    g_family=binomial(link="logit"),
    formula="y ~ TRT01P + BWTGR1"
  )
  # Labeled treatment groups
  res2 <- robincar_glm(
    df=DATA5,
    response_col="y",
    treat_col="TRT01P",
    car_scheme="simple",
    g_family=binomial(link="logit"),
    formula="y ~ TRT01P + BWTGR1"
  )
  expect_equal(res1$result$estimate, res2$result$estimate)
  expect_equal(res1$result$se, res2$result$se)
})

library(MASS)
library(dplyr)

DATA <- RobinCar:::data_sim
DATA$A <- as.factor(DATA$A)
DATA$y_bin <- ifelse(DATA$y > 2, 1, 0)

# Ensure that the naming/capitalization
# of the treatment variable does not impact results.

DATA <- DATA %>% mutate(
  A2=paste0("TREAT", A),
  A3=ifelse(A2 == "TREAT1", "treat1", A2)
)

# Doing DATA %>% group_by(A3) %>% summarize(n()) will
# order the levels by c("TREAT0", "TREAT2", "treat1")
#
# whereas factor(DATA$A3) will order the levels
# by c("TREAT0", "TREAT1", "TREAT2").

test_that("Test ordering of treatment factors in GLM adjustment", {

  # The locale is actually the issue -- testthat
  # automatically sets locale C but the issue only
  # appears under locale != C. So here we set it to
  # en_US.UTF-8.

  original_locale <- Sys.getlocale("LC_COLLATE")
  Sys.setlocale("LC_COLLATE", "en_US.UTF-8")

  # English language names
  langENG <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A2",
    car_scheme="simple",
    formula="y ~ A2 + x1")

  # C language names
  langC <- robincar_glm(
    df=DATA,
    response_col="y",
    treat_col="A3",
    car_scheme="simple",
    formula="y ~ A3 + x1")

  # Check that both of the orderings worked,
  # regardless of the final order the output is in
  # -----------------------------------------

  # Benchmark ordering
  namesENG <- c("TREAT0", "TREAT1", "TREAT2")
  namesC <- c("TREAT0", "treat1", "TREAT2")

  # Current ordering
  currentENG <- langENG$result$treat
  currentC <- langC$result$treat

  # Get re-ordering indexes
  reorderENG <- unname(sapply(currentENG, function(x) which(x == namesENG)))
  reorderC <- unname(sapply(currentC, function(x) which(x == namesC)))

  # Re-order estimate output and make sure they're the same
  estENG <- unname(langENG$result$estimate[reorderENG])
  estC <- unname(langC$result$estimate[reorderC])
  expect_equal(estENG, estC)

  # Re-order variance output and make sure they're the same
  varENG <- unname(langENG$varcov[reorderENG, reorderENG])
  varC <- unname(langC$varcov[reorderC, reorderC])
  expect_equal(varENG, varC)

  # Reset the locale
  Sys.setlocale("LC_COLLATE", original_locale)

})

#' Generate simple randomization treatment assignments
#'
#' @param n Number of observations
#' @param p_trt Proportion allotted to treatment
#' @export
#'
#' @importFrom stats rbinom
#'
#' @examples
#' car_sr(10, p_trt=0.4)
#'
#' @returns A vector of treatment assignments as 0's and 1's based on simple randomization.
car_sr <- function(n, p_trt){
  I <- stats::rbinom(n, 1, p_trt)
  return(I)
}

#' Generate permuted block treatment assignments
#'
#' @param z The car_strata design matrix, as a data frame with factor variables
#' @param trt_label Treatment label
#' @param trt_alc Treatment allocation vector
#' @param blocksize Permuted block blocksize
#'
#' @export
#' @importFrom dplyr summarise_all group_by_all cur_group_id mutate
#'
#' @examples
#' # Create car_strata variables
#' library(fastDummies)
#' library(dplyr)
#'
#' x <- runif(100)
#' z <- cut(x, breaks=c(0, 0.25, 0.5, 0.75, 1.0))
#' z <- dummy_cols(z) %>%
#'      mutate(across(where(is.numeric), as.factor))
#'
#' car_pb(z[, 2:5], c(0, 1, 2), trt_alc=c(1/4, 1/2, 1/4), blocksize=4L)
#'
#' @returns A vector of treatment assignments with labels from the `trt_label` argument, based on stratified permuted block randomization.
car_pb <- function(z, trt_label, trt_alc, blocksize=4L) {

  car_strata <- z

  if(!all(summarise_all(car_strata, is.factor) == TRUE)){
   stop("Please make sure every feature(column) of z is a factor!")
  }
  if(!is.integer(blocksize)){
    stop("blocksize should be an integer, e.g., blocksize = 4L")
  }

  ntrt_blk <- as.integer(blocksize*trt_alc/sum(trt_alc))
  A_blk <- rep(trt_label, ntrt_blk)

  if(length(A_blk) != blocksize){
    stop("car.blocksize*car.trt_alc/sum(car.trt_alc) should be integers!")
  }

  strata_cross <- car_strata %>%
    group_by_all %>%
    dplyr::mutate(strata_ind = dplyr::cur_group_id()) %>%
    ungroup()

  A <- rep(NA, nrow(car_strata))
  for (i in 1 : max(strata_cross$strata_ind)){
    car_strata.index.temp <- which(strata_cross$strata_ind == i)
    car_strata.size.temp <- length(car_strata.index.temp)
    A[car_strata.index.temp] <- as.vector(
      replicate(n = ceiling(car_strata.size.temp/blocksize),
                expr = sample(x = A_blk, size = blocksize))[1:car_strata.size.temp]
    )
  }

  return(A)
}

#' Generate Pocock-Simon minimization treatment assignments
#'
#' @param z The car_strata design matrix
#' @param treat A vector of length k (the number of treatment arms), which labels the treatment arms being compared.
#' @param ratio A vector of length k (the number of treatment arms), which indicates the allocation ratio, e.g., c(1,1,1) for equal allocation with three treatment arms.
#' @param imb_measure What measure of imbalance should be minimzed during randomization -- either "Range" or "SD"
#' @param p_bc The biased probability, i.e., the probability of assigning each patient to the arm that minimizes the imbalance. Default is 0.8
#' @return
#' \describe{
#' \item{res}{treatment assignment vector}
#' }
#'
#' @author Ting Ye Yanyao Yi
#' @import tidyverse
#' @import fastDummies
#' @importFrom stats sd
#' @export
#'
#' @examples
#'
#' # Create car_strata variables
#' library(fastDummies)
#' library(dplyr)
#'
#' x <- runif(100)
#' z <- cut(x, breaks=c(0, 0.25, 0.5, 0.75, 1.0))
#' z <- dummy_cols(z)
#' A <- car_ps(
#'   z=z[, 2:5],
#'   treat=c(0, 1, 2),
#'   ratio=c(1, 1, 1),
#'   imb_measure="Range"
#' )
#'
#' @returns A vector of treatment assignments with labels from the `treat` argument, based on Pocock-Simon's minimization.
car_ps <- function(z, treat, ratio, imb_measure, p_bc=0.8){

  if(!imb_measure %in% c("Range", "SD")) stop("Unrecognized imbalance measure.")
  z <- data.frame(z)

  miniRand <- function(j, result){

    ns <- ncol(z)
    ntrt <- length(treat)
    covwt <- rep(1 / ns, ns)

    if(j > 1){
      applydat <- z[1:(j - 1), , drop = FALSE]
      matchx <- apply(applydat, 1,
                      function(x, xrow) {
                        as.numeric(x == xrow)
                      }, z[j,])

      if(is.vector(matchx)){
        matchx <- matrix(matchx, nrow=1)
        n_matchx <- matrix(0, 1, ntrt)
        for(k in 1:ntrt){
          n_matchx[, k] <- sum(matchx[1, result[1:(j - 1)] == treat[k]])
        }
      } else{
        n_matchx <- matrix(0, ncol(z), ntrt)
        for (k in 1:ntrt) {
          kdat <- as.matrix(matchx[, result[1:(j - 1)] == treat[k]])
          n_matchx[, k] <- apply(kdat, 1, sum)
        }
      }

      imbalance <- rep(0, ntrt)
      for (i in 1:ntrt) {
        temp <- n_matchx
        temp[, i] <- temp[, i] + 1
        num_level <- temp %*% diag(1 / ratio)
        if(imb_measure == "Range"){
          range_level <- apply(num_level, 1, range)
          imb_margin <- range_level[2,] - range_level[1,]
        } else {
          sd_level <- apply(num_level, 1, stats::sd)
          imb_margin <- sd_level
        }
        imbalance[i] <- sum(covwt %*% imb_margin)
      }

      trt.highprob <- treat[imbalance == min(imbalance)]
      trt.lowprob <- treat[imbalance != min(imbalance)]

      res <- ifelse(
        length(trt.highprob) < ntrt,
        sample(
          c(trt.highprob, trt.lowprob),
          1,
          replace=TRUE,
          prob=c(rep(
            p_bc / length(trt.highprob), length(trt.highprob)
          ), rep((1 - p_bc) / length(trt.lowprob), length(trt.lowprob)
          ))
        ),
        sample(
          treat,
          1,
          replace=TRUE,
          prob=rep(1 / ntrt, ntrt)
        )
      )
    }
    return(res)
  }

  # Create result vector
  A <- rep(NA, nrow(z))
  A[1] <- sample(treat, 1, replace=TRUE,
                 prob=ratio / sum(ratio))

  # Loop through rows to create treatment assignments
  for(row in 2:nrow(z)){
    A[row] <- miniRand(j=row, result=A)
  }

  return(A)
}

library(fastDummies)
library(dplyr)

test_that("sr", {
  set.seed(0)
  a <- car_sr(10000, p_trt=0.5)
  expect_equal(mean(a), 0.5, tolerance=1e-2)
})

test_that("pb", {
  set.seed(0)
  x <- runif(10000)
  z <- cut(x, breaks=c(0, 0.25, 0.5, 0.75, 1.0))
  z <- dummy_cols(z) %>%
       mutate(across(where(is.numeric), as.factor))

  a <- car_pb(z[, 2:5], c(0, 1, 2), trt_alc=c(1/4, 1/4, 2/4), blocksize=4L)
  expect_equal(mean(a == 0), 0.25, tolerance=1e-2)
  expect_equal(mean(a == 1), 0.25, tolerance=1e-2)
  expect_equal(mean(a == 2), 0.50, tolerance=1e-2)
})

test_that("ps", {
  set.seed(0)
  x <- runif(500)
  z <- cut(x, breaks=c(0, 0.25, 0.5, 0.75, 1.0))
  z <- dummy_cols(z)

  a <- car_ps(
    z=z[, 2:5],
    treat=c(0, 1, 2),
    ratio=c(1, 1, 1),
    imb_measure="Range"
  )
  expect_equal(mean(a==0), 0.33, tolerance=1e-1)
  expect_equal(mean(a==1), 0.33, tolerance=1e-1)
  expect_equal(mean(a==2), 0.33, tolerance=1e-1)
})

#' Perform linear or joint calibration
#'
#' Uses linear or joint calibration to "calibrate" the estimates from a linear or GLM-type adjustment.
#' Linear calibration fits a linear model with treatment (and treatment-by-covariate interactions) and with the predicted \eqn{\hat{\bm \mu}(X_i) = (\hat{\mu}_1(X_i), \dots, \hat{\mu}_K(X_i))} as constructed covariates where \eqn{K} is the number of treatment groups;
#' joint calibration also includes \eqn{Z_i} the strata variables as covariates.
#'
#' @param result A GLMModelResult
#' @param joint If true, then performs joint calibration
#'              with the \eqn{\hat{\bm \mu}(X_i)} and strata \eqn{Z_i}
#'              to achieve universality and efficiency gain
#'              rather than just linear calibration that uses \eqn{\hat{\bm \mu}(X_i)}.
#' @param add_x Additional x to use in the calibration. Must have been in
#'              the original dataset that robincar_glm was called on.
#' @export
#'
#' @returns A result object that has the same structure as \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}, with the argument `result` included as "original" in the list.
robincar_calibrate <- function(result, joint=FALSE,
                               add_x=NULL){

  # Get the g-computation predictions
  mu_names <- paste0("mu_", result$data$treat_levels)
  mu_a <- data.frame(result$mu_a)
  colnames(mu_a) <- mu_names

  # Get new dataset to pass to new robincar_glm func
  newdat <- cbind(result$original_df, mu_a)

  # Add on more covariates
  if(!is.null(add_x)){
    for(cname in c(add_x)){
      if(cname %in% colnames(newdat)) next
      if(!cname %in% colnames(result$original_df)) .miss.covariate.calibrate()
      newdat <- cbind(newdat, result$original_df[cname])
    }
  }

  # Run robincar_glm using the heterogeneous
  # working model and identity link
  if(joint){
    # Use JOINT strata levels with formula notation
    strata <- colnames(result$data$car_strata)
    strata <- paste0(strata, collapse="*")
    covariate_cols <- c(mu_names, add_x, strata)
  } else {
    covariate_cols <- c(mu_names, add_x)
  }

  cal_result <- robincar_linear(
    df=newdat,
    response_col=result$data$response_col,
    treat_col=result$data$treat_col,
    covariate_cols=covariate_cols,
    car_strata_cols=colnames(result$data$car_strata),
    car_scheme=result$settings$car_scheme,
    adj_method="ANHECOVA"
  )
  # Designate as calibration result
  # for the print statement to work
  class(cal_result) <- c(
    "CalibrationResult",
    class(cal_result)
  )
  # Save additional settings
  cal_result[["original"]] <- result
  cal_result[["joint"]] <- joint
  cal_result[["add_x"]] <- add_x

  return(cal_result)
}

.return.error <- function(err){
  if(length(err) > 0) stop(paste0(err, sep="\n"))
}

.x.exist.warn <- function() warning("Covariates specified, but no adjustment desired. Ignoring them.")
.z.exist.warn <- function() warning("Strata specified, but no adjustment desired. Ignoring them.")
.z.exist.warn.simple <- function() warning("Strata specified, but simple randomization chosen. Ignoring Z in adjustment.")

.x.miss.err <- function() warning("Must have some covariates specified.")
.x.miss.warn <- function() warning("Covariates not specified, but adjustment desired. Changing method to ANOVA.")
.z.miss.err <- function() stop("No car_strata specified, but covariate-adaptive randomization desired. Please provide car_strata.")

.car.min.err <- function() stop("Minimization is not compatible with desired adjustment method. Please use ANHECOVA instead.")

.z.include.warn <- function() warning("Must include car_strata variables Z as covariates, modifying the covariates to include...")

# ERRORS FOR GLM

.form.warn <- function() warning("Formula specified; ignoring arguments for covariate variables and overriding adjustment method.")

.pu.warn <- function() warning("Prediction unbiasedness does not hold.")
.pu.z.warn <- function() warning("Prediction unbiasedness does not hold in all joint levels of Z.")
.pu.z.calibrate <- function() warning("We suggest you try out robin_calibrate.")

.homogeneous.min.error <- function() stop("Inapplicable settings. Try method = 'heterogeneous', include car_strata as covariates, and a g-family with canonical link.")

.miss.covariate.calibrate <- function() stop("Cannot find add_x covariate in original dataset.")

# ERRORS FOR TTE
.csl.nostrata.warn <- function() warning("If stratified test needs to be performed under simple randomization, please specify car_scheme as permuted-block or biased-coin.")
.cox.carscheme.error <- function() stop("No applicable robust score test for the specified car_scheme, try to use robtest_logrank() function.")
.tte.nomethod.error <- function() stop("No applicable adjustment methods for the specified adj_method.")
.sparse.car_strata.warn <- function() warning("Conditional variance in some car_strata equals 0, removing these car_strata.")

.lin.dep.error <- function() warning("Removing model variables that are linearly dependent with the stratification variables.")
.lin.dep.strat.error <- function() warning("The adjusted covariates are linearly dependent on car_strata, covariate stratified logrank test reduces to the stratified logrank test.")

# FUNCTIONS TO COMPUTE THE ASYMPTOTIC VARIANCE
# OF ESTIMATES FROM SIMPLE AND COVARIATE-ADAPTIVE RANDOMIZATION

# Gets the diagonal sandwich variance component
# for all linear models in the asymptotic variance formula.
#' @importFrom rlang .data
#' @importFrom dplyr mutate group_by summarize
vcov_sr_diag <- function(data, mod, residual=NULL){
  # Calculate the SD of the residuals from the model fit (or responses),
  # in order to compute sandwich variance -- this is
  # the asymptotic variance, not yet divided by n.
  if(is.null(residual)){
    residual <- stats::residuals(mod)
  }
  result <- dplyr::tibble(
    resid=residual,
    treat=data$treat
  ) %>%
    dplyr::group_by(.data$treat) %>%
    dplyr::summarize(se=stats::sd(.data$resid), .groups="drop") %>%
    dplyr::mutate(se=.data$se*sqrt(1/data$pie))

  return(diag(c(result$se)**2))
}

#' @importFrom rlang .data
#' @importFrom dplyr filter
get.erb <- function(model, data, mod, mu_hat=NULL){

  if(is.null(mu_hat)){
    mu_hat <- stats::predict(mod)
  }
  residual <- data$response - mu_hat

  # Calculate Omega Z under simple
  omegaz_sr <- omegaz.closure("simple")(data$pie)

  # Adjust this variance for Z

  # Calculate Omega Z under the covariate adaptive randomization scheme
  # Right now this will only be zero, but it's here for more generality
  # if we eventually include the urn design
  omegaz <- model$omegaz_func(data$pie)

  # Calculate the expectation of the residuals within each level of
  # car_strata variables Z
  dat <- dplyr::tibble(
    treat=data$treat,
    car_strata=data$joint_strata,
    resid=residual
  ) %>%
    dplyr::group_by(.data$treat, .data$car_strata) %>%
    dplyr::summarize(mean=mean(.data$resid), .groups="drop")

  # Calculate car_strata levels and proportions for
  # the outer expectation
  strata_levels <- data$joint_strata %>% levels
  strata_props <- data$joint_strata %>% table %>% proportions

  # Estimate R(B) by first getting the conditional expectation
  # vector for a particular car_strata (vector contains
  # all treatment groups), then dividing by the pi_t
  .get.cond.exp <- function(s) dat %>%
    dplyr::filter(.data$car_strata==s) %>%
    dplyr::arrange(.data$treat) %>%
    dplyr::pull(mean)

  .get.rb <- function(s) diag(.get.cond.exp(s) / c(data$pie))

  # Get the R(B) matrix for all car_strata levels
  rb_z <- lapply(strata_levels, .get.rb)

  # Compute the R(B)[Omega_{SR} - Omega_{Z_i}]R(B) | Z_i
  # for each Z_i
  rb_omega_rb_z <- lapply(rb_z, function(x) x %*% (omegaz_sr - omegaz) %*% x)

  # Compute the outer expectation
  ERB <- mapply(function(x, y) x*y, x=rb_omega_rb_z, y=strata_props, SIMPLIFY=FALSE)
  ERB <- Reduce("+", ERB)
  return(ERB)
}

# Gets AIPW asymptotic variance under simple randomization
#' @importFrom stats cov
#' @importFrom stats var
vcov_car <- function(model, data, mod, mutilde){

  # Get predictions for observed treatment group
  preds <- matrix(nrow=data$n, ncol=1)
  for(t_id in 1:length(data$treat_levels)){
    t_group <- data$treat == data$treat_levels[t_id]
    preds[t_group] <- mutilde[, t_id][t_group]
  }

  # Compute residuals for the diagonal portion
  residual <- data$response - preds

  # Get covariance between observed Y and predicted \mu counterfactuals
  get.cov.Ya <- function(a){
    t_group <- data$treat == a
    cv <- stats::cov(data$response[t_group], mutilde[t_group, ])
    return(cv)
  }
  # Covariance matrix between Y and \mu
  cov_Ymu <- t(sapply(data$treat_levels, get.cov.Ya))

  # Get specific variance type for simple randomization
  if(model$variance_type == 1){

    # We are avoiding the situation where we have issues in estimating
    # the variance when many (or all) of the Y_a are zero in one group
    var_mutilde <- stats::var(mutilde)
    diag_mutilde <- diag(diag(var_mutilde))
    diag_covYmu <- diag(diag(cov_Ymu))
    diag_pi <- diag(1/c(data$pie))

    # Formula for variance calculation, doing a decomposition of variance
    # rather than calculating variance of residual
    diagmat <- vcov_sr_diag(data, mod, residual=data$response) +
      (diag_mutilde - 2 * diag_covYmu) * diag_pi

    # Sum of terms to compute simple randomization variance
    v <- diagmat + cov_Ymu + t(cov_Ymu) - stats::var(mutilde)

  } else if(model$variance_type == 2) {

    # This is the simplest type which just calculates
    # the variance of the residuals directly
    diagmat <- vcov_sr_diag(data, mod, residual=residual)
    v <- diagmat + cov_Ymu + t(cov_Ymu) - stats::var(mutilde)

  } else if(model$variance_type == 3){

    # This variance type is based on @Eureeca's work
    # which has a SR variance matrix which is for sure positive definite
    # in some special cases
    #
    # Biggest difference is that it uses observations in group a and b
    # to calculate the covariance between \mu_a(X) and \mu_b(X), rather than the
    # whole sample.
    #
    # This is helpful when there are large chance imbalances
    # in the distribution of X across the treatment groups, since
    # the covariance of Y_a and \mu_a(X) can only be calculated within group a.

    K <- length(data$treat_levels)
    v <- matrix(data=NA, nrow=K, ncol=K)

    for(a in 1:K){
      for(b in a:K){

        # Diagonal entries in the vcov matrix
        if(a == b){

          t_group <- data$treat == data$treat_levels[[a]]

          term1 <- mean(as.numeric(t_group) * residual^2) / data$pie[[a]]^2
          term2 <- cov(residual[which(t_group)], mutilde[which(t_group), a])
          term3 <- var(mutilde[which(t_group), a])

          v[a, a] <- term1 + 2 * term2 + term3

        # Off-diagonal entries in the vcov matrix
        } else {

          a_group <- data$treat == data$treat_levels[[a]]
          b_group <- data$treat == data$treat_levels[[b]]

          term1 <- cov(data$response[which(a_group)], mutilde[which(a_group), b])
          term2 <- cov(data$response[which(b_group)], mutilde[which(b_group), a])
          term3 <- cov(mutilde[which(a_group), c(a, b)])[1, 2]
          term4 <- cov(mutilde[which(b_group), c(a, b)])[1, 2]

          v[a, b] <- v[b, a] <- term1 + term2 - (term3 + term4) / 2

        }
      }
    }

  } else {
    stop("Unrecognized variance type.")
  }

  # Adjust for Z if necessary
  if(!is.null(model$omegaz_func)) v <- v - get.erb(model, data, mod, mu_hat=preds)

  return(v)
}

# Function to format results.
format_results <- function(labels, estimates, varcov, label_name="treat", ...){

  # Extract estimates and create results data
  result <- dplyr::tibble(
    label=labels,
    estimate=c(estimates),
    se=diag(varcov**0.5)
  )
  colnames(result) <- c(label_name, "estimate", "se")

  # Compute p-values based on the correct variance estimates
  result <- result %>% dplyr::mutate(
    `pval (2-sided)`=2*stats::pnorm(abs(.data$estimate/.data$se), lower.tail=F)
  )
  return(result)
}

.make.model <- function(data, ...){
  UseMethod(".make.model", data)
}

# Makes a model class for the specified glm adjustment method
# with settings for covariate randomization
# scheme and vcovHC type.
.make.model.RoboDataGLM <- function(data, car_scheme, vcovHC,
                                    g_family, g_accuracy, variance_type, ...) {

  # Get logic for car scheme + adjustment methods
  z_exists <- !is.null(data$car_strata)
  logic <- glmlogic(car_scheme=car_scheme, z_exists=z_exists)

  model <- structure(
    list(
      vcovHC=vcovHC,
      g_family=g_family,
      g_accuracy=g_accuracy,
      car_scheme=car_scheme,
      pu_joint_z=logic$pu_joint_z,
      pu_funcs=logic$pu_funcs,
      omegaz_func=logic$omegaz_func,
      variance_type=variance_type
    ),
    class=c(logic$method)
  )

  return(model)
}

# Makes a model class for the specified glm adjustment method
# with settings for covariate randomization
# scheme and vcovHC type.
.make.model.RoboDataSL <- function(data, car_scheme, vcovHC,
                                   covariate_to_include_strata,
                                   SL_libraries, SL_learners,
                                   k_split,
                                   g_accuracy, variance_type, ...) {

  x_exists <- !is.null(data$covariate)
  z_exists <- !is.null(data$car_strata)

  # Get logic for adjustment methods
  logic <- SLlogic(car_scheme=car_scheme,
                    x_exists=x_exists, z_exists=z_exists,
                    cov_strata=covariate_to_include_strata)
  model <- structure(
    list(
      vcovHC=vcovHC,
      SL_libraries=SL_libraries,
      SL_learners=SL_learners,
      k_split=k_split,
      car_scheme=car_scheme,
      adj_se_z=logic$adj_se_z,
      adj_vars=logic$adj_vars,
      pu_joint_z=logic$pu_joint_z,
      pu_funcs=logic$pu_funcs,
      omegaz_func=logic$omegaz_func,
      g_accuracy=g_accuracy,
      variance_type=variance_type
    ),
    class=c(logic$method)
  )

  return(model)
}


# Makes a model class for the specified TTE adjustment method
# with settings for covariate randomization
# scheme and vcovHC type.
.make.model.RoboDataTTE <- function(data, adj_method, car_scheme,
                                    p_trt, ref_arm, return_influence, ...) {

  x_exists <- !is.null(data$covariate)
  z_exists <- !is.null(data$car_strata)

  # Get logic for adjustment methods
  logic <- ttelogic(adj_method=adj_method, car_scheme=car_scheme,
                    x_exists=x_exists, z_exists=z_exists)

  if(adj_method %in% c("CL", "CSL")){
    classtype <- "LogRank"
  } else if(adj_method %in% c("coxscore")) {
    classtype <- "CoxScore"
  } else {
    stop("Unrecognized adjustment method.")
  }

  model <- structure(
    list(
      method=logic$method,
      adj_cov=logic$adj_cov,
      adj_strata=logic$adj_strata,
      car_strata=logic$car_strata,
      car_scheme=car_scheme,
      p_trt=p_trt,
      ref_arm=ref_arm,
      return_influence=return_influence,
      ...
    ),
    class=c(classtype, logic$method)
  )

  if ("return_influence" %in% names(model)){
    if (!is.logical(model$return_influence)) stop("return_influence must be either TRUE or FALSE.")
    if (model$return_influence) {
      if (!"id_col" %in% names(data)) stop("id_col must not be NULL if return_influence is TRUE")
      if (!is.null(data$covariate)){
        if ("id" %in% colnames(data$covariate)){
          stop("if return_influence is TRUE, no covariate can be called 'id'.")
        }
      }
    }
  }
  
  return(model)
}


# Makes a model class for the MH method
# with settings for estimand and CI type
.make.model.RoboDataMH <- function(data, estimand, ci_type, strata_cols, ...) {
  
  model <- structure(
    list(
      estimand = estimand,
      ci_type = ci_type,
      strata_cols = strata_cols
    ),
    class=c("MHModel")
  )
  
  return(model)
}

# SILENCE R CMD CHECK WORRIES
# regarding no visible binding for global variables

if(getRversion() >= "2.15.1"){
  # This is for using formula notation with glm
  utils::globalVariables(c("."))

  # This is for print.TTEResult and data.table dplyr issues
  utils::globalVariables(c("car_strata", "treat", "N", "name", "strata_col"))
}

#' Robust cox score adjustment
#'
#' @param ... Arguments to robincar_tte, other than `adj_method`
#'
#' @export
#'
#' @returns A result object with the following attributes:
#'
#' \item{result}{A list: "statistic" is the robust Cox score test statistic which can be used to obtain p-values; "U" and "se" are the numerator and denominator of the test statistic, respectively.}
#' \item{settings}{The covariate adjustment settings used.}
#' \item{original_df}{The dataset supplied by the user.}
#'
robincar_coxscore <- function(...){

  result <- robincar_tte(
    adj_method="coxscore", ...
  )

  return(result)

}

SLlogic <- function(x_exists, z_exists, car_scheme, cov_strata){

  adj_vars <- NULL

  aipw <- TRUE
  adj_se_z <- FALSE

  # Whether to check prediction un-biasedness in all joint levels
  # of Z, or just overall, and whether to report an error or warning
  pu_joint_z <- FALSE
  # Biased action will be one of "", "warning", "error"
  pu_funcs <- function(){}

  omegaz_func <- NULL

  if(car_scheme == "simple"){
    if(z_exists) .z.exist.warn.simple()
    if(x_exists){
      adj_vars <- "x"
      pu_funcs <- .pu.warn
    } else {
      .x.miss.err()
    }
  }
  if(car_scheme == "pocock-simon"){
    if(!z_exists){
      .z.miss.err()
    } else {
      omegaz_func <- omegaz.closure(car_scheme)
      if(!cov_strata){
        .z.include.warn()
      }
      pu_joint_z <- TRUE
      if(x_exists){
        adj_vars <- "joint_z_x"
        pu_funcs <- .pu.z.warn
      } else {
        adj_vars <- "joint_z"
        pu_funcs <- c(.pu.z.calibrate, .pu.z.warn)
      }
    }
  }
  if(car_scheme %in% APPLICABLE.SCHEMES){

    if(z_exists){
      omegaz_func <- omegaz.closure(car_scheme)

      adj_se_z <- TRUE
      if(x_exists){
        if(cov_strata){
          adj_vars <- "joint_z_x"
          pu_funcs <- .pu.warn
        } else {
          adj_vars <- "x"
        }
      } else {
        if(cov_strata){
          adj_vars <- "joint_z"
          pu_funcs <- .pu.warn
        } else {
        }
      }
    } else {
      .z.miss.err()
    }
  }
  method=c("SLModel")
  return(list(
    method=method,
    adj_vars=adj_vars,
    adj_se_z=adj_se_z,
    pu_joint_z=pu_joint_z,
    pu_funcs=pu_funcs,
    omegaz_func=omegaz_func
  ))
}

#' Covariate-adjusted estimators for time to event data
#'
#' Estimate a covariate-adjusted hazard ratio (`adj_method="CL"`),
#' or a covariate-adjusted stratified hazard ratio (`adj_method="CSL"`).
#'
#'
#' @param df A data.frame with the required columns
#' @param treat_col Name of column in df with treatment variable
#' @param response_col Name of the column in df with response variable
#' @param event_col Name of column in df with event indicator
#'                  (0/FALSE=no event, 1/TRUE=event)
#' @param car_strata_cols Names of columns in df with car_strata variables
#' @param covariate_cols Names of columns in df with covariate variables
#' @param car_scheme Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".
#' @param ref_arm Reference arm of the treatment group, defaults to NULL,
#'                which results in using the first element of `unique(data[, treat_col])`.
#' @param p_trt Treatment allocation ratio for the reference arm.
#' @param adj_method Adjustment method (one of "CL", "CSL")
#' @param interval Interval for uniroot function
#'
#' @export
#'
#' @returns An object with attribute named "result", which lists:
#'
#' \item{theta_L}{estimate of the hazard ratio}
#' \item{se_theta_L}{SE estimate of the hazard ratio}
#' \item{theta_CL}{estimate of the covariate-adjusted hazard ratio}
#' \item{se_theta_CL}{SE estimate of the covariate-adjusted hazard ratio}
#'
#' Other attributes are the settings used, data attributes, and the original data frame supplied by the user.
#'
robincar_covhr <- function(df,
                           treat_col, response_col, event_col,
                           car_strata_cols=NULL, covariate_cols=NULL,
                           p_trt=0.5, ref_arm=NULL,
                           car_scheme="simple",
                           adj_method="CL",
                           interval=c(-10, 10)){

  .check.car_scheme(car_scheme, car_strata_cols)

  data <- .make.data(
    df=df,
    classname="RoboDataTTE",
    treat_col=treat_col,
    response_col=response_col,
    event_col=event_col,
    car_strata_cols=car_strata_cols,
    covariate_cols=covariate_cols
  )
  validate(data, ref_arm)

  # Create model object
  model <- .make.model(
    data=data,
    adj_method=adj_method,
    car_scheme=car_scheme,
    p_trt=p_trt,
    ref_arm=ref_arm,
    interval=interval, 
    return_influence=F
  )

  # Append the CovHR classification
  # so that we perform estimation rather than testing
  class(model) <- c("CovHR", class(model))

  # Perform adjustment
  result <- adjust(model, data)
  result$original_df <- df

  return(result)

}

diff <- function(est){
  base <- est[1]
  cont <- est[-1] - base
  return(cont)
}

odds_ratio <- function(est){
  base <- est[1] / (1-est[1])
  cont <- est[-1] / (1-est[-1]) / base
  return(cont)
}

log_odds_ratio <- function(est){
  base <- est[1] / (1-est[1])
  cont <- log(est[-1] / (1-est[-1]) / base)
  return(cont)
}

ratio <- function(est){
  base <- est[1]
  cont <- est[-1] / base
  return(cont)
}

log_ratio <- function(est){
  base <- est[1]
  cont <- log(est[-1] / base)
  return(cont)
}

jacobian <- function (settings, est) {
  UseMethod("jacobian", settings)
}

#' @exportS3Method
jacobian.DIFF <- function(settings, est){
  k <- length(est)
  jacob <- cbind(rep(-1, k-1), diag(1, k-1))
  return(jacob)
}

#' @exportS3Method
jacobian.RATIO <- function(settings, est){
  col1 <- -1 * est[-1] / est[1]^2
  mat1 <- diag(1/est[1], length(est)-1)
  jacob <- cbind(col1, mat1)
  return(jacob)
}

#' @exportS3Method
jacobian.LOGRATIO <- function(settings, est){
  col1 <- -1 / est[1]
  mat1 <- diag(1/est[-1], length(est)-1)
  jacob <- cbind(col1, mat1)
  return(jacob)
}

#' @exportS3Method
jacobian.ODDSRATIO <- function(settings, est){
  col1 <- -1 / est[1] ** 2 * est[-1] / (1-est[-1])
  mat1 <- diag((1-est[-1]) / est[-1] / (1-est[1]) ** 2, length(est)-1)
  jacob <- cbind(col1, mat1)
  return(jacob)
}

#' @exportS3Method
jacobian.LOGODDSRATIO <- function(settings, est){
  col1 <- -1 / (est[1] * (1 - est[1]))
  mat1 <- diag(1 / (est[-1] * (1 - est[-1])), length(est)-1)
  jacob <- cbind(col1, mat1)
  return(jacob)
}

#' @exportS3Method
jacobian.CUSTOM <- function(settings, est){
  if(!is.null(settings$dh)){
    jacob <- settings$dh(est)
  } else {
    jacob <- numDeriv::jacobian(
      func=settings$h,
      x=est
    )
  }
  return(jacob)
}

# Create contrast settings based on a contrast function
contrast.settings <- function(k, contrast_h, contrast_dh=NULL){

  if(is.function(contrast_h)){
    h <- contrast_h
    type <- "CUSTOM"
    d <- length(h(rep(1, k))) # size of output
    name <- function(tl) paste0("contrast ", 1:d)
  } else if(contrast_h == "diff"){
    h <- diff
    type <- "DIFF"
    name <- function(tl) paste0("treat ", tl[-1], " - " , tl[1])
  } else if(contrast_h == "ratio"){
    h <- ratio
    type <- "RATIO"
    name <- function(tl) paste0("treat ", tl[-1], " / ", tl[1])
  } else if(contrast_h == "log_ratio"){
    h <- log_ratio
    type <- "LOGRATIO"
    name <- function(tl) paste0("log(treat ", tl[-1], " / ", tl[1], ")")
  } else if(contrast_h == "odds_ratio"){
    h <- odds_ratio
    type <- "ODDSRATIO"
    name <- function(tl) paste0("odds(treat ", tl[-1], ") / odds(", tl[1], ")")
  } else if(contrast_h == "log_odds_ratio"){
    h <- log_odds_ratio
    type <- "LOGODDSRATIO"
    name <- function(tl) paste0("log(odds(treat ", tl[-1], ")) - log(odds(", tl[1], "))")
  } else {
    stop("Unrecognized contrast function name.")
  }

  obj <- structure(
    list(
      h=h,
      dh=contrast_dh,
      name=name
    ),
    class=c("CONTRAST", type)
  )
  return(obj)
}

#' @importFrom emulator quad.tform
contrast <- function(settings, treat, est, varcov){

  # Get labels
  lab <- settings$name(treat)

  # If using an odds ratio contrast, make sure
  # give a warning if the estimates are not between 1 and 0.
  if(class(settings)[2] %in% c("ODDSRATIO", "LOGODDSRATIO")){
    if(any(est < 0) || any(est > 1)){
      warning("Estimates are not between 0 and 1: are you sure you want an odds ratio?")
    }
  }

  # If using a ratio or odds ratio, give a warning
  # that the performance is better with logs.
  if(class(settings)[2] %in% c("ODDSRATIO", "RATIO")){
    warning("Using a ratio or odds ratio is not recommended.
            Consider using log_ratio or log_odds_ratio for better
            performance of the variance estimator.")
  }

  # Get transformed estimate
  c_est <- settings$h(est)

  # Get Jacobian matrix
  jac <- jacobian(settings, est)

  # Get new variance covariance matrix based on old one & Jacobian
  vcv <- quad.tform(varcov, jac)

  # Format results
  result <- format_results(lab, c_est, vcv, label_name="contrast")

  return(list(result=result, varcov=vcv, settings=settings))
}

#' Estimate a treatment contrast
#'
#' Estimate a treatment contrast using the result of \link[RobinCar:robincar_linear]{RobinCar::robincar_linear()}, \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}, or \link[RobinCar:robincar_SL]{RobinCar::robincar_SL()} using
#' the delta method.
#'
#' @param result A LinModelResult or GLMModelResult
#' @param contrast_h An optional function to specify a desired contrast
#' @param contrast_dh An optional jacobian function for the contrast
#' @importFrom rlang .data
#' @export
#'
#' @returns A contrast object which has the following attributes:
#'
#'  \item{result}{A \link[dplyr:tibble]{dplyr::tibble()} with the label of the treatment contrast (e.g., 1 vs. 0), the estimate of the treatment contrast, estimated SE, and p-value based on a z-test with estimate and SE.}
#'  \item{varcov}{The variance-covariance matrix for the treatment contrast estimates.}
#'  \item{settings}{List of model settings used for the contrast.}
#'
robincar_contrast <- function(result, contrast_h, contrast_dh=NULL){

  # Get input dimension
  k <- nrow(result$result)

  # Create contrast settings
  settings <- contrast.settings(k, contrast_h, contrast_dh)

  # Run the contrast function
  c_result <- contrast(
    settings,
    treat=result$result$treat,
    est=result$result$estimate,
    varcov=result$varcov
  )
  return(
    structure(
      class="ContrastResult",
      list(result=c_result$result,
           varcov=c_result$varcov,
           settings=settings)
    )
  )
}

Robin_calibrate <- function(robin.g_object = G_zx$robin.g_object, # robin.g_object includes robin.g_data, car.scheme and car.z
                            robin.add_x = c("x1"),
                            robin.add_x_to_include_z = TRUE, # if robin.g_object$car.scheme = SR, this arg will be ignored (force to FALSE)
                            robin.vcovHC = c("HC0", "HC1", "HC3")){

  # Robin_calibrate applies ANHECOVA (robin_linear(robin.adj_method = "ANHECOVA")) to robin.g_object with some additional options/arguments
  # customized calibrate on all "g_pred_"s and customized "robin.add_x"
  # default double calibration can use all "g_pred_"s and all "x" and "z" used in robin.g_object to generate "g_pred_"

  robin.c_data <- robin.g_object$robin.g_data %>%
    rename_with(.cols = contains("g_pred_"), .fn = ~ str_replace(.x, "g_pred_", "mu_"))

  robin.c_x <- c(robin.add_x, grep("mu_", x = colnames(robin.c_data), value=TRUE)) # !is.null(robin.add_x) --> double calibration

  if(robin.g_object$car.scheme == "minimization"){
    robin.add_x_to_include_z <- TRUE # give a warning if robin.add_x_to_include_z=F under minimization
  }

  # We can also use robin_linear with ANHECOVA
  robin.c_return <- Robin_g(robin.data = robin.c_data,
                            car.scheme = robin.g_object$car.scheme,
                            car.z = robin.g_object$car.z,
                            robin.x = robin.c_x,
                            robin.formula = NULL,
                            robin.x_to_include_z = robin.add_x_to_include_z, # default
                            robin.adj_method = "heterogeneous",
                            robin.g_family = gaussian(link = "identity"), # ANHECOVA
                            robin.g_accuracy = 7,
                            robin.vcovHC = c("HC0", "HC1", "HC3"))

  return(robin.c_return) # robin.c_return can also be robin objective, with similar structure as robin_g or robin_linear
}

#' Estimate Mantel-Haenszel Risk Difference
#'
#' @description
#' This function estimates Mantel-Haenszel risk difference and average treatment effect.
#'
#' @param df A data.frame with the required columns
#' @param treat_col Name of column in df with treatment variable. Must be binary
#' @param response_col Name of the column in df with response variable
#' @param strata_cols Names of columns in df with strata variables
#' @param estimand A character string specifying the estimand. One of "MH" or "ATE" (default). See Details
#' @param ci_type A character string specifying the type of confidence interval. One of "GR", "mGR" (default), "Sato"

#' @details
#' The estimand of interest can be either Mantel-Haenszel risk difference or Average Treatment Effect (ATE). 
#' The latter is the default option of `estimand`. When `estimand="ATE"`, `ci_type` is limited to the modified Greenland variance estimator (mGR).
#'  Otherwise, Greenland's variance estimator (GR) and Sato's variance estimator are optional.
#' 
#' @examples
#' df <- RobinCar:::data_sim
#' df$y_bin = ifelse(df$y>2.5, 1, 0)
#' robincar_mh(df = df[df$A!=2,],
#'             treat_col = "A",
#'             response_col = "y_bin",
#'             strata_cols = c("z1", "z2"),
#'             estimand = "MH",
#'             ci_type = "mGR")
#' 
#' @export
robincar_mh <- function(df, treat_col, response_col, strata_cols, estimand="ATE", ci_type='mGR') {
  
  .check.estimand_ci(estimand, ci_type)
  
  data <- .make.data(
    df = df,
    classname = "RoboDataMH",
    treat_col = treat_col,
    response_col = response_col,
    car_strata_cols = strata_cols,
    formula = NULL
  )
  validate(data)
  
  model <- .make.model(
    data = data,
    estimand = estimand,
    ci_type = ci_type,
    strata_cols = strata_cols
  )
  
  
  result <- adjust(model, data)
  
  return(result)
}

DATA <- RobinCar:::data_sim
DATA$y_bin <- ifelse(DATA$y > 2, 1, 0)
DATA_01 <- DATA[DATA$A!=2, ]
DATA_02 <- DATA[DATA$A!=1, ]
DATA_12 <- DATA[DATA$A!=0, ]

test_that("MH errors", {
  expect_error(robincar_mh(df = DATA,
                           treat_col = "A",
                           response_col = "y_bin",
                           strata_cols = c("z1", "z2"),
                           estimand = "MH",
                           ci_type = "mGR"))
  
  expect_error(robincar_mh(df = DATA_01,
                           treat_col = "A",
                           response_col = "y",
                           strata_cols = c("z1", "z2"),
                           estimand = "MH",
                           ci_type = "mGR"))
  expect_error(robincar_mh(df = DATA_12,
                           treat_col = "A",
                           response_col = "y_bin",
                           strata_cols = c("z1", "z2"),
                           estimand = "ATE",
                           ci_type = "GR"))
  
})

test_that("MH works", {
  fit.MH <- robincar_mh(df = DATA_01,
                       treat_col = "A",
                       response_col = "y_bin",
                       strata_cols = c("z1", "z2"),
                       estimand = "MH",
                       ci_type = "mGR")
  expect_equal(fit.MH$result$estimate, 0.2183309, tolerance = 1e-5)
  expect_equal(fit.MH$result$se, 0.03593319, tolerance = 1e-5)
  
  fit.ATE <- robincar_mh(df = DATA_01,
                         treat_col = "A",
                         response_col = "y_bin",
                         strata_cols = c("z1"),
                         estimand = "ATE",
                         ci_type = "mGR")
  expect_equal(fit.ATE$result$estimate, 0.2094867, tolerance = 1e-5)
  expect_equal(fit.ATE$result$se, 0.04296656, tolerance = 1e-5)
  
})

ALLOWED_COX_CARSCHEME <- c("simple", "permuted-block", "biased-coin")

# @title Logic for performing robincar_coxscore or robincar_logrank for time to event endpoints
# @return warning or error messages based on the user input,
#         whether or not to adjust for covariates, and whether
#         to account for covariate-adaptive randomization
#
# logic_CL <- ttelogic(x_exists=TRUE, z_exists=TRUE,
#                               car_scheme="permuted-block",
#                               adj_method="CL")
#
# logic_CSL <- ttelogic(x_exists=TRUE, z_exists=TRUE,
#                                car_scheme="permuted-block",
#                                adj_method="CSL")
#
# logic_coxscore <- ttelogic(x_exists=TRUE, z_exists=TRUE,
#                                     car_scheme="permuted-block",
#                                     adj_method="coxscore")
ttelogic <- function(x_exists, z_exists,
                     car_scheme, adj_method){

  if(adj_method == "coxscore" & !(car_scheme %in% ALLOWED_COX_CARSCHEME)){
    .cox.carscheme.error()
  }

  if(adj_method %in% c("CL", "CSL", "coxscore")){

    if(x_exists){
      adj_cov <- TRUE
    } else {
      adj_cov <- FALSE
    }
    if(car_scheme == "simple"){
      adj_strata <- FALSE
      if(z_exists){
        .z.exist.warn.simple()
        if(adj_method == "CSL") .csl.nostrata.warn()
      }
    } else {
      if(!z_exists) .z.miss.err()
      adj_strata <- TRUE
    }
  } else{
    .tte.nomethod.error()
  }

  if(adj_method == "CSL" & car_scheme != "simple"){
    car_strata <- TRUE
  } else {
    car_strata <- FALSE
  }

  return(list(
    method=adj_method,
    adj_cov=adj_cov,
    adj_strata=adj_strata,
    car_strata=car_strata
  ))
}

test_that("logic for CL", {

  # 1. Z yes X yes
  # SR
  expect_warning(
    zyxy_sr <- ttelogic(
      x_exists=TRUE,
      z_exists=TRUE,
      car_scheme="simple",
      adj_method="CL"
    )
  )
  expect_true(zyxy_sr$adj_cov)
  expect_false(zyxy_sr$adj_strata)
  expect_false(zyxy_sr$car_strata)

  # CAR
  zyxy_car <- ttelogic(
    x_exists=TRUE,
    z_exists=TRUE,
    car_scheme=c("permuted-block"),
    adj_method=c("CL")
  )
  expect_true(zyxy_car$adj_cov)
  expect_true(zyxy_car$adj_strata)
  expect_false(zyxy_car$car_strata)


  # 2. Z yes X no
  # SR
  expect_warning(
    zyxn_sr <- ttelogic(
      x_exists=FALSE,
      z_exists=TRUE,
      car_scheme=c("simple"),
      adj_method=c("CL")
    )
  )
  expect_false(zyxn_sr$adj_cov)
  expect_false(zyxn_sr$adj_strata)
  expect_false(zyxn_sr$car_strata)

  # CAR
  zyxn_car <- ttelogic(
    x_exists=FALSE,
    z_exists=TRUE,
    car_scheme=c("biased-coin"),
    adj_method=c("CL")
  )
  expect_false(zyxn_car$adj_cov)
  expect_true(zyxn_car$adj_strata)
  expect_false(zyxn_car$car_strata)


  # 3. Z no X yes
  # SR
  znxy_sr = ttelogic(
    x_exists=TRUE,
    z_exists=FALSE,
    car_scheme=c("simple"),
    adj_method=c("CL")
  )
  expect_true(znxy_sr$adj_cov)
  expect_false(znxy_sr$adj_strata)
  expect_false(znxy_sr$car_strata)

  # CAR
  expect_error(
    znxy_car <- ttelogic(
      x_exists=TRUE,
      z_exists=FALSE,
      car_scheme=c("permuted-block"),
      adj_method=c("CL")
    )
  )

  # 4. Z no X no
  # SR
  znxn_sr <- ttelogic(
    x_exists=FALSE,
    z_exists=FALSE,
    car_scheme=c("simple"),
    adj_method=c("CL")
  )
  expect_false(znxn_sr$adj_cov)
  expect_false(znxn_sr$adj_strata)
  expect_false(znxn_sr$car_strata)

  # CAR
  expect_error(znxn_car <- ttelogic(
    x_exists=FALSE,
    z_exists=FALSE,
    car_scheme=c("permuted-block"),
    adj_method=c("CL")
  ))

})

test_that("logic for CSL", {

  # 1. Z yes X yes
  # SR
  expect_warning(
    zyxy_sr <- ttelogic(
      x_exists=TRUE,
      z_exists=TRUE,
      car_scheme="simple",
      adj_method="CSL"
    )
  )
  expect_true(zyxy_sr$adj_cov)
  expect_false(zyxy_sr$adj_strata)
  expect_false(zyxy_sr$car_strata)

  # CAR
  zyxy_car <- ttelogic(
    x_exists=TRUE,
    z_exists=TRUE,
    car_scheme=c("permuted-block"),
    adj_method=c("CSL")
  )
  expect_true(zyxy_car$adj_cov)
  expect_true(zyxy_car$adj_strata)
  expect_true(zyxy_car$car_strata)

  # 2. Z yes X no
  # SR
  expect_warning(
    zyxn_sr <- ttelogic(
      x_exists=FALSE,
      z_exists=TRUE,
      car_scheme=c("simple"),
      adj_method=c("CSL")
    )
  )
  expect_false(zyxn_sr$adj_cov)
  expect_false(zyxn_sr$adj_strata)
  expect_false(zyxn_sr$car_strata)

  # CAR
  zyxn_car <- ttelogic(
    x_exists=FALSE,
    z_exists=TRUE,
    car_scheme=c("biased-coin"),
    adj_method=c("CSL")
  )
  expect_false(zyxn_car$adj_cov)
  expect_true(zyxn_car$adj_strata)
  expect_true(zyxn_car$car_strata)


  # 3. Z no X yes
  # SR
  znxy_sr = ttelogic(
    x_exists=TRUE,
    z_exists=FALSE,
    car_scheme=c("simple"),
    adj_method=c("CSL")
  )
  expect_true(znxy_sr$adj_cov)
  expect_false(znxy_sr$adj_strata)
  expect_false(znxy_sr$car_strata)

  # CAR
  expect_error(
    znxy_car <- ttelogic(
      x_exists=TRUE,
      z_exists=FALSE,
      car_scheme=c("permuted-block"),
      adj_method=c("CSL")
    )
  )

  # 4. Z no X no
  # SR
  znxn_sr <- ttelogic(
    x_exists=FALSE,
    z_exists=FALSE,
    car_scheme=c("simple"),
    adj_method=c("CSL")
  )
  expect_false(znxn_sr$adj_cov)
  expect_false(znxn_sr$adj_strata)
  expect_false(znxn_sr$car_strata)

  # CAR
  expect_error(znxn_car <- ttelogic(
    x_exists=FALSE,
    z_exists=FALSE,
    car_scheme=c("permuted-block"),
    adj_method=c("CSL")
  ))

})

test_that("logic for CoxScore",{

  # 1. Z yes X yes
  # SR
  expect_warning(
    zyxy_sr <- ttelogic(
      x_exists=TRUE,
      z_exists=TRUE,
      car_scheme=c("simple"),
      adj_method=c("coxscore")
    )
  )
  expect_true(zyxy_sr$adj_cov)
  expect_false(zyxy_sr$adj_strata)
  expect_false(zyxy_sr$car_strata)

  # CAR
  zyxy_car <- ttelogic(
    x_exists=TRUE,
    z_exists=TRUE,
    car_scheme=c("permuted-block"),
    adj_method=c("coxscore")
  )
  expect_true(zyxy_car$adj_cov)
  expect_true(zyxy_car$adj_strata)
  expect_false(zyxy_car$car_strata)

  # 2. Z yes X no
  # SR
  expect_warning(
    zyxn_sr <- ttelogic(
      x_exists=FALSE,
      z_exists=TRUE,
      car_scheme=c("simple"),
      adj_method=c("coxscore")
    )
  )
  expect_false(zyxn_sr$adj_cov)
  expect_false(zyxn_sr$adj_strata)
  expect_false(zyxn_sr$car_strata)

  # CAR
  zyxn_car <- ttelogic(
    x_exists=FALSE,
    z_exists=TRUE,
    car_scheme = c("biased-coin"),
    adj_method = c("coxscore")
  )
  expect_false(zyxn_car$adj_cov)
  expect_true(zyxn_car$adj_strata)
  expect_false(zyxn_car$car_strata)

  # 3. Z no X yes
  # SR
  znxy_sr <- ttelogic(
    x_exists=TRUE,
    z_exists=FALSE,
    car_scheme=c("simple"),
    adj_method=c("coxscore")
  )
  expect_true(znxy_sr$adj_cov)
  expect_false(znxy_sr$adj_strata)
  expect_false(znxy_sr$car_strata)

  # CAR
  expect_error(
    znxy_car <- ttelogic(
      x_exists=TRUE,
      z_exists=FALSE,
      car_scheme=c("permuted-block"),
      adj_method=c("coxscore")
    )
  )

  # 4. Z no X no
  # SR
  znxn_sr <- ttelogic(
    x_exists=FALSE,
    z_exists=FALSE,
    car_scheme = c("simple"),
    adj_method = c("coxscore")
  )
  expect_false(znxn_sr$adj_cov)
  expect_false(znxn_sr$adj_strata)
  expect_false(znxn_sr$car_strata)

  # CAR
  expect_error(znxn_car <- ttelogic(
    x_exists=FALSE,
    z_exists=FALSE,
    car_scheme = c("permuted-block"),
    adj_method = c("coxscore")
  ))

})

# Compute the Omega(Z) matrix
# based on the given covariate adaptive randomization scheme.
#
# This function returns a function to obtain the correct matrix
# for a specified vector of randomization probabilities pi_t.
omegaz.closure <- function(car_scheme){

  omegaz.func <- function(pi_t){
    # TODO: Check that minimization works out to 0
    if(car_scheme %in% c("simple", "pocock-simon")){
      pi_t <- c(pi_t)
      omegaz <- diag(pi_t) - pi_t %*% t(pi_t)
    } else {
      omegaz <- matrix(0, nrow=length(pi_t), ncol=length(pi_t))
    }
    return(omegaz)
  }
  return(omegaz.func)
}

# Calculate nu_d for the randomization design
nu.d <- function(car_scheme, p_trt=0.5){

  if(length(car_scheme) > 1 | length(car_scheme) == 0){
    nu_d <- NA
  }else{
    if(is.na(car_scheme)){
      nu_d <- NA
    }else{
      if(car_scheme %in% c("permuted-block", "biased-coin")){
        nu_d <- 0
      }else if(car_scheme == "simple"){
        nu_d <- p_trt * (1 - p_trt)
      }
      else{
        nu_d <- NA
      }
    }
  }
  return(nu_d)
}

test_that("nu.d function",{

  p <- 0.3

  expect_equal(nu.d("simple", p_trt=1/2), 1/4)
  expect_equal(nu.d("simple", p_trt=p), p*(1-p))
  expect_equal(nu.d("permuted-block"), 0)
  expect_equal(nu.d("biased-coin"), 0)
  expect_equal(nu.d("urn"), NA)
  expect_equal(nu.d("whatever"), NA)
})

# Functions for validating data and creating data classes
# to be used across all analysis methods.

.check.response <- function(data){
  if(!is.numeric(data$response)){
    return("Response column must be numeric.")
  }
}

.check.event <- function(data){
  if(!all(data$event %in% c(0, 1))){
    return("Event column must contain only 0 and 1.")
  }
}

.check.treat <- function(data){
  if(!is.factor(data$treat)){
    return("Treatment column must be a factor variable.")
  }
}

.check.bin_response <- function(data){
  if(!all(data$response %in% c(0, 1))){
    return("Response column must contain only 0 and 1.")
  }
}

.check.bin_treat <- function(data){
  .check.treat(data)
  if(length(data$treat_levels)>2){
    return("Treatment column must be a binary variable.")
  }
}

.check.id <- function(data){
  if (!length(data$id) == length(unique(data$id))){
    return("Id column must not have duplicated values")
  }
}

.check.attributes <- function(x, ...){
  required <- c(...)
  existing <- names(x)
  missing <- which(!required %in% existing)

  if(length(missing > 0)){
    return(paste0("Missing data attributes ", paste(missing)))
  }
}

validate <- function (data, ...) {
  UseMethod("validate", data)
}

#' @exportS3Method
validate.RoboDataLinear <- function(data, ...){

  errors <- character()
  errors <- c(errors, .check.attributes(data, "treat", "response"))
  errors <- c(errors, .check.response(data))
  errors <- c(errors, .check.treat(data))

  .return.error(errors)
}

#' @exportS3Method
validate.RoboDataGLM <- function(data, ...){

  errors <- character()
  .return.error(errors)

}

#' @exportS3Method
validate.RoboDataSL <- function(data, ...){

  errors <- character()
  .return.error(errors)

}

#' @exportS3Method
validate.RoboDataTTE <- function(data, ref_arm, ...){

  errors <- character()
  errors <- c(errors, .check.attributes(data, "treat", "response", "event"))
  errors <- c(errors, .check.response(data))
  errors <- c(errors, .check.event(data))
  errors <- c(errors, .check.treat(data))

  if(length(data$treat_levels) != 2){
    errors <- c(errors, "Need to have only two treatment levels.")
  }
  if(!is.null(ref_arm)){
    if(!ref_arm %in% data$treat_levels){
      errors <- c(
        errors,
        message(
          "The reference group %s is not in the treatment levels %s",
          ref_arm,
          data$treat_levels
        )
      )
    }
  }
  if (!is.null(data$id_col)) {
    errors <- c(errors, .check.attributes(data, "id"))
    errors <- c(errors, .check.id(data))
  }

  .return.error(errors)
}

#' @exportS3Method
validate.RoboDataMH <- function(data, ...){
  
  errors <- character()
  errors <- c(errors, .check.attributes(data, "treat", "response", "joint_strata"))
  errors <- c(errors, .check.bin_treat(data))
  errors <- c(errors, .check.bin_response(data))
  
  .return.error(errors)
  
}

.edit.formula <- function(form, response_col, treat_col){

  # Convert formula to character
  form <- deparse(stats::as.formula(form))

  # Replace formula with proper names for response and treatment
  # -- response should only be at beginning
  newform <- gsub(paste0("^", response_col), "response", form)
  # -- only replace for the entire word
  newform <- gsub(paste0("\\b", treat_col, "\\b"), "treat", newform)

  # Extract covariate names from formula
  vars <- strsplit(newform, c("(~|\\+|:|\\*)"))[[1]]
  vars <- sapply(vars, function(x) gsub(" ", "", x))
  vars <- sapply(vars, function(x) gsub("\\(", "", x))
  vars <- sapply(vars, function(x) gsub("\\)", "", x))

  # Check to make sure that response and treatment columns
  # are in the formula
  if(vars[1] != "response") stop("Must include response variable on LHS of formula.")
  if(!"treat" %in% vars) stop("Must include treatment variable in formula.")

  vars <- vars[!vars %in% c("response", "treat", "0", "1")]
  vars <- unique(vars)

  return(list(newform, vars))
}

# Creates formula for robincar_linear
# based on adj_method=c("ANOVA", "ANCOVA", "ANHECOVA").
.create.formula <- function(adj_method, response_col, treat_col, covariate_cols){

  if(adj_method == "ANOVA"){
    form <- paste0(response_col, " ~ ", treat_col)
  } else {

    if(is.null(covariate_cols)) stop("Must provide covariates if
                                     adjustment method is ANCOVA or ANHECOVA.")
    covariates <- paste(covariate_cols, collapse=" + ")

    if(adj_method == "ANCOVA"){
      form <- paste0(response_col, " ~ ", treat_col, " + ", covariates)
    } else if(adj_method == "ANHECOVA"){
      form <- paste0(response_col, " ~ ", treat_col, " * (", covariates, ")")
    } else {
      stop("Unrecognized adjustment method.")
    }

  }
  return(form)
}

.df.toclass <- function(df, classname, ...){

  data <- list()
  atts <- list(...)

  for(i in 1:length(atts)){

    att_name <- names(atts)[i]
    att <- atts[[i]]

    # save the original names of the attributes
    data[[att_name]] <- att

    if(att_name == "formula"){

      if(is.null(att)) next

      response_col <- atts["response_col"]
      treat_col <- atts["treat_col"]

      if(is.null(response_col)) stop("Must provide a response column name.")
      if(is.null(treat_col)) stop("Must provide a treatment column name.")

      # Edit formula and get names of formula variables
      # The function below returns a list with [[1]] formula and [[2]] formula variable names
      forms <- .edit.formula(att, response_col, treat_col)

      # Include the covariates that will be needed for the formula
      # in a formula vector
      data[["formula"]] <- forms[[1]]

    } else if(grepl("col", att_name)){

      att_name <- gsub("_cols", "", att_name)
      att_name <- gsub("_col", "", att_name)
      if(!is.null(att)){
        for (a in att) {
          if (!a %in% colnames(df)) {
            stop(paste0("Column ", a, " not found."))
          }
        }
      }
      data[[att_name]] <- df[att]

    } else {

      stop(paste0("Unrecognized column arguments ",
                  att_name, ". All columns must have
                                            _col or _cols as a suffix."))

    }
  }
  argnames <- list(...)

  class(data) <- c("Data", classname)

  return(data)
}

.make.data <- function(df, classname, ...){

  # Convert data frame to object
  data <- .df.toclass(df=df, classname=classname, ...)

  if(!is.null(data$treat)){
    data$treat <- data$treat[[1]]
    if(!is.factor(data$treat)) data$treat <- as.factor(data$treat)
  }
  if(!is.null(data$response)){
    data$response <- data$response[[1]]
  }
  if(!is.null(data$event)){
    data$event <- data$event[[1]]
  }
  if(ncol(data$car_strata) == 0){
    data$car_strata <- NULL
  }
  if (!is.null(data$id)){
    if (ncol(data$id) > 0) {
      data$id <- data$id[[1]]
    }
  }
  if(!is.null(data$car_strata)){
    # Create joint car_strata levels
    data$joint_strata <- joint.car_strata(data$car_strata)
    data$joint_strata_levels <- levels(data$joint_strata)

    # Make sure all car_strata are factors
    for(col in colnames(data$car_strata)){
      data$car_strata[col] <- as.factor(data$car_strata[[col]])
    }
  }
  if(!is.null(data$covariate)){
    if(ncol(data$covariate) == 0){
      data$covariate <- NULL
    }
  }

  # Save original data frame
  data$df <- df

  # Add additional data attributes
  data$n <- nrow(df)
  data$k <- length(levels(data$treat))
  data$treat_levels <- levels(data$treat)

  # Estimate treatment allocation proportion
  data$pie <- table(data$treat) %>%
    proportions() %>%
    as.matrix()

  return(data)
}

test_that("edit formula", {
  form <- "y ~ A:(x1+x2)"
  edited <- .edit.formula(form, response_col="y", treat_col="A")
  expect_equal(edited[[1]], "response ~ treat:(x1 + x2)")
  expect_equal(edited[[2]], c("x1", "x2"))
})

test_that("edit formula", {
  form <- "y ~ A*(x1+x2)"
  edited <- .edit.formula(form, response_col="y", treat_col="A")
  expect_equal(edited[[1]], "response ~ treat * (x1 + x2)")
  expect_equal(edited[[2]], c("x1", "x2"))
})

test_that("edit formula with odd covariates 1", {
  form <- "y ~ A + y1"
  edited <- .edit.formula(form, response_col="y", treat_col="A")
  expect_equal(edited[[1]], "response ~ treat + y1")
  expect_equal(edited[[2]], c("y1"))
})

test_that("edit formula with odd covariates 2", {
  form <- "y ~ A + A1"
  edited <- .edit.formula(form, response_col="y", treat_col="A")
  expect_equal(edited[[1]], "response ~ treat + A1")
  expect_equal(edited[[2]], c("A1"))
})

#' Robust (potentially stratified) logrank adjustment
#'
#' Perform a robust covariate-adjusted logrank test ("CL") that can be stratified ("CSL") if desired.
#'
#' Note: Since RobinCar version 0.4.0, the variance of the test statistic has changed
#' to better accommodate tied event times.
#'
#' \eqn{\hat{\sigma}_{\rm CL}^2} and \eqn{\hat{\sigma}_{\rm CSL}^2} for the
#' covariate-adjusted stratified log-rank test are given by Ye, Shao, and Yi (2024) after
#' equation (8) on page 700, with \eqn{\hat{\sigma}_{\rm SL}^2} replaced
#' by the following estimator, which is the standard denominator of the logrank test:
#'
#' \deqn{\frac1n \sum_{j} \sum_{i} v_j(t_i)}
#' \deqn{v_j(t_i) = \frac{Y_{0,j}(t)Y_{1,j}(t)d_j(t)\left[Y_j(t) - d_j(t)\right]}{Y_j(t)^2\left[Y_j(t) - 1 \right]}}
#' where \eqn{t_i} are strata-specific unique failure times, \eqn{d_j(t)} is the number of events at time \eqn{t} in strata \eqn{j},
#' \eqn{Y_j(t)} is the number at risk within strata \eqn{j} at time \eqn{t},
#' and \eqn{Y_{a,j}(t)} is the number at risk within strata \eqn{j} and treatment \eqn{a} at time \eqn{t}.
#'
#' Please see Ye, Shao, and Yi (2024)'s "Covariate-adjusted log-rank test: guaranteed efficiency
#' gain and universal applicability" in \emph{Biometrika} for more details about \eqn{\hat{\sigma}_{\rm CSL}^2}.
#'
#' @param adj_method Adjustment method, one of "CL", "CSL"
#' @param ... Additional arguments to `robincar_tte`
#'
#' @export
#'
#' @examples
#' library(magrittr)
#' library(dplyr)
#' library(forcats)
#' set.seed(0)
#' n=100
#' data.simu0=data_gen(n=n,
#'                     theta=0,
#'                     randomization="permuted_block",
#'                     p_trt=0.5,
#'                     case="case2") %>% mutate(strata1=sample(letters[1:3],n,replace=TRUE),
#'                                              strata2=sample(LETTERS[4:5],n,replace=TRUE))
#'
#' out <- robincar_logrank(df=data.simu0,
#'                         treat_col="I1",
#'                         p_trt=0.5,
#'                         ref_arm=0,
#'                         response_col="t",
#'                         event_col="delta",
#'                         covariate_cols=c("model_z1", "model_z2"),
#'                         car_scheme="simple",
#'                         adj_method=c("CL"))
#'
#' set.seed(0)
#' n=100
#' data.simu0=data_gen(n=n,
#'                     theta=0,
#'                     randomization="permuted_block",
#'                     p_trt=0.5,
#'                     case="case1")
#'
#' data.simu <- data.simu0 %>%
#'   tidyr::pivot_longer(cols=starts_with("car_strata"),
#'                       names_prefix="car_strata",
#'                       names_to="strt") %>%
#'   filter(value==1) %>% select(-value) %>%
#'   mutate(strt=forcats::as_factor(strt)) %>%
#'   select(t,strt) %>%
#'   left_join(data.simu0, .)
#'
#' out1 <- robincar_logrank(df=data.simu,
#'                          treat_col="I1",
#'                          p_trt=0.5,
#'                          ref_arm=0,
#'                          response_col="t",
#'                          event_col="delta",
#'                          car_strata_cols="strt",
#'                          covariate_cols=NULL,
#'                          car_scheme=c("permuted-block"),
#'                          adj_method=c("CSL")
#' )
#'
#' @returns A result object with the following attributes:
#'
#' \item{result}{A list: "statistic" is the adjusted logrank test statistic which can be used to obtain p-values; "U" and "se" are the numerator and denominator of the test statistic, respectively; if the input parameter "return_influence" is set to TRUE, the list will contain an element called "inf_func", which is a data.frame with columns "id" and "inf_func", which is the influence function for each subject for the covariate-adjusted log-rank test (stratified or unstratified). This influence function can be used, e.g., to calculate the correlation between interim and final test statistics in a group sequential design.}
#' \item{settings}{The covariate adjustment settings used.}
#' \item{original_df}{The dataset supplied by the user.}
#'
robincar_logrank <- function(adj_method, ...){

  .check.adj_method.logrank(adj_method)
  result <- robincar_tte(adj_method, ...)

  return(result)

}

library(survival)
library(tidyverse)

files <- Sys.glob("~/Documents/GitHub/RobinCar/R/*.R")
map(files, source)

DATA <- ovarian %>%
  rename(tte = futime, obs = fustat) %>%
  arrange(tte)

test_that("Logrank influence function error checking 6", {
  DATA$id <- DATA$ecog.ps
  expect_error(robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs",
    return_influence = T,
    id_col = "idx"
  ), "Column idx not found")
})

test_that("Logrank influence function error checking 5", {
  DATA$id <- DATA$ecog.ps
  expect_no_error(robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    covariate_col = "id",
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs"
  ))
})

test_that("Logrank influence function error checking 4", {
  DATA$idx <- 1:nrow(DATA)
  DATA$id <- DATA$ecog.ps
  expect_error(robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    covariate_col = "id",
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs",
    return_influence = T,
    id_col = "idx"
  ), "if return_influence is TRUE, no covariate can be called 'id'.")
})

test_that("Logrank influence function error checking 3", {
  DATA$idx <- 1:nrow(DATA)
  expect_error(robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs",
    return_influence = "cheese",
    id_col = "idx"
  ), "return_influence must be either TRUE or FALSE")
})

test_that("Logrank influence function error checking 2", {
  DATA$idx <- 5
  expect_error(robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs",
    return_influence = T,
    id_col = "idx"
  ), "Id column must not have duplicated values")
})

test_that("Logrank influence function error checking 1", {
  expect_error(robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs",
    return_influence = T
  ), "id_col must not be NULL if return_influence is TRUE")
})

test_that("Logrank influence function", {
  DATA$idx <- 1:nrow(DATA)
  RC1 <- robincar_logrank(
    adj_method = "CL",
    df = DATA,
    treat_col = "rx",
    p_trt = 0.5,
    ref_arm = 1,
    response_col = "tte",
    event_col = "obs",
    return_influence = T,
    id_col = "idx"
  )
  expect_equal(sum(RC1$result$inf_func$inf_func), RC1$result$strata_sum$U_SL_z[1])
})

###### Author: Ting Ye ######
###### Last modified: Jan 1, 2023 ######
# Use null variance #
# Add estimation #

#########################
#   Added Jan 1, 2023   #
#########################

# covariate adjusted hazard ratio estimation proposed in Ye, Yi, Shao (2022)
covariate_adjusted_logrank_estimation<-function(data.simu,p_trt){
  n<-dim(data.simu)[1]
  data.simu$fz<-ind_to_factor(data.simu)
  data.rev<-data.sort(data.simu)
  T.seq<-data.rev$t
  T.rep<-c(0,T.seq)[1:n]
  T.diff<-T.seq-T.rep
  same.ind<-which(T.diff==0)
  n_col<-dim(data.rev)[2]
  for(ind in same.ind){
    data.rev[ind,(n_col-2):n_col]<-data.rev[ind-1,(n_col-2):n_col]
  }
  score<-function(theta){
    S.alt<-sum(data.rev$delta*(data.rev$I1-exp(theta)*data.rev$Y1/(exp(theta)*data.rev$Y1+data.rev$Y0)))/n
    return(S.alt)
  }
  theta_L<-uniroot(score,interval=c(-10,10))$root # get the unadjusted estimator

  x.model<-as.matrix(data.rev[,grepl("model",names(data.rev)),drop=FALSE]) #only those has model
  if(length(levels(data.rev$fz))==1){
    x.mat<-model.matrix(~x.model,data=data.rev)[,-1,drop=FALSE]
  }else{
    x.mat<-model.matrix(~factor(fz)+x.model,data=data.rev)[,-1,drop=FALSE]
  }
  x.centered<-sweep(x.mat, 2, colMeans(x.mat))
  # calculating beta0 and beta1
  data.rev$O.hat1<-data.rev$delta*data.rev$Y0/(exp(theta_L)*data.rev$Y1+data.rev$Y0) -
    exp(theta_L)*cumsum(data.rev$delta*data.rev$Y0/(exp(theta_L)*data.rev$Y1+data.rev$Y0)^2)
  data.rev$O.hat0<-exp(theta_L)*data.rev$delta*data.rev$Y1/(exp(theta_L)*data.rev$Y1+data.rev$Y0) -
    exp(theta_L)*cumsum(data.rev$delta*data.rev$Y1/(exp(theta_L)*data.rev$Y1+data.rev$Y0)^2)

  fit1.Ohat<-lm(data.rev$O.hat1[data.rev$I1==1]~x.mat[data.rev$I1==1,])
  fit0.Ohat<-lm(data.rev$O.hat0[data.rev$I1==0]~x.mat[data.rev$I1==0,])
  beta1.Ohat<-fit1.Ohat$coefficients[-1]
  beta0.Ohat<-fit0.Ohat$coefficients[-1]

  ind.na<-which(is.na(beta1.Ohat))
  if(length(ind.na)!=0){
    warning("Removing model variables that are linearly dependent with the stratification variables.")
    x.mat<-x.mat[,-ind.na,drop=FALSE]
    x.centered<-x.centered[,-ind.na,drop=FALSE]
    beta1.Ohat<-beta1.Ohat[-ind.na]
    beta0.Ohat<-beta0.Ohat[-ind.na]
  }
  score_CL<-function(theta){
    res<-score(theta)-sum(data.rev$I1*(t((x.centered)%*% beta1.Ohat )) -
                            data.rev$I0*(t((x.centered)%*% beta0.Ohat)))/n
    return(res)
  }
  theta_CL<-uniroot(score_CL,interval=c(-10,10))$root # get the adjusted estimator

  # Modification for making consistent with current code for testing (theta_L in place of theta_CL)
  sigma2_L<-exp(theta_L)*sum(data.rev$delta*data.rev$Y0*data.rev$Y1/(exp(theta_L)*data.rev$Y1+data.rev$Y0)^2)/n
  sigma2_CL<-sigma2_L-p_trt*(1-p_trt)*(beta1.Ohat+beta0.Ohat) %*% var(x.mat) %*% (beta1.Ohat+beta0.Ohat)
  var_est<-sigma2_CL/sigma2_L^2/n
  return(list(theta_L=theta_L,se.theta_L=sqrt(1/sigma2_L/n),
              theta_CL=theta_CL,se.theta_CL=sqrt(var_est)))
}

covariate_adjusted_stratified_logrank_estimation<-function(data.simu,p_trt){
  n<-dim(data.simu)[1]
  data.simu$fz<-ind_to_factor(data.simu)
  fz.n<-unique(data.simu$fz)
  # obtain U_SL
  score_SL<-function(theta){
    S.alt<-0
    for(z in fz.n){
      data.tmp<-data.sort(data.simu[data.simu$fz==z,])
      T.seq<-data.tmp$t
      T.rep<-c(0,T.seq)[1:length(T.seq)]
      T.diff<-T.seq-T.rep
      same.ind<-which(T.diff==0)
      n_col<-dim(data.tmp)[2]
      for(ind in same.ind){
        data.tmp[ind,(n_col-2):n_col]<-data.tmp[ind-1,(n_col-2):n_col]
      }
      S.alt<-S.alt+sum(data.tmp$delta*(data.tmp$I1-exp(theta)*data.tmp$Y1/(exp(theta)*data.tmp$Y1+data.tmp$Y0)))/n
    }
    return(S.alt)
  }
  theta_SL<-uniroot(score_SL,interval=c(-10,10))$root # get the unadjusted stratified estimator

  data.rev<-data.sort(data.simu[data.simu$fz==fz.n[1],])
  T.seq<-data.rev$t
  T.rep<-c(0,T.seq)[1:length(T.seq)]
  T.diff<-T.seq-T.rep
  same.ind<-which(T.diff==0)
  n_col<-dim(data.rev)[2]
  for(ind in same.ind){
    data.rev[ind,(n_col-2):n_col]<-data.rev[ind-1,(n_col-2):n_col]
  }
  data.rev$O.hat1<-data.rev$delta*data.rev$Y0/(exp(theta_SL)*data.rev$Y1+data.rev$Y0) -
    exp(theta_SL)*cumsum(data.rev$delta*data.rev$Y0/(exp(theta_SL)*data.rev$Y1+data.rev$Y0)^2)
  data.rev$O.hat0<-exp(theta_SL)*data.rev$delta*data.rev$Y1/(exp(theta_SL)*data.rev$Y1+data.rev$Y0) -
    exp(theta_SL)*cumsum(data.rev$delta*data.rev$Y1/(exp(theta_SL)*data.rev$Y1+data.rev$Y0)^2)

  # calculate stratum-specific O_zij
  for(z in fz.n[-1]){
    data.tmp<-data.sort(data.simu[data.simu$fz==z,])
    T.seq<-data.tmp$t
    T.rep<-c(0,T.seq)[1:length(T.seq)]
    T.diff<-T.seq-T.rep
    same.ind<-which(T.diff==0)
    n_col<-dim(data.tmp)[2]
    for(ind in same.ind){
      data.tmp[ind,(n_col-2):n_col]<-data.tmp[ind-1,(n_col-2):n_col]
    }
    data.tmp$O.hat1<- data.tmp$O.hat1<-data.tmp$delta*data.tmp$Y0/(exp(theta_SL)*data.tmp$Y1+data.tmp$Y0) -
      exp(theta_SL)*cumsum(data.tmp$delta*data.tmp$Y0/(exp(theta_SL)*data.tmp$Y1+data.tmp$Y0)^2)
    data.tmp$O.hat0<-exp(theta_SL)*data.tmp$delta*data.tmp$Y1/(exp(theta_SL)*data.tmp$Y1+data.tmp$Y0) -
      exp(theta_SL)*cumsum(data.tmp$delta*data.tmp$Y1/(exp(theta_SL)*data.tmp$Y1+data.tmp$Y0)^2)
    data.rev<-rbind(data.rev,data.tmp)
  }
  n<-dim(data.rev)[1]

  x.model<-as.matrix(data.rev[,grepl("model",names(data.rev)),drop=FALSE]) #only those has model
  fit.anhc1<-lm(O.hat1~factor(fz)+x.model[data.rev$I1==1,],data=data.rev[data.rev$I1==1,])
  fit.anhc0<-lm(O.hat0~factor(fz)+x.model[data.rev$I1==0,],data=data.rev[data.rev$I1==0,])
  which.model<-which(grepl("x.model",names(fit.anhc1$coefficients)))
  which.model.notNA<-which(!is.na(fit.anhc1$coefficients[which.model]))
  if(length(which.model.notNA)==0){
    stop("The model variables are linearly dependent on Z. The covariate-adjusted stratified log-rank is the same as the stratified log-rank.")
  }else{
    fit.anhc1.coefx<-fit.anhc1$coefficients[which.model[which.model.notNA]]
    fit.anhc0.coefx<-fit.anhc0$coefficients[which.model[which.model.notNA]]

    x.coefx<-model.matrix(~factor(fz)+as.matrix(x.model),data=data.rev)[,which.model[which.model.notNA],drop=FALSE]
    for(z in fz.n){
      x.coefx[data.rev$fz==z,]<-sweep(x.coefx[data.rev$fz==z,,drop=FALSE], 2, colMeans(x.coefx[data.rev$fz==z,,drop=FALSE]))
    }
    x.coefx.centered.byZ<-x.coefx
    score_CSL<-function(theta){
      res<-score_SL(theta)-
        sum(data.rev$I1*as.vector(x.coefx.centered.byZ %*% fit.anhc1.coefx) - data.rev$I0*as.vector(x.coefx.centered.byZ %*% fit.anhc0.coefx))/n
      return(res)
    }
    theta_CSL<-uniroot(score_CSL,interval=c(-10,10))$root # get the adjusted estimator
  }
  ### calculate variance ###
  # Modification for making consistent with current code for testing (theta_SL in place of theta_CSL)
  sigma2_SL<-exp(theta_SL)*sum(data.rev$delta*data.rev$Y0*data.rev$Y1/(exp(theta_SL)*data.rev$Y1+data.rev$Y0)^2)/n
  sigma2_CSL<-sigma2_SL
  for(z in fz.n){
    pz<-length(which(data.rev$fz==z))/n
    SigmaXz<-var(x.coefx[data.rev$fz==z,,drop=FALSE])
    sigma2_CSL<-sigma2_CSL-  p_trt*(1-p_trt)*(fit.anhc1.coefx+fit.anhc0.coefx) %*% SigmaXz %*%(fit.anhc1.coefx+fit.anhc0.coefx)*pz
  }
  var_est<-sigma2_CSL/sigma2_SL^2/n
  return(list(theta_SL=theta_SL,se.theta_SL=sqrt(1/sigma2_SL/n),
              theta_CSL=theta_CSL,se.theta_CSL=sqrt(var_est)))
}

#' @importFrom dplyr mutate group_by summarise arrange n
#' @exportS3Method
adjust.LogRank <- function(model, data, ...){

  # Creates data
  df <- create.tte.df(model, data)

  # Risk set and failure time times
  df <- process.tte.df(df, ref_arm=model$ref_arm)
  df <- df %>% mutate(lin_pred=0)

  # Get ordered data
  df <- get.ordered.data(df, ref_arm=model$ref_arm)
  df <- get.tte.adjustment(df, model, data)

  df <- df %>%
    dplyr::mutate(
      uu_cl  = .data$trt1 * (.data$O.hat - .data$adjust1) -
               .data$trt0 * (.data$O.hat - .data$adjust0),
      ssig_l = .data$event * .data$Y0 * .data$Y1 * (.data$Y - .data$n.events) / .data$Y^2 / (.data$Y-1)
    )

  # Summarize by car_strata (if CL, then single car_strata)
  ss <- df %>%
    dplyr::filter(!is.na(.data$uu_cl)) %>%
    dplyr::group_by(.data$car_strata) %>%
    dplyr::summarise(
      U_SL_z  = sum(.data$uu_cl),
      var_adj = model$p_trt * (1 - model$p_trt) * unique(.data$bsigb) * dplyr::n(),
      .groups = "drop"
    ) %>%
    dplyr::arrange(.data$car_strata)

  # Final quantities for the C(S)L statistic
  U_CSL     <- sum(df$uu_cl) / data$n
  var_CSL   <- (sum(df$ssig_l, na.rm=TRUE) - sum(ss$var_adj)) / data$n^2
  se        <- sqrt(var_CSL)
  statistic <- U_CSL / se
  
  result <- list(
    strata_sum=ss,
    U=U_CSL,
    se=se,
    statistic=statistic
  )
  if (model$return_influence) {
    influence_function <- df$uu_cl + (model$p_trt) * df$adjust1 - (1 - model$p_trt) * df$adjust0
    result$inf_func <- data.frame(id = df$id, inf_func = influence_function)
  } 
  return(
    structure(
      class="TTEResult",
      list(result=result, settings=model, data=data)
    )
  )
}

#' BETA: Covariate adjustment using working models from the super learner libraries
#' through the AIPW package with cross-fitting.
#'
#' Estimate treatment-group-specific response means and (optionally)
#' treatment group contrasts using a generalized linear working model.
#'
#' *WARNING: This function is still under development and has not been extensively tested.*
#' This function currently only works for two treatment groups.
#' Before using this function, you must load the \link[SuperLearner]{SuperLearner} library with
#' `library(SuperLearner)`, otherwise the function call will fail.
#'
#' @param df A data.frame with the required columns
#' @param treat_col Name of column in df with treatment variable
#' @param response_col Name of the column in df with response variable
#' @param car_strata_cols Names of columns in df with car_strata variables
#' @param covariate_cols Names of columns in df with covariate variables
#' @param car_scheme Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".
#' @param covariate_to_include_strata Whether to include car_strata variables in covariate adjustment. Defaults to F for ANOVA and ANCOVA; defaults to T for ANHECOVA. User may override by passing in this argument.
#' @param contrast_h An optional function to specify a desired contrast
#' @param contrast_dh An optional jacobian function for the contrast (otherwise use numerical derivative)
#' @param SL_libraries Vector of super-learner libraries to use for the covariate adjustment (see \link[SuperLearner:listWrappers]{SuperLearner::listWrappers})
#' @param SL_learners Optional list of super-learner "learners" to use for the covariate adjustment (see \link[SuperLearner:create.Learner]{SuperLearner::create.Learner())}
#' @param k_split Number of splits to use in cross-fitting
#' @param g_accuracy Level of accuracy to check prediction un-biasedness (in digits).
#' @param variance_type The type of variance estimator to use, type 1, 2, or 3. All three are asymptotically equivalent. See details of \link[RobinCar:robincar_glm]{RobinCar::robincar_glm} for more.
#'
#' @export
#' @importFrom SuperLearner listWrappers
#' @import SuperLearner
#' @import AIPW
#'
#' @examples
#'
#' library(SuperLearner)
#' library(ranger)
#' n <- 1000
#' set.seed(10)
#' DATA2 <- data.frame(A=rbinom(n, size=1, prob=0.5),
#'                     y=rbinom(n, size=1, prob=0.2),
#'                     x1=rnorm(n),
#'                     x2=rnorm(n),
#'                     x3=as.factor(rbinom(n, size=1, prob=0.5)),
#'                     z1=rbinom(n, size=1, prob=0.5),
#'                     z2=rbinom(n, size=1, prob=0.5))
#' DATA2[, "y"] <- NA
#' As <- DATA2$A == 1
#' DATA2[DATA2$A == 1, "y"] <- rbinom(
#'   sum(As),
#'   size=1,
#'   prob=exp(DATA2[As,]$x1)/(1+exp(DATA2[As,]$x1)))
#' DATA2[DATA2$A == 0, "y"] <- rbinom(
#'   n-sum(As),
#'   size=1,
#'   prob=exp(1 +
#'     5*DATA2[!As,]$x1 + DATA2[!As,]$x2)/
#'     (1+exp(1 + 5*DATA2[!As,]$x1 + DATA2[!As,]$x2)))
#' DATA2$A <- as.factor(DATA2$A)
#'
#' sl.mod <- robincar_SL(
#'   df=DATA2,
#'   response_col="y",
#'   treat_col="A",
#'   car_strata_cols=c("z1"),
#'   covariate_cols=c("x1"),
#'   SL_libraries=c("SL.ranger"),
#'   car_scheme="permuted-block",
#'   covariate_to_include_strata=TRUE
#' )
#'
#' sl.mod$result
#'
#' @returns See value of \link[RobinCar:robincar_glm]{RobinCar::robincar_glm}, but the working model for \eqn{\hat{\mu}(X_i)} is based on the \link[AIPW:AIPW]{AIPW::AIPW} package that uses specified SuperLearner libraries and cross-fitting.
#' Also, `mod` attribute is an object of class \link[AIPW:AIPW]{AIPW::AIPW}.
robincar_SL <- function(df,
                        treat_col, response_col, car_strata_cols=NULL, covariate_cols=NULL,
                        car_scheme="simple",
                        covariate_to_include_strata=NULL,
                        SL_libraries=c(), SL_learners=c(),
                        k_split=2,
                        g_accuracy=7,
                        contrast_h=NULL, contrast_dh=NULL,
                        variance_type=1){

  .check.car_scheme(car_scheme, car_strata_cols)
  # .check.sl.libraries(SL_libraries)

  if(length(SL_libraries) + length(SL_learners) == 0){
    stop("Must provide at least one SL_libraries or SL_learners.")
  }

  # Add a k split index to .make.data, so that we can index
  # the dataset on the kth index set for cross-fitting
  data <- .make.data(
    df=df, classname="RoboDataSL",
    treat_col=treat_col,
    response_col=response_col,
    car_strata_cols=car_strata_cols,
    covariate_cols=covariate_cols
  )
  validate(data)

  # Create model object
  model <- .make.model(
    data=data,
    car_scheme=car_scheme,
    vcovHC="HC0",
    covariate_to_include_strata=covariate_to_include_strata,
    SL_libraries=SL_libraries,
    SL_learners=SL_learners,
    k_split=k_split,
    g_accuracy=g_accuracy,
    variance_type=variance_type
  )

  # Perform adjustment
  result <- adjust(model, data)
  # This is to save the original dataset for calibration add_x later on
  result$original_df <- df

  # Create transformation object
  if(!is.null(contrast_h)){
    c_result <- robincar_contrast(
      result=result,
      contrast_h=contrast_h,
      contrast_dh=contrast_dh
    )
    result <- list(main=result, contrast=c_result)
  }

  return(result)
}

#' BETA: Covariate adjustment using working models from the super learner libraries
#' through the AIPW package with cross-fitting, with median adjustment.
#'
#' Estimate treatment-group-specific response means and (optionally)
#' treatment group contrasts using a generalized linear working model.
#' Perform median adjustment to limit randomness induced from cross-fitting.
#'
#' *WARNING: This function is still under development and has not been extensively tested.*
#' This function currently only works for two treatment groups.
#' Before using this function, you must load the SuperLearner library with
#' `library(SuperLearner)`, otherwise the function call will fail.
#'
#' @param n_times Number of times to run the robincar_SL function
#' @param seed Seed to set before running the set of functions
#' @inheritParams robincar_SL
#' @export
#'
#' @returns See value of \link[RobinCar:robincar_SL]{RobinCar::robincar_SL}.
#' Attributes `mods` and `mu_as` are lists of `mod` and `mu_a` attributes, respectively, for each replicate of `robincar_SL` used in the median.
robincar_SL_median <- function(n_times, seed, df,
                               treat_col, response_col,
                               car_strata_cols=NULL, covariate_cols=NULL,
                               car_scheme="simple",
                               covariate_to_include_strata=NULL,
                               SL_libraries=c(), SL_learners=c(),
                               k_split=2,
                               g_accuracy=7,
                               contrast_h=NULL, contrast_dh=NULL){

  if(n_times %% 2 == 0) stop("Must have an odd number of n_times.")

  # Set the seed before running any of the robincar function
  # that will do sample splitting.
  set.seed(seed)
  res <- list()
  for(i in 1:n_times){
    cat(".")
    res[[i]] <- robincar_SL(
      df=df,
      treat_col=treat_col,
      response_col=response_col,
      car_strata_cols=car_strata_cols,
      covariate_cols=covariate_cols,
      car_scheme=car_scheme,
      covariate_to_include_strata=covariate_to_include_strata,
      SL_libraries=SL_libraries,
      SL_learners=SL_learners,
      k_split=k_split,
      g_accuracy=g_accuracy,
      contrast_h=contrast_h,
      contrast_dh=contrast_dh)
  }

  estimates <- sapply(res, function(x) x$result$estimate)
  varcovs <- lapply(res, function(x) x$varcov)

  # Get median of estimates
  estimate <- apply(estimates, 1, stats::median)

  # Create varcov_tilde which adds on the residual
  varcov_tilde <- list()
  for(i in 1:n_times){
    c_est <- estimates[, i] - estimate # centered estimates
    varcov_tilde[[i]] <- varcovs[[i]] + c_est %*% t(c_est)
  }
  varcov_opnorm <- sapply(varcov_tilde, function(x) norm(x, type="2"))
  # TODO: What if there are multiple matrices with the same operator norm?
  idx <- which(varcov_opnorm == stats::median(varcov_opnorm))
  variance <- varcov_tilde[[idx]]

  data <- res[[1]]$data

  result <- format_results(data$treat_levels, estimate, variance)

  mods <- lapply(res, function(x) x$mod)
  mu_as <- lapply(res, function(x) x$mu_a)

  return(
    structure(
      class="SLModelResult",
      list(result=result, varcov=variance,
           settings=res[[1]]$settings,
           data=res[[1]]$data, mod=mods, mu_a=mu_as,
           original_df=res[[1]]$original_df)
    )
  )
}

library(SuperLearner)

n <- 1000
set.seed(10)
DATA2 <- data.frame(A=rbinom(n, size=1, prob=0.5),
                    y=rbinom(n, size=1, prob=0.2),
                    x1=rnorm(n),
                    x2=rnorm(n),
                    x3=as.factor(rbinom(n, size=1, prob=0.5)),
                    z1=rbinom(n, size=1, prob=0.5),
                    z2=rbinom(n, size=1, prob=0.5))
DATA2[, "y"] <- NA
As <- DATA2$A == 1
DATA2[DATA2$A == 1, "y"] <- rbinom(sum(As), size=1, prob=exp(DATA2[As,]$x1)/(1+exp(DATA2[As,]$x1)))
DATA2[DATA2$A == 0, "y"] <- rbinom(n-sum(As), size=1, prob=exp(1 + 5*DATA2[!As,]$x1 + DATA2[!As,]$x2)/(1+exp(1 + 5*DATA2[!As,]$x1 + DATA2[!As,]$x2)))
DATA2$A <- as.factor(DATA2$A)

test_that("simple super learner with random forest.", {

  sl.mod <- robincar_SL(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    covariate_cols=c("x1"),
    SL_libraries=c("SL.ranger"),
    car_scheme="permuted-block",
    covariate_to_include_strata=TRUE
  )
})

test_that("simple super learner with median adjustment, random forest.", {

  sl.mod <- robincar_SL_median(
    seed=1,
    n_times=3,
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    covariate_cols=c("x1"),
    SL_libraries=c("SL.ranger"),
    car_scheme="permuted-block",
    covariate_to_include_strata=TRUE
  )

})

test_that("super learner with custom learner", {

  skip("This works, but not in the testthat environment.")

  create_rf <- create.Learner("SL.ranger",
                              tune = list(num.trees=c(25, 50),
                                          max.depth=c(0.1, 0.5)))

  sl.mod <- robincar_SL(
    df=DATA2,
    response_col="y",
    treat_col="A",
    car_strata_cols=c("z1"),
    covariate_cols=c("x1"),
    SL_learners=list(create_rf),
    car_scheme="permuted-block",
    covariate_to_include_strata=TRUE
  )

})

#' Covariate adjustment using linear working model
#'
#' Estimate treatment-group-specific response means and (optionally)
#' treatment group contrasts using a linear working model for continuous outcomes.
#'
#' * Adjustment method "ANOVA" fits a linear model with formula `Y ~ A` where
#' `A` is the treatment group indicator and `Y` is the response.
#' * "ANCOVA" fits a linear model with `Y ~ A + X` where `X` are the variables
#' specified in the `covariate_cols` argument.
#' * "ANHECOVA" fits a linear model with `Y ~ A*X`, the main effects and treatment-by-covariate
#' interactions.
#'
#' @param df A data.frame with the required columns
#' @param treat_col Name of column in df with treatment variable
#' @param response_col Name of the column in df with response variable
#' @param car_strata_cols Names of columns in df with car_strata variables
#' @param covariate_cols Names of columns in df with covariate variables. **If you want to include the strata variables as covariates also, add them here.**
#' @param car_scheme Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".
#' @param adj_method Name of linear adjustment method to use. One of: "ANOVA", "ANCOVA", "ANHECOVA".
#' @param contrast_h An optional function to specify a desired contrast
#' @param contrast_dh An optional jacobian function for the contrast (otherwise use numerical derivative)
#' @export
#'
#' @returns See value of \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}, this function is a wrapper
#' using a linear link function.
robincar_linear <- function(df,
                             treat_col, response_col, car_strata_cols=NULL, covariate_cols=NULL,
                             car_scheme="simple", adj_method="ANOVA",
                             contrast_h=NULL, contrast_dh=NULL){

  # Define the formula based on adj_method:
  formula <- .create.formula(
    adj_method=adj_method,
    treat_col=treat_col,
    response_col=response_col,
    covariate_cols=covariate_cols)

  obj <- robincar_glm(
    df=df,
    treat_col=treat_col,
    response_col=response_col,
    car_strata_cols=car_strata_cols,
    car_scheme=car_scheme,
    formula=formula,
    contrast_h=contrast_h,
    contrast_dh=contrast_dh,
    g_family=stats::gaussian,
    g_accuracy=7
  )

  return(obj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-simdata-car.R
\name{car_sr}
\alias{car_sr}
\title{Generate simple randomization treatment assignments}
\usage{
car_sr(n, p_trt)
}
\arguments{
\item{n}{Number of observations}

\item{p_trt}{Proportion allotted to treatment}
}
\value{
A vector of treatment assignments as 0's and 1's based on simple randomization.
}
\description{
Generate simple randomization treatment assignments
}
\examples{
car_sr(10, p_trt=0.4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-mh.R
\name{robincar_mh}
\alias{robincar_mh}
\title{Estimate Mantel-Haenszel Risk Difference}
\usage{
robincar_mh(
  df,
  treat_col,
  response_col,
  strata_cols,
  estimand = "ATE",
  ci_type = "mGR"
)
}
\arguments{
\item{df}{A data.frame with the required columns}

\item{treat_col}{Name of column in df with treatment variable. Must be binary}

\item{response_col}{Name of the column in df with response variable}

\item{strata_cols}{Names of columns in df with strata variables}

\item{estimand}{A character string specifying the estimand. One of "MH" or "ATE" (default). See Details}

\item{ci_type}{A character string specifying the type of confidence interval. One of "GR", "mGR" (default), "Sato"}
}
\description{
This function estimates Mantel-Haenszel risk difference and average treatment effect.
}
\details{
The estimand of interest can be either Mantel-Haenszel risk difference or Average Treatment Effect (ATE). 
The latter is the default option of `estimand`. When `estimand="ATE"`, `ci_type` is limited to the modified Greenland variance estimator (mGR).
 Otherwise, Greenland's variance estimator (GR) and Sato's variance estimator are optional.
}
\examples{
df <- RobinCar:::data_sim
df$y_bin = ifelse(df$y>2.5, 1, 0)
robincar_mh(df = df[df$A!=2,],
            treat_col = "A",
            response_col = "y_bin",
            strata_cols = c("z1", "z2"),
            estimand = "MH",
            ci_type = "mGR")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-covhr.R
\name{robincar_covhr}
\alias{robincar_covhr}
\title{Covariate-adjusted estimators for time to event data}
\usage{
robincar_covhr(
  df,
  treat_col,
  response_col,
  event_col,
  car_strata_cols = NULL,
  covariate_cols = NULL,
  p_trt = 0.5,
  ref_arm = NULL,
  car_scheme = "simple",
  adj_method = "CL",
  interval = c(-10, 10)
)
}
\arguments{
\item{df}{A data.frame with the required columns}

\item{treat_col}{Name of column in df with treatment variable}

\item{response_col}{Name of the column in df with response variable}

\item{event_col}{Name of column in df with event indicator
(0/FALSE=no event, 1/TRUE=event)}

\item{car_strata_cols}{Names of columns in df with car_strata variables}

\item{covariate_cols}{Names of columns in df with covariate variables}

\item{p_trt}{Treatment allocation ratio for the reference arm.}

\item{ref_arm}{Reference arm of the treatment group, defaults to NULL,
which results in using the first element of `unique(data[, treat_col])`.}

\item{car_scheme}{Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".}

\item{adj_method}{Adjustment method (one of "CL", "CSL")}

\item{interval}{Interval for uniroot function}
}
\value{
An object with attribute named "result", which lists:

\item{theta_L}{estimate of the hazard ratio}
\item{se_theta_L}{SE estimate of the hazard ratio}
\item{theta_CL}{estimate of the covariate-adjusted hazard ratio}
\item{se_theta_CL}{SE estimate of the covariate-adjusted hazard ratio}

Other attributes are the settings used, data attributes, and the original data frame supplied by the user.
}
\description{
Estimate a covariate-adjusted hazard ratio (`adj_method="CL"`),
or a covariate-adjusted stratified hazard ratio (`adj_method="CSL"`).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-coxscore.R
\name{robincar_coxscore}
\alias{robincar_coxscore}
\title{Robust cox score adjustment}
\usage{
robincar_coxscore(...)
}
\arguments{
\item{...}{Arguments to robincar_tte, other than `adj_method`}
}
\value{
A result object with the following attributes:

\item{result}{A list: "statistic" is the robust Cox score test statistic which can be used to obtain p-values; "U" and "se" are the numerator and denominator of the test statistic, respectively.}
\item{settings}{The covariate adjustment settings used.}
\item{original_df}{The dataset supplied by the user.}
}
\description{
Robust cox score adjustment
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-calibrate.R
\name{robincar_calibrate}
\alias{robincar_calibrate}
\title{Perform linear or joint calibration}
\usage{
robincar_calibrate(result, joint = FALSE, add_x = NULL)
}
\arguments{
\item{result}{A GLMModelResult}

\item{joint}{If true, then performs joint calibration
with the \eqn{\hat{\bm \mu}(X_i)} and strata \eqn{Z_i}
to achieve universality and efficiency gain
rather than just linear calibration that uses \eqn{\hat{\bm \mu}(X_i)}.}

\item{add_x}{Additional x to use in the calibration. Must have been in
the original dataset that robincar_glm was called on.}
}
\value{
A result object that has the same structure as \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}, with the argument `result` included as "original" in the list.
}
\description{
Uses linear or joint calibration to "calibrate" the estimates from a linear or GLM-type adjustment.
Linear calibration fits a linear model with treatment (and treatment-by-covariate interactions) and with the predicted \eqn{\hat{\bm \mu}(X_i) = (\hat{\mu}_1(X_i), \dots, \hat{\mu}_K(X_i))} as constructed covariates where \eqn{K} is the number of treatment groups;
joint calibration also includes \eqn{Z_i} the strata variables as covariates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.LinModelResult}
\alias{print.LinModelResult}
\title{Print linear model result}
\usage{
\method{print}{LinModelResult}(x, ...)
}
\arguments{
\item{x}{A LinModelResult object}

\item{...}{Additional arguments}
}
\value{
Prints the treatment mean estimates (and variances) based on a linear working model,
along with the settings used. See \link[RobinCar:robincar_linear]{RobinCar::robincar_linear()}.
}
\description{
Print linear model result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.CalibrationResult}
\alias{print.CalibrationResult}
\title{Print calibration result}
\usage{
\method{print}{CalibrationResult}(x, ...)
}
\arguments{
\item{x}{A GLMModel result. If you'd like to calibrate a linear
adjustment, use `robincar_glm` instead of `robincar_linear`.}

\item{...}{Additional arguments}
}
\value{
Prints the treatment mean estimates (and variances) based on a calibration on top of a
GLM working model, along with the settings used. See \link[RobinCar:robincar_calibrate]{RobinCar::robincar_calibrate()}.
}
\description{
Print calibration result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-SL.R
\name{robincar_SL_median}
\alias{robincar_SL_median}
\title{BETA: Covariate adjustment using working models from the super learner libraries
through the AIPW package with cross-fitting, with median adjustment.}
\usage{
robincar_SL_median(
  n_times,
  seed,
  df,
  treat_col,
  response_col,
  car_strata_cols = NULL,
  covariate_cols = NULL,
  car_scheme = "simple",
  covariate_to_include_strata = NULL,
  SL_libraries = c(),
  SL_learners = c(),
  k_split = 2,
  g_accuracy = 7,
  contrast_h = NULL,
  contrast_dh = NULL
)
}
\arguments{
\item{n_times}{Number of times to run the robincar_SL function}

\item{seed}{Seed to set before running the set of functions}

\item{df}{A data.frame with the required columns}

\item{treat_col}{Name of column in df with treatment variable}

\item{response_col}{Name of the column in df with response variable}

\item{car_strata_cols}{Names of columns in df with car_strata variables}

\item{covariate_cols}{Names of columns in df with covariate variables}

\item{car_scheme}{Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".}

\item{covariate_to_include_strata}{Whether to include car_strata variables in covariate adjustment. Defaults to F for ANOVA and ANCOVA; defaults to T for ANHECOVA. User may override by passing in this argument.}

\item{SL_libraries}{Vector of super-learner libraries to use for the covariate adjustment (see \link[SuperLearner:listWrappers]{SuperLearner::listWrappers})}

\item{SL_learners}{Optional list of super-learner "learners" to use for the covariate adjustment (see \link[SuperLearner:create.Learner]{SuperLearner::create.Learner())}}

\item{k_split}{Number of splits to use in cross-fitting}

\item{g_accuracy}{Level of accuracy to check prediction un-biasedness (in digits).}

\item{contrast_h}{An optional function to specify a desired contrast}

\item{contrast_dh}{An optional jacobian function for the contrast (otherwise use numerical derivative)}
}
\value{
See value of \link[RobinCar:robincar_SL]{RobinCar::robincar_SL}.
Attributes `mods` and `mu_as` are lists of `mod` and `mu_a` attributes, respectively, for each replicate of `robincar_SL` used in the median.
}
\description{
Estimate treatment-group-specific response means and (optionally)
treatment group contrasts using a generalized linear working model.
Perform median adjustment to limit randomness induced from cross-fitting.
}
\details{
*WARNING: This function is still under development and has not been extensively tested.*
This function currently only works for two treatment groups.
Before using this function, you must load the SuperLearner library with
`library(SuperLearner)`, otherwise the function call will fail.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.GLMModelResult}
\alias{print.GLMModelResult}
\title{Print glm model result}
\usage{
\method{print}{GLMModelResult}(x, ...)
}
\arguments{
\item{x}{A GLMModelResult object}

\item{...}{Additional arguments}
}
\value{
Prints the treatment mean estimates (and variances) based on a GLM working model,
along with the settings used. See \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}.
}
\description{
Print glm model result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-simdata.R
\name{data_gen}
\alias{data_gen}
\title{Data generation function from JRSS-B paper}
\usage{
data_gen(
  n,
  theta,
  randomization,
  p_trt,
  case = c("case1", "case2", "case3", "case4", "case5")
)
}
\arguments{
\item{n}{total number of subjects to be generated}

\item{theta}{true treatment effect}

\item{randomization}{randomization method in c("SR","CABC","permuted_block","minimization","urn")}

\item{p_trt}{proportion of treatment arm}

\item{case}{simulation case in the paper}
}
\value{
A data frame with the following columns:

\item{t}{event time}
\item{delta}{event indicator}
\item{I1}{assignment to treatment group 1}
\item{I0}{assignment to treatment group 0}
\item{model_z1, model_z2}{covariates}
\item{car_strata1, ...}{strata variables}
}
\description{
Data generation function from JRSS-B paper
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-linear.R
\name{robincar_linear}
\alias{robincar_linear}
\title{Covariate adjustment using linear working model}
\usage{
robincar_linear(
  df,
  treat_col,
  response_col,
  car_strata_cols = NULL,
  covariate_cols = NULL,
  car_scheme = "simple",
  adj_method = "ANOVA",
  contrast_h = NULL,
  contrast_dh = NULL
)
}
\arguments{
\item{df}{A data.frame with the required columns}

\item{treat_col}{Name of column in df with treatment variable}

\item{response_col}{Name of the column in df with response variable}

\item{car_strata_cols}{Names of columns in df with car_strata variables}

\item{covariate_cols}{Names of columns in df with covariate variables. **If you want to include the strata variables as covariates also, add them here.**}

\item{car_scheme}{Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".}

\item{adj_method}{Name of linear adjustment method to use. One of: "ANOVA", "ANCOVA", "ANHECOVA".}

\item{contrast_h}{An optional function to specify a desired contrast}

\item{contrast_dh}{An optional jacobian function for the contrast (otherwise use numerical derivative)}
}
\value{
See value of \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}, this function is a wrapper
using a linear link function.
}
\description{
Estimate treatment-group-specific response means and (optionally)
treatment group contrasts using a linear working model for continuous outcomes.
}
\details{
* Adjustment method "ANOVA" fits a linear model with formula `Y ~ A` where
`A` is the treatment group indicator and `Y` is the response.
* "ANCOVA" fits a linear model with `Y ~ A + X` where `X` are the variables
specified in the `covariate_cols` argument.
* "ANHECOVA" fits a linear model with `Y ~ A*X`, the main effects and treatment-by-covariate
interactions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-SL.R
\name{robincar_SL}
\alias{robincar_SL}
\title{BETA: Covariate adjustment using working models from the super learner libraries
through the AIPW package with cross-fitting.}
\usage{
robincar_SL(
  df,
  treat_col,
  response_col,
  car_strata_cols = NULL,
  covariate_cols = NULL,
  car_scheme = "simple",
  covariate_to_include_strata = NULL,
  SL_libraries = c(),
  SL_learners = c(),
  k_split = 2,
  g_accuracy = 7,
  contrast_h = NULL,
  contrast_dh = NULL,
  variance_type = 1
)
}
\arguments{
\item{df}{A data.frame with the required columns}

\item{treat_col}{Name of column in df with treatment variable}

\item{response_col}{Name of the column in df with response variable}

\item{car_strata_cols}{Names of columns in df with car_strata variables}

\item{covariate_cols}{Names of columns in df with covariate variables}

\item{car_scheme}{Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".}

\item{covariate_to_include_strata}{Whether to include car_strata variables in covariate adjustment. Defaults to F for ANOVA and ANCOVA; defaults to T for ANHECOVA. User may override by passing in this argument.}

\item{SL_libraries}{Vector of super-learner libraries to use for the covariate adjustment (see \link[SuperLearner:listWrappers]{SuperLearner::listWrappers})}

\item{SL_learners}{Optional list of super-learner "learners" to use for the covariate adjustment (see \link[SuperLearner:create.Learner]{SuperLearner::create.Learner())}}

\item{k_split}{Number of splits to use in cross-fitting}

\item{g_accuracy}{Level of accuracy to check prediction un-biasedness (in digits).}

\item{contrast_h}{An optional function to specify a desired contrast}

\item{contrast_dh}{An optional jacobian function for the contrast (otherwise use numerical derivative)}

\item{variance_type}{The type of variance estimator to use, type 1, 2, or 3. All three are asymptotically equivalent. See details of \link[RobinCar:robincar_glm]{RobinCar::robincar_glm} for more.}
}
\value{
See value of \link[RobinCar:robincar_glm]{RobinCar::robincar_glm}, but the working model for \eqn{\hat{\mu}(X_i)} is based on the \link[AIPW:AIPW]{AIPW::AIPW} package that uses specified SuperLearner libraries and cross-fitting.
Also, `mod` attribute is an object of class \link[AIPW:AIPW]{AIPW::AIPW}.
}
\description{
Estimate treatment-group-specific response means and (optionally)
treatment group contrasts using a generalized linear working model.
}
\details{
*WARNING: This function is still under development and has not been extensively tested.*
This function currently only works for two treatment groups.
Before using this function, you must load the \link[SuperLearner]{SuperLearner} library with
`library(SuperLearner)`, otherwise the function call will fail.
}
\examples{

library(SuperLearner)
library(ranger)
n <- 1000
set.seed(10)
DATA2 <- data.frame(A=rbinom(n, size=1, prob=0.5),
                    y=rbinom(n, size=1, prob=0.2),
                    x1=rnorm(n),
                    x2=rnorm(n),
                    x3=as.factor(rbinom(n, size=1, prob=0.5)),
                    z1=rbinom(n, size=1, prob=0.5),
                    z2=rbinom(n, size=1, prob=0.5))
DATA2[, "y"] <- NA
As <- DATA2$A == 1
DATA2[DATA2$A == 1, "y"] <- rbinom(
  sum(As),
  size=1,
  prob=exp(DATA2[As,]$x1)/(1+exp(DATA2[As,]$x1)))
DATA2[DATA2$A == 0, "y"] <- rbinom(
  n-sum(As),
  size=1,
  prob=exp(1 +
    5*DATA2[!As,]$x1 + DATA2[!As,]$x2)/
    (1+exp(1 + 5*DATA2[!As,]$x1 + DATA2[!As,]$x2)))
DATA2$A <- as.factor(DATA2$A)

sl.mod <- robincar_SL(
  df=DATA2,
  response_col="y",
  treat_col="A",
  car_strata_cols=c("z1"),
  covariate_cols=c("x1"),
  SL_libraries=c("SL.ranger"),
  car_scheme="permuted-block",
  covariate_to_include_strata=TRUE
)

sl.mod$result

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-logrank.R
\name{robincar_logrank}
\alias{robincar_logrank}
\title{Robust (potentially stratified) logrank adjustment}
\usage{
robincar_logrank(adj_method, ...)
}
\arguments{
\item{adj_method}{Adjustment method, one of "CL", "CSL"}

\item{...}{Additional arguments to `robincar_tte`}
}
\value{
A result object with the following attributes:

\item{result}{A list: "statistic" is the adjusted logrank test statistic which can be used to obtain p-values; "U" and "se" are the numerator and denominator of the test statistic, respectively; if the input parameter "return_influence" is set to TRUE, the list will contain an element called "inf_func", which is a data.frame with columns "id" and "inf_func", which is the influence function for each subject for the covariate-adjusted log-rank test (stratified or unstratified). This influence function can be used, e.g., to calculate the correlation between interim and final test statistics in a group sequential design.}
\item{settings}{The covariate adjustment settings used.}
\item{original_df}{The dataset supplied by the user.}
}
\description{
Perform a robust covariate-adjusted logrank test ("CL") that can be stratified ("CSL") if desired.
}
\details{
Note: Since RobinCar version 0.4.0, the variance of the test statistic has changed
to better accommodate tied event times.

\eqn{\hat{\sigma}_{\rm CL}^2} and \eqn{\hat{\sigma}_{\rm CSL}^2} for the
covariate-adjusted stratified log-rank test are given by Ye, Shao, and Yi (2024) after
equation (8) on page 700, with \eqn{\hat{\sigma}_{\rm SL}^2} replaced
by the following estimator, which is the standard denominator of the logrank test:

\deqn{\frac1n \sum_{j} \sum_{i} v_j(t_i)}
\deqn{v_j(t_i) = \frac{Y_{0,j}(t)Y_{1,j}(t)d_j(t)\left[Y_j(t) - d_j(t)\right]}{Y_j(t)^2\left[Y_j(t) - 1 \right]}}
where \eqn{t_i} are strata-specific unique failure times, \eqn{d_j(t)} is the number of events at time \eqn{t} in strata \eqn{j},
\eqn{Y_j(t)} is the number at risk within strata \eqn{j} at time \eqn{t},
and \eqn{Y_{a,j}(t)} is the number at risk within strata \eqn{j} and treatment \eqn{a} at time \eqn{t}.

Please see Ye, Shao, and Yi (2024)'s "Covariate-adjusted log-rank test: guaranteed efficiency
gain and universal applicability" in \emph{Biometrika} for more details about \eqn{\hat{\sigma}_{\rm CSL}^2}.
}
\examples{
library(magrittr)
library(dplyr)
library(forcats)
set.seed(0)
n=100
data.simu0=data_gen(n=n,
                    theta=0,
                    randomization="permuted_block",
                    p_trt=0.5,
                    case="case2") \%>\% mutate(strata1=sample(letters[1:3],n,replace=TRUE),
                                             strata2=sample(LETTERS[4:5],n,replace=TRUE))

out <- robincar_logrank(df=data.simu0,
                        treat_col="I1",
                        p_trt=0.5,
                        ref_arm=0,
                        response_col="t",
                        event_col="delta",
                        covariate_cols=c("model_z1", "model_z2"),
                        car_scheme="simple",
                        adj_method=c("CL"))

set.seed(0)
n=100
data.simu0=data_gen(n=n,
                    theta=0,
                    randomization="permuted_block",
                    p_trt=0.5,
                    case="case1")

data.simu <- data.simu0 \%>\%
  tidyr::pivot_longer(cols=starts_with("car_strata"),
                      names_prefix="car_strata",
                      names_to="strt") \%>\%
  filter(value==1) \%>\% select(-value) \%>\%
  mutate(strt=forcats::as_factor(strt)) \%>\%
  select(t,strt) \%>\%
  left_join(data.simu0, .)

out1 <- robincar_logrank(df=data.simu,
                         treat_col="I1",
                         p_trt=0.5,
                         ref_arm=0,
                         response_col="t",
                         event_col="delta",
                         car_strata_cols="strt",
                         covariate_cols=NULL,
                         car_scheme=c("permuted-block"),
                         adj_method=c("CSL")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-contrast.R
\name{robincar_contrast}
\alias{robincar_contrast}
\title{Estimate a treatment contrast}
\usage{
robincar_contrast(result, contrast_h, contrast_dh = NULL)
}
\arguments{
\item{result}{A LinModelResult or GLMModelResult}

\item{contrast_h}{An optional function to specify a desired contrast}

\item{contrast_dh}{An optional jacobian function for the contrast}
}
\value{
A contrast object which has the following attributes:

 \item{result}{A \link[dplyr:tibble]{dplyr::tibble()} with the label of the treatment contrast (e.g., 1 vs. 0), the estimate of the treatment contrast, estimated SE, and p-value based on a z-test with estimate and SE.}
 \item{varcov}{The variance-covariance matrix for the treatment contrast estimates.}
 \item{settings}{List of model settings used for the contrast.}
}
\description{
Estimate a treatment contrast using the result of \link[RobinCar:robincar_linear]{RobinCar::robincar_linear()}, \link[RobinCar:robincar_glm]{RobinCar::robincar_glm()}, or \link[RobinCar:robincar_SL]{RobinCar::robincar_SL()} using
the delta method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.ContrastResult}
\alias{print.ContrastResult}
\title{Print contrast result}
\usage{
\method{print}{ContrastResult}(x, ...)
}
\arguments{
\item{x}{A ContrastResult object}

\item{...}{Additional arguments}
}
\value{
Prints estimates (and variances) of treatment contrasts based on a linear or GLM working model,
along with the settings used. See \link[RobinCar:robincar_contrast]{RobinCar::robincar_contrast()}
}
\description{
Print contrast result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-simdata-car.R
\name{car_ps}
\alias{car_ps}
\title{Generate Pocock-Simon minimization treatment assignments}
\usage{
car_ps(z, treat, ratio, imb_measure, p_bc = 0.8)
}
\arguments{
\item{z}{The car_strata design matrix}

\item{treat}{A vector of length k (the number of treatment arms), which labels the treatment arms being compared.}

\item{ratio}{A vector of length k (the number of treatment arms), which indicates the allocation ratio, e.g., c(1,1,1) for equal allocation with three treatment arms.}

\item{imb_measure}{What measure of imbalance should be minimzed during randomization -- either "Range" or "SD"}

\item{p_bc}{The biased probability, i.e., the probability of assigning each patient to the arm that minimizes the imbalance. Default is 0.8}
}
\value{
\describe{
\item{res}{treatment assignment vector}
}

A vector of treatment assignments with labels from the `treat` argument, based on Pocock-Simon's minimization.
}
\description{
Generate Pocock-Simon minimization treatment assignments
}
\examples{

# Create car_strata variables
library(fastDummies)
library(dplyr)

x <- runif(100)
z <- cut(x, breaks=c(0, 0.25, 0.5, 0.75, 1.0))
z <- dummy_cols(z)
A <- car_ps(
  z=z[, 2:5],
  treat=c(0, 1, 2),
  ratio=c(1, 1, 1),
  imb_measure="Range"
)

}
\author{
Ting Ye Yanyao Yi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-glm.R
\name{robincar_glm}
\alias{robincar_glm}
\title{Covariate adjustment using generalized linear working model}
\usage{
robincar_glm(
  df,
  treat_col,
  response_col,
  formula = NULL,
  car_strata_cols = NULL,
  car_scheme = "simple",
  g_family = stats::gaussian,
  g_accuracy = 7,
  contrast_h = NULL,
  contrast_dh = NULL,
  variance_type = 1
)
}
\arguments{
\item{df}{A data.frame with the required columns}

\item{treat_col}{Name of column in df with treatment variable}

\item{response_col}{Name of the column in df with response variable}

\item{formula}{The formula to use for adjustment specified using as.formula("..."). This overrides car_strata_cols and covariate_cols.}

\item{car_strata_cols}{Names of columns in df with car_strata variables}

\item{car_scheme}{Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".}

\item{g_family}{Family that would be supplied to glm(...), e.g., binomial. If no link specified, will use default link, like behavior in glm.
If you wish to use a negative binomial working model with an unknown dispersion parameter, then use `g_family="nb"`.}

\item{g_accuracy}{Level of accuracy to check prediction un-biasedness.}

\item{contrast_h}{An optional function to specify a desired contrast}

\item{contrast_dh}{An optional jacobian function for the contrast (otherwise use numerical derivative)}

\item{variance_type}{The type of variance estimator to use, type 1, 2, or 3. All three are asymptotically equivalent. See details for more.}
}
\value{
If `contrast_h` argument is used, outputs a `main` and a `contrast` object. The `main` object has the following structure:

 \item{result}{A \link[dplyr:tibble]{dplyr::tibble()} with the treatment label, treatment mean estimate using AIPW, estimated SE, and p-value based on a z-test with estimate and SE.}
 \item{varcov}{The variance-covariance matrix for the treatment mean estimates.}
 \item{settings}{List of model settings used in covariate adjustment.}
 \item{original_df}{The original dataset provided by the user.}
 \item{mod}{The fit from the \link[stats:glm]{glm()} working model used for covariate adjustment.}
 \item{mu_a}{Predicted potential outcomes for each treatment category (columns) and individual (rows). These are the \eqn{\hat{\mu}_a}}.
 \item{g.estimate}{The G-computation estimate based only on \eqn{\frac{1}{n} \sum_{i=1}^{n} \hat{\mu}_a(X_i)}. This is equivalent to the AIPW estimate when a canonical link function is used.}
 \item{data}{Attributes about the dataset.}

 The `contrast` object has a structure that is documented in \link[RobinCar:robincar_contrast]{RobinCar::robincar_contrast()}.
}
\description{
Estimate treatment-group-specific response means and (optionally)
treatment group contrasts using a generalized linear working model.
}
\details{
The output is the AIPW estimator given by (for each treatment group \eqn{a}):

\deqn{\frac{1}{n} \sum_{i=1}^{n} \hat{\mu}_a(X_i) + \frac{1}{n_a} \sum_{i:A_i=a} \{Y_i - \hat{\mu}(X_i)\}}

where \eqn{Y_i} is the outcome, \eqn{A_i} is the treatment assignment, \eqn{X_i} are the covariates, \eqn{n_a = \sum_{i=1}^{n} A_i=a},
and \eqn{\hat{\mu}_a} is the estimated conditional mean function based on the GLM working model.
This working model has treatment \eqn{a}-specific coefficients if `adj_method` is "heterogeneous". Otherwise, they are shared across the treatment arms.
Alternatively, if `formula` is used, the working model can be specified according to the user.

Importantly, the estimated variance accounts for misspecification of the working model, and for covariate-adaptive randomization.
The variance estimator is given by
\deqn{\hat{V} = \hat{V}_{\rm SR} - \hat{V}_{\Omega}}
where \eqn{\hat{V}_{\rm SR}} is the contribution to the variance under simple randomization, and \eqn{\hat{V}_{\Omega}} is a term
that only appears when a covariate-adaptive randomization scheme is used.
The \eqn{\hat{V}_{\Omega}} is the second line of \eqn{\hat{V}} in \insertCite{bannickGeneralFormCovariate2025}{RobinCar}.

There are three different estimators available for \eqn{\hat{V}_{\rm SR}}, which the user
can choose with the argument \code{variance_type}. We describe these here.

The three variance types are given as follows:

\itemize{
 \item{
   Type 1 (default):
   \deqn{\mathrm{diag}\left[\hat{\pi}_a^{-1} (\mathrm{Var}_a(Y_i) - 2\hat{Q}_{a,a} + \hat{\Sigma}_{a,a} ), a = 1, \dots, K \right] + \hat{Q} + \hat{Q}^T - \hat{\Sigma}}
 }
 \item{
   Type 2:
   \deqn{\mathrm{diag}\left[\hat{\pi}_a^{-1} (\mathrm{Var}_a(Y_i - \hat{\mu}_a(X_i)) - 2\hat{Q}_{a,a} + \hat{\Sigma}_{a,a} ), a = 1, \dots, K \right] + \hat{Q} + \hat{Q}^T - \hat{\Sigma}}
 }
 \item{
   Type 3:
   \deqn{\mathrm{diag}\left[\hat{\pi}_a^{-1} E_a([Y_i - \hat{\mu}_a(X_i)]^2), a = 1, \dots, K \right] + \hat{A}}
 }
}

where \eqn{\hat{\pi}_a} is the treatment proportion for group a,
\eqn{\hat{Q}_{a,b} = \mathrm{Cov}_a(Y_i, \hat{\mu}_b(X_i))},
\eqn{\hat{\Sigma}_{a,b} = \mathrm{Cov}(\hat{\mu}_a(X_i), \hat{\mu}_b(X_i))},
and the matrix \eqn{\hat{A}} has diagonal entries for \eqn{(a, a)} given by
\deqn{2\mathrm{Cov}_a(Y_i - \hat{\mu}_a(X_i), \hat{\mu}_a(X_i)) + \mathrm{Var}_a(\hat{\mu}_a(X_i))}
and off-diagonal entries for \eqn{(a, b)} given by
\deqn{\mathrm{Cov}_a(Y_i, \hat{\mu}_b(X_i)) + \mathrm{Cov}_b(Y_i, \hat{\mu}_a(X_i)) - (1/2) \left[\mathrm{Cov}_a(\hat{\mu}_a(X_i), \hat{\mu}_b(X_i)) + \mathrm{Cov}_b(\hat{\mu}_a(X_i), \hat{\mu}_b(X_i)) \right].}
We use \eqn{E_a}, \eqn{\mathrm{Var}_a}, and \eqn{\mathrm{Cov}_a} to refer to the empirical expectation, variance, and
 covariance among observations in group a only, and \eqn{\mathrm{Cov}} is the covariance within
the entire sample.

Please see the Supplemental Material Sect. H of \insertCite{bannickGeneralFormCovariate2025}{RobinCar} for a discussion
of the merits of each type of variance estimator. Briefly, we recommend
variance types 1 generally, and variance type 3 if it is anticipated
that the distribution of \eqn{X} varies substantially over treatment groups.
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robincar-tte.R
\name{robincar_tte}
\alias{robincar_tte}
\title{Covariate adjustment for time to event data}
\usage{
robincar_tte(
  df,
  treat_col,
  response_col,
  event_col,
  adj_method,
  car_strata_cols = NULL,
  covariate_cols = NULL,
  p_trt = 0.5,
  ref_arm = NULL,
  sparse_remove = TRUE,
  car_scheme = "simple",
  return_influence = FALSE,
  id_col = NULL
)
}
\arguments{
\item{df}{A data.frame with the required columns}

\item{treat_col}{Name of column in df with treatment variable}

\item{response_col}{Name of the column in df with response variable}

\item{event_col}{Name of column in df with event indicator
(0/FALSE=no event, 1/TRUE=event)}

\item{adj_method}{Adjustment method (one of "CL", "CSL", or "coxscore")}

\item{car_strata_cols}{Names of columns in df with car_strata variables}

\item{covariate_cols}{Names of columns in df with covariate variables}

\item{p_trt}{Treatment allocation ratio for the treatment arm.}

\item{ref_arm}{Reference arm of the treatment group, defaults to NULL,
which results in using the first element of `unique(data[, treat_col])`.}

\item{sparse_remove}{Remove sparse car_strata from calculation}

\item{car_scheme}{Name of the type of covariate-adaptive randomization scheme. One of: "simple", "pocock-simon", "biased-coin", "permuted-block".}

\item{return_influence}{Return the influence function for the covariate-adjusted log-rank test for each individual}

\item{id_col}{Name of the column in df indicating individual ids}
}
\value{
For adjustment method "CL" or "CSL", see value of \link[RobinCar:robincar_linear]{RobinCar::robincar_logrank()}; for adjustment method "coxscore" see value of \link[RobinCar:robincar_coxscore]{RobinCar::robincar_coxscore()}.
}
\description{
Perform a covariate-adjusted logrank test (`adj_method="CL"`),
covariate-adjusted stratified logrank test (`adj_method="CSL"`),
or a covariate-adjusted robust Cox score test (`adj_method="coxscore"`).
}
\details{
`robincar_coxscore` and `robincar_logrank` are wrapper functions around
`robincar_tte`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-simdata-car.R
\name{car_pb}
\alias{car_pb}
\title{Generate permuted block treatment assignments}
\usage{
car_pb(z, trt_label, trt_alc, blocksize = 4L)
}
\arguments{
\item{z}{The car_strata design matrix, as a data frame with factor variables}

\item{trt_label}{Treatment label}

\item{trt_alc}{Treatment allocation vector}

\item{blocksize}{Permuted block blocksize}
}
\value{
A vector of treatment assignments with labels from the `trt_label` argument, based on stratified permuted block randomization.
}
\description{
Generate permuted block treatment assignments
}
\examples{
# Create car_strata variables
library(fastDummies)
library(dplyr)

x <- runif(100)
z <- cut(x, breaks=c(0, 0.25, 0.5, 0.75, 1.0))
z <- dummy_cols(z) \%>\%
     mutate(across(where(is.numeric), as.factor))

car_pb(z[, 2:5], c(0, 1, 2), trt_alc=c(1/4, 1/2, 1/4), blocksize=4L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.TTEResult}
\alias{print.TTEResult}
\title{Print TTE result}
\usage{
\method{print}{TTEResult}(x, ...)
}
\arguments{
\item{x}{A TTEResult object}

\item{...}{Additional arguments}
}
\value{
Prints results of time-to-event covariate adjusted analyses including covariate-adjusted (stratified) logrank,
robust Cox score, and covariate-adjusted hazard ratio. Prints summary statistics about number of observations and events, possibly by strata,
and the test statistics and/or estimates, and p-values. See \link[RobinCar:robincar_tte]{RobinCar::robincar_tte()} and \link[RobinCar:robincar_covhr]{RobinCar::robincar_covhr()}.
}
\description{
Print TTE result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.MHResult}
\alias{print.MHResult}
\title{Print MH result}
\usage{
\method{print}{MHResult}(x, ...)
}
\arguments{
\item{x}{A ContrastResult object}

\item{...}{Additional arguments}
}
\value{
Prints estimates (and variances) of treatment contrasts based on MH risk difference or ATE
}
\description{
Print MH result
}
